/*
 ****************************************************************************************
 * Program        : ADSOFT.CPP                                                          *
 * Description    : Demo program for analog input function with software triggering     *
 * Boards Supp.   : PCL-818 series/816/1800/812PG/711B, MIC-2718, PCM-3718              *
 *                  PCI-1710/1713, PCL-813B, Demo board,                                *
 *                  ADAM-4011/4011D/4012/4014D/4018/4018M/5018/4017/4013/5017/4016      *
 * APIs used      : DRV_DeviceOpen,DRV_DeviceClose, DRV_AIVoltageIn                     *
 *                  DRV_GetErrorMessage                                                 *
 * Revision       : 1.00                                                                *
 * Date           : 7/8/1999                   Advantech Co., Ltd.                      *
 ****************************************************************************************
 */
#include <windows.h>
#include <windef.h>
#include <stdio.h>
#include <conio.h>

#include "..\..\..\include\driver.h"

/******************************
 * Local function declaration *
 ******************************/
void ErrorHandler(DWORD dwErrCde);
void ErrorStop(long*, DWORD);

void main()
{
    DWORD  dwErrCde;
    ULONG  lDevNum;
    long   lDriverHandle;
    float  fVoltage;
	DEVFEATURES     DevFeatures;    // structure for device features
    DEVCONFIG_AI    DevCfg;         // structure for DEVCONFIG_AI table
    PT_AIConfig        ptAIConfig;        // structure for MAIConfig table
    PT_AIVoltageIn     ptAIVoltageIn;     // structure for MAIVoltageIn table
    PT_AIVoltageInExp  ptAIVoltageInExp;  // structure for MAIVoltageInExp table
    PT_DeviceGetFeatures  ptDevFeatures;    // structure for DeviceGetFeatures
    PT_AIGetConfig  ptAIGetConfig;          // structure for AIGetConfig table
	USHORT      usChan;                    // start channel
	USHORT		usExpFlag;



    //Step 1: Display hardware and software settings for running this example
    printf("Before running this example, please\n");
    printf("use the device installation utility to add the device.\n");

    //Step 2: Input parameters
    printf("\nPlease input parameters:");
    printf("\nDevice Number (check the device installation utility): ");
    scanf("%d", &lDevNum);

    printf("Input Channel: ");
    scanf("%d", &usChan);




    //Step 3: Open device
    dwErrCde = DRV_DeviceOpen(lDevNum, &lDriverHandle);   
    if (dwErrCde != SUCCESS)
    {
        ErrorHandler(dwErrCde);
        DRV_DeviceClose((LONG far *)&lDriverHandle);
        printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1) ;
    }

    //Step 4: Config device
    ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
    ptDevFeatures.size = sizeof(DEVFEATURES);
    if ((dwErrCde = DRV_DeviceGetFeatures(lDriverHandle,
        (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
    {
		ErrorStop(&lDriverHandle, dwErrCde);
		return;
    }

	//get the configuration data of the specifed analog input channel
    ptAIGetConfig.buffer = (LPDEVCONFIG_AI)&DevCfg;
    ptAIGetConfig.size = sizeof(DEVCONFIG_AI);

    if ((dwErrCde = DRV_AIGetConfig(lDriverHandle,
        (LPT_AIGetConfig)&ptAIGetConfig)) != SUCCESS)
    {
		ErrorStop(&lDriverHandle, dwErrCde);
		return;
    }

    // check start and stop channel if expansion board ?
	// configures the gain for the specifed analog input channel
    usExpFlag = 0;

     if(DevCfg.Daughter[usChan&0xf].dwBoardID == 0)
    {
       ptAIConfig.DasChan   = usChan;
       ptAIConfig.DasGain = 0;

       if ((dwErrCde = DRV_AIConfig(lDriverHandle,
            (LPT_AIConfig)&ptAIConfig)) != 0)
       {
			ErrorStop(&lDriverHandle, dwErrCde);
			return;
       }
    }

    
    // Step 5: Read data

     if(DevCfg.Daughter[usChan&0xf].dwBoardID == 0)
        {
           ptAIVoltageIn.chan   = usChan ;
           ptAIVoltageIn.gain = 0;
           ptAIVoltageIn.TrigMode = 0;            // internal trigger
           ptAIVoltageIn.voltage = (FLOAT far *)&fVoltage;

           if ((dwErrCde = DRV_AIVoltageIn(lDriverHandle,
                      (LPT_AIVoltageIn)&ptAIVoltageIn)) != 0)
           {
				ErrorStop(&lDriverHandle, dwErrCde);
				return;
           }

        }
        else
        {

           ptAIVoltageInExp.DasChan = usChan;
           ptAIVoltageInExp.DasGain = 0;
           ptAIVoltageInExp.ExpChan = 0;
           ptAIVoltageInExp.voltage = (FLOAT far *)&fVoltage;

           if ((dwErrCde = DRV_AIVoltageInExp(lDriverHandle,
                      (LPT_AIVoltageInExp)&ptAIVoltageInExp)) != 0)
           {
				ErrorStop(&lDriverHandle, dwErrCde);
				return;
           }

        }
	// Step 6: Display reading data
     if(DevCfg.Daughter[usChan&0xf].dwBoardID == 0)
		printf("Read data = %10.6f\n", fVoltage);
	 else
		printf("Read data with Exp = %10.6f\n", fVoltage);


    // Step 7: Close device
    dwErrCde = DRV_DeviceClose(&lDriverHandle);
	if (dwErrCde != SUCCESS)
    {
        ErrorStop(&lDriverHandle, dwErrCde);
        return;
    }
    printf("\nPress any key to exit....");
    getch();
}//main

/**********************************************************************
 * Function: ErrorHandler
 *           Show the error message for the corresponding error code
 * input:    dwErrCde, IN, Error code
 * return:   none
 **********************************************************************/
void ErrorHandler(DWORD dwErrCde)
{
    char szErrMsg[180];

    DRV_GetErrorMessage(dwErrCde, szErrMsg);
    printf("\nError(%d): %s\n", dwErrCde & 0xffff, szErrMsg);
}//ErrorHandler

/**********************************************************************
 * Function:   ErrorStop
 *             Release all resource and terminate program if error occurs 
 * Paramaters: pDrvHandle, IN/OUT, pointer to Driver handle
 *             dwErrCde, IN, Error code.
 * return:     none             
 **********************************************************************/
void ErrorStop(long *pDrvHandle, DWORD dwErrCde)
{
    //Error message 
    ErrorHandler(dwErrCde);
    printf("Program terminated!\n");
    
    //Close device
    DRV_DeviceClose(pDrvHandle);

    printf("Press any key to exit....");
    getch();
    exit(1);
}//ErrorStop
