/*
 ****************************************************************************************
 * Program        : DIINT.CPP                                                           *
 * Description    : Demo program for digital input function with interrupt event        *
 * Boards Supp.   :                                                                     *
 * APIs used      : DRV_DeviceOpen,DRV_DeviceClose, DRV_GetErrorMessage,                *
 *                  DRV_EnableEvent, DRV_CheckEvent                                     *
 * Revision       : 1.00                                                                *
 * Date           : 7/8/1999                   Advantech Co., Ltd.                      *
 ****************************************************************************************
 */
#include <windows.h>
#include <windef.h>
#include <stdio.h>
#include <conio.h>

#include "..\..\..\..\include\driver.h"

/******************************
 * Local function declaration *
 ******************************/
void ErrorHandler(DWORD dwErrCde);
void ErrorStop(long*, DWORD);

void main()
{
    DWORD  dwErrCde;
	DWORD	dwEventCount = 0;
    ULONG  lDevNum;
    long   lDriverHandle;

    USHORT	usEventType;
	ULONG   ulRetData;
	PT_EnableEvent EventSetting;
	PT_CheckEvent ptCheckEvent;
	PT_FDITransfer ptFDITransfer;

    //Step 1: Display hardware and software settings for running this example
    printf("Before running this example, please\n");
    printf("use the device installation utility to add the device.\n");

    //Step 2: Input parameters
    printf("\nPlease input parameters:");
    printf("\nDevice Number (check the device installation utility): ");
    scanf("%d", &lDevNum);

    //Step 3: Open device
    dwErrCde = DRV_DeviceOpen(lDevNum, &lDriverHandle);   
    if (dwErrCde != SUCCESS)
    {
        ErrorHandler(dwErrCde);
        printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1);
    }

    // Step 4: Enable DI with interrupt by DRV_EnableEvent
    EventSetting.EventType = ADS_EVT_INTERRUPT;   // event type: DI with interrupt 
    EventSetting.Enabled = true;      // event DI with interrupt
    EventSetting.Count = 1;           // each interrupt generates an event
    dwErrCde = DRV_EnableEvent( lDriverHandle, &EventSetting );
	if (dwErrCde != SUCCESS)
	{
		ErrorHandler(dwErrCde);
		printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1);
    }

    // Step 5: Check event by DRV_CheckEvent in while loop, exit at pressing
    //         any key. If DRV_CheckEvent returns successful, increases
    //         dwEventCount
    dwEventCount = 0;        // initial event count = 0
	ptCheckEvent.EventType = &usEventType;
	ptCheckEvent.Milliseconds = 300;
	ptFDITransfer.usEventType = ADS_EVT_INTERRUPT;
	ptFDITransfer.ulRetData = &ulRetData;			

	
    printf("\nWaiting DI interrupt signal.\n");
    printf("Press any key to exit the waiting loop....");
    while (!kbhit())
    {
		dwErrCde = DRV_CheckEvent( lDriverHandle, &ptCheckEvent );

        //Check function calling successful 
		if (dwErrCde == SUCCESS)
        {
            //Check desired event generation
            if (usEventType == ADS_EVT_INTERRUPT )
            {   
                dwEventCount++;
				DRV_FDITransfer(lDriverHandle,&ptFDITransfer);
		        printf("\nInterrupt Count = %lu, Interrupt Group = %1X, Latch Data = %2X;", 
					dwEventCount,(ulRetData>>16)&0x0000ffff,ulRetData&0x0000ffff);
            }
        }
    }
    if (kbhit()) getch();

    // Step 7: Disable event by DRV_EnableEvent
	EventSetting.EventType = ADS_EVT_INTERRUPT;   // event type: DI with interrupt 
    EventSetting.Enabled = false;      // event DI with interrupt
    EventSetting.Count = 1;           // each interrupt generates an event
    dwErrCde = DRV_EnableEvent( lDriverHandle, &EventSetting );
	if (dwErrCde != SUCCESS)
	{
		ErrorHandler(dwErrCde);
		printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1);
	}

    // Step 8: Close device
    dwErrCde = DRV_DeviceClose(&lDriverHandle);
	if (dwErrCde != SUCCESS)
	{
		ErrorHandler(dwErrCde);
		printf("Program terminated!\n");

        printf("Press any key to exit....");
        getch();
        exit(1);
	}

    printf("\nPress any key to exit....");
    getch();

}//main

/**********************************************************************
 * Function: ErrorHandler
 *           Show the error message for the corresponding error code
 * input:    dwErrCde, IN, Error code
 * return:   none
 **********************************************************************/
void ErrorHandler(DWORD dwErrCde)
{
    char szErrMsg[180];

    DRV_GetErrorMessage(dwErrCde, szErrMsg);
    printf("\nError(%d): %s\n", dwErrCde & 0xffff, szErrMsg);
}//ErrorHandler

/**********************************************************************
 * Function:   ErrorStop
 *             Release all resource and terminate program if error occurs 
 * Paramaters: pDrvHandle, IN/OUT, pointer to Driver handle
 *             dwErrCde, IN, Error code.
 * return:     none             
 **********************************************************************/
void ErrorStop(long *pDrvHandle, DWORD dwErrCde)
{
    //Error message 
    ErrorHandler(dwErrCde);
    printf("Program terminated!\n");
    
    //Close device
    DRV_DeviceClose(pDrvHandle);
    exit(1);
}//ErrorStop
