unit FrmMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, Driver, Thread, CheckForm;

type
  TFrmADInt = class(TForm)
    Label1: TLabel;
    DeviceSelect: TButton;
    Label2: TLabel;
    Datalist: TListBox;
    cmbScanChl: TComboBox;
    GroupBox1: TGroupBox;
    Inprange: TComboBox;
    GroupBox2: TGroupBox;
    Internal: TRadioButton;
    External: TRadioButton;
    GroupBox3: TGroupBox;
    Pacer: TEdit;
    Conversion: TEdit;
    GroupBox4: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Cyclic: TCheckBox;
    Evtenable: TCheckBox;
    Datafloat: TCheckBox;
    GroupBox5: TGroupBox;
    FifoEnable: TCheckBox;
    txtFifoSize: TEdit;
    GroupBox6: TGroupBox;
    cmdRun: TButton;
    cmdExit: TButton;
    Label6: TLabel;
    cmdStop: TButton;
    procedure DeviceSelectClick(Sender: TObject);
    procedure cmbScanChlChange(Sender: TObject);
    procedure InprangeChange(Sender: TObject);
    procedure InternalClick(Sender: TObject);
    procedure ExternalClick(Sender: TObject);
    procedure CyclicClick(Sender: TObject);
    procedure EvtenableClick(Sender: TObject);
    procedure DatafloatClick(Sender: TObject);
    procedure FifoEnableClick(Sender: TObject);
    procedure cmdRunClick(Sender: TObject);
    procedure cmdStopClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmdExitClick(Sender: TObject);
    procedure CheckEvent();
    procedure Terminate();
  private
    { Private declarations }
  public
    { Public declarations }
  end;

const MaxEntries = 255;

var
  FrmADInt: TFrmADInt;
  DeviceHandle            : Longint;
  ptDevGetFeatures        : PT_DeviceGetFeatures;
  lpDevFeatures           : DEVFEATURES;
  devicelist              : array [0..MaxEntries] of PT_DEVLIST;
  SubDevicelist           : array [0..MaxEntries] of PT_DEVLIST;
  ErrCde                  : Longint;
  lpAIConfig              : PT_AIConfig;
  ptAIGetConfig           : PT_AIGetConfig;
  lpDEVCONFIG_AI          : DEVCONFIG_AI;
  AiCtrMode               : Integer;
  wGainCode               : array [0..15] of Integer;
  iGainIndex              : array [0..15] of Integer;
  hbuf                    : HGLOBAL;
  hUserbuf                : HGLOBAL;     { for voltage data }
  ptFAIIntStart           : PT_FAIIntStart;      { FAIIntStart table }
  ptFAITransfer           : PT_FAITransfer;      { FAITransfer table }
  ptEnableEvent           : PT_EnableEvent;      { Enable event }
  ptCheckEvent            : PT_CheckEvent;      // Check event
  szbuffer                : string[80];
  { used for fixed memory for the installed devices }
  bRun                    : Boolean;             { flag for running }
  gwGain                  : array [0..15] of Integer;
  dwDeviceNum             : Longint;
  gdwPacerRate            : Longint;             { pacer rate }
  gulConvNum              : Integer;             { conversion number }
  gwStartChl              : Integer;             { scan channels }
  gbCyclicCount           : Boolean;             { number of cyclic count }
  gbEvtFlag               : Boolean;             { event enable(0) }
  gbDataType              : Boolean;             { display type : FLOAT(1) }
  gbFifoEnable            : Boolean;             { Fifo }
  gwFifoSize              : Integer;             { Fifo Size }
  gwGainList              : Integer;             { gain list flag }
  gwGainCode              : Integer;             { gaincode }
  gwExtTrig               : Integer;             { external or internal trigger }
  gwCyclicMode            : Integer;             { cyclic or non-cyclic mode }
  gwBufferMode            : Integer;             { buffer: single or double }
  gwIntrCount             : Integer;
  gwActiveBuf             : Integer;             { return by FAICheck }
  gwOverrun               : Integer;             { return by FAICheck, FAITransfer }
  gwStopped               : Integer;             { return by FAICheck }
  gulRetrieved            : Longint;             { return by FAICheck }
  gwHalfReady             : Integer;             { return by FAICheck }
  ulIntCounter           : Longint = 0;
  ulBfChgCounter         : Longint = 0;
  ulOverrunCounter       : Longint = 0;
  chkThread               : CheckThread;
  bThreadloop             : boolean = true;

implementation

{$R *.dfm}

Function ChkErr(lErrCde: Longint): Boolean;
var
  szErrMsg   : string[100];
  pszErrMsg  : PChar;
begin
  If (lErrCde <> 0) Then
  begin
    pszErrMsg := @szErrMsg;
    DRV_GetErrorMessage(lErrCde, pszErrMsg);
    Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
    ChkErr := True;
  end
  Else
    ChkErr := False;
end;

procedure TFrmADInt.DeviceSelectClick(Sender: TObject);
var
  i       : Integer;
  tempNum : Integer;
  lpDEVCONFIG_AI : DEVCONFIG_AI;
begin
  { Step 1: Select Device }
  ErrCde := DRV_SelectDevice(Handle, False, dwDeviceNum, @szbuffer[0]);
  If (ChkErr(ErrCde)) Then
    Exit;
  DeviceSelect.Caption := szbuffer;

  { Step 2: Open Devicev }
  ErrCde := DRV_DeviceOpen(dwDeviceNum, DeviceHandle);
  If (ChkErr(ErrCde)) Then
    Exit;

  { Step 3:Get device features }
  ptDevGetFeatures.buffer := @lpDevFeatures;
  ErrCde := DRV_DeviceGetFeatures(DeviceHandle, ptDevGetFeatures);
  If (ChkErr(ErrCde)) Then
  begin
    DRV_DeviceClose(DeviceHandle);
    Exit;
  end;  
     
  { Step 4 : Initialize Input Range List Combobox with device features }
  If (lpDevFeatures.usNumGain <> 0) Then
  begin
    Inprange.Enabled := True;
    Inprange.Clear;
    For i := 0 To lpDevFeatures.usNumGain - 1 do
    begin
      Inprange.Items.Add(lpDevFeatures.glGainList[i].szGainStr);
    end;
    Inprange.ItemIndex := 0;
  end
  Else
    Inprange.Enabled := False;
    
  { Step 5 : Init scan channel }
  ptAIGetConfig.buffer := @lpDEVCONFIG_AI;
  ErrCde := DRV_AIGetConfig(DeviceHandle, ptAIGetConfig);
  If (ChkErr(ErrCde)) Then
  begin
    DRV_DeviceClose(DeviceHandle);
    Exit;
  end;
  tempNum := lpDevFeatures.usMaxAISiglChl ;
  For i := 0 To tempNum - 1 do
    cmbScanChl.Items.Add(IntToStr(i));
  cmbScanChl.ItemIndex := 0;

  { Step 6 : Close device }
  DRV_DeviceClose(DeviceHandle);
  cmdRun.Enabled := True;
end;

procedure TFrmADInt.cmbScanChlChange(Sender: TObject);
begin
  gwStartChl := cmbScanChl.ItemIndex;
end;

procedure TFrmADInt.InprangeChange(Sender: TObject);
begin
  gwGainCode := Inprange.ItemIndex;
end;

procedure TFrmADInt.InternalClick(Sender: TObject);
begin
  Pacer.Enabled := True;
  gwExtTrig := 0;
end;

procedure TFrmADInt.ExternalClick(Sender: TObject);
begin
  gwExtTrig := 1;
  Pacer.Enabled := False;
end;

procedure TFrmADInt.CyclicClick(Sender: TObject);
begin
  gbCyclicCount := Cyclic.Checked;
end;

procedure TFrmADInt.EvtenableClick(Sender: TObject);
begin
  gbEvtFlag := Evtenable.Checked;
end;

procedure TFrmADInt.DatafloatClick(Sender: TObject);
begin
  gbDataType := Datafloat.Checked;
end;

procedure TFrmADInt.FifoEnableClick(Sender: TObject);
var
  iFifoSize : Integer;
begin
  gbFifoEnable := FifoEnable.Checked;
  If (gbFifoEnable) Then
  begin
    txtFifoSize.Enabled := True;

    { Step 1: Device open }
    ErrCde := DRV_DeviceOpen(dwDeviceNum, DeviceHandle);
    If (ChkErr(ErrCde)) Then
      Exit;

    { Step 2: Get FIFO size }
    ErrCde := DRV_GetFIFOSize(DeviceHandle, iFifoSize);
    If (ChkErr(ErrCde)) Then
      Exit;

    { Step 3: Close device }
    DRV_DeviceClose(DeviceHandle);

    gwFifoSize := iFifoSize div 2; { divide by 2 for conversion from byte to word }
    txtFifoSize.Text := IntToStr(gwFifoSize);
    Conversion.Text := IntToStr(iFifoSize);
  end
  Else
  begin
    txtFifoSize.Enabled := False;
    txtFifoSize.Text := '';
  end;
end;

procedure TFrmADInt.cmdRunClick(Sender: TObject);
begin
  { check parameter }
  try
    StrToInt(Conversion.Text);
  except
    Application.MessageBox('Please input Numeric Conv.#!', 'Error',
                            MB_OK Or MB_ICONINFORMATION);
    Exit;
  end;
  
  If (gwExtTrig <> 0) Then       { if is internal trigger }
  begin
    try
      StrToInt(Pacer.Text);
    except
      Application.MessageBox('Please input Numeric PacerRate!', 'Error',
                              MB_OK Or MB_ICONINFORMATION);
      Exit;
    end;
  end;

  { get pace rate and conv number }
  gdwPacerRate := StrToInt(Pacer.Text);
  gulConvNum := StrToInt(Conversion.Text);
    Datalist.Clear;
      
  { Step 1: Device open }
  ErrCde := DRV_DeviceOpen(dwDeviceNum, DeviceHandle);
  If (ChkErr(ErrCde)) Then
    Exit;
  
  { Step 2: Get device features }
  ptDevGetFeatures.buffer := @lpDevFeatures;
  ErrCde := DRV_DeviceGetFeatures(DeviceHandle, ptDevGetFeatures);
  If (ChkErr(ErrCde)) Then
  begin
    DRV_DeviceClose(DeviceHandle);
    Exit;
  end;

  { Step 3: Allocate memory for driver }
  hBuf := GlobalAlloc(GPTR, sizeof(Single) * (gulConvNum + 1));
  if (hBuf = 0) then
  begin
    Application.MessageBox('Not enough memory for buffer', 'Error!!', MB_OK);
    DRV_DeviceClose(DeviceHandle);
    Exit;
  end;

  { Step 4: Allocate memory for Voltage data or Raw data }
   GlobalUnlock(hUserbuf);
  GlobalFree(hUserbuf);

  If (gbDataType) Then
  begin
     hUserbuf := GlobalAlloc(GPTR, sizeof(Single) * (gulConvNum + 1));
  end
  Else
  begin
     hUserbuf := GlobalAlloc(GPTR, sizeof(Word) * (gulConvNum + 1));
  end;
   ptFAITransfer.DataBuffer := Pointer(GlobalLock(hUserbuf));

  { prepare some informations to run }

  ptFAIIntStart.buffer := Pointer(GlobalLock(hbuf));

  { Step 5: Enable event feature }
  ptEnableEvent.EventType := ADS_EVT_INTERRUPT Or
                             ADS_EVT_BUFCHANGE Or
                             ADS_EVT_OVERRUN   Or
                             ADS_EVT_TERMINATED;
  ptEnableEvent.Enabled := Smallint(gbEvtFlag);
  If (gbFifoEnable And (gwFifoSize > 0)) Then
     ptEnableEvent.Count := gwFifoSize
  Else
     ptEnableEvent.Count := 1;
  ErrCde := DRV_EnableEvent(DeviceHandle, ptEnableEvent);
  If (ChkErr(ErrCde)) Then
  begin
    GlobalUnlock(hBuf);
    GlobalFree(hBuf);
    GlobalUnlock(hUserbuf);
    GlobalFree(hUserbuf);
    DRV_DeviceClose(DeviceHandle);
    Exit;
  end;

  ptFAIIntStart.TrigSrc := gwExtTrig;
  ptFAIIntStart.SampleRate := gdwPacerRate;
  ptFAIIntStart.chan := gwStartChl;
  ptFAIIntStart.gain := lpDevFeatures.glGainList[gwGainCode].usGainCde;
  ptFAIIntStart.Count := gulConvNum;
  ptFAIIntStart.Cyclic := Smallint(gbCyclicCount);
  If (gbFifoEnable And (gwFifoSize > 0)) Then
    ptFAIIntStart.IntrCount := gwFifoSize
  Else
    ptFAIIntStart.IntrCount := 1;

  ErrCde := DRV_FAIIntStart(DeviceHandle, ptFAIIntStart);
  If (ChkErr(ErrCde)) Then
  begin
    GlobalUnlock(hBuf);
    GlobalFree(hBuf);
    GlobalUnlock(hUserbuf);
    GlobalFree(hUserbuf);
    DRV_DeviceClose(DeviceHandle);
    Exit;
  end;
    frmCheck.btnExit.Caption := 'Terminate';
   ulIntCounter := 0;
   ulBfChgCounter := 0;
   bThreadloop := true;
   chkThread :=CheckThread.Create(false);
   frmCheck.prgStatus.Max := gulConvNum;
   frmCheck.prgStatus.Position :=0;
   frmCheck.tmrCheck.Enabled := true;
   frmCheck.Show();


  { disable controls }
  cmdStop.Enabled := True;
  cmdRun.Enabled := False;
  cmdExit.Enabled := False;
  DeviceSelect.Enabled := False;
  cmbScanChl.Enabled := False;
  Cyclic.Enabled := False;
  Datafloat.Enabled := False;
  Inprange.Enabled := False;
  Internal.Enabled := False;
  External.Enabled := False;
  bRun := True;
  Enabled := False;

 end;

procedure TFrmADInt.cmdStopClick(Sender: TObject);
var
     dwExitCode : DWORD;
begin
   bThreadloop := false;
   chkThread.Terminate();
    Terminate();

   GetExitCodeThread( chkThread.Handle ,dwExitCode) ;
    if(dwExitCode = STILL_ACTIVE ) then
        TerminateThread( chkThread.Handle, dwExitCode);


end;


procedure TFrmADInt.CheckEvent();
var
    usEventType : integer;
begin
    ulIntCounter := 0;
    ulBfChgCounter := 0;


    while( bThreadloop = true )   do
    begin
        ptCheckEvent.EventType := @usEventType;
        if ( gbCyclicCount  ) then
            ptCheckEvent.Milliseconds := INFINITE
        else
            ptCheckEvent.Milliseconds := (1000 * gulConvNum) div gdwPacerRate + 2000;;


        ErrCde := DRV_CheckEvent(DeviceHandle,ptCheckEvent);
        If (ChkErr(ErrCde)) Then
        begin
           cmdStopClick(nil);
           Exit;
        end;

        // Process interrupt event
        if ((usEventType and ADS_EVT_INTERRUPT)<>0 ) then
            ulIntCounter:=ulIntCounter+1;

        // Process buffer change event
        if ((usEventType and ADS_EVT_BUFCHANGE)<>0 )  then
            ulBfChgCounter:=ulBfChgCounter+1;

        // Process overrun event
        if ((usEventType and ADS_EVT_OVERRUN) <>0 ) then
            ulOverrunCounter:=ulOverrunCounter+1;

        frmCheck.txtInt.Text := IntToStr(ulIntCounter);
        frmCheck.txtBuffChange.Text := IntToStr(ulBfChgCounter);

        // Process terminate event
        if ((usEventType and ADS_EVT_TERMINATED) <>0 )  then
        begin
             bThreadloop := false;
            Terminate();
            Exit;
        end;
    end;


end;
procedure TFrmADInt.Terminate();
var
  i : Integer;
  tempStr : AnsiString;
  p        : Pointer;
begin

  { Step 1: Stop A/D conversion for high speed }
  chkThread.Terminate;

  frmCheck.tmrCheck.Enabled := false;
  frmCheck.btnExit.Caption := 'Waiting';
  Enabled := true;
  ErrCde := DRV_FAIStop(DeviceHandle);
  If (ChkErr(ErrCde)) Then
  begin
    DRV_DeviceClose(DeviceHandle);
    Exit;
  end;

  { Step 2: Get real voltage of Buffer from driver memory buffer }
  ptFAITransfer.Overrun := @gwOverrun;
  ptFAITransfer.Count := gulConvNum;
  ptFAITransfer.start := 0;                   { Start from buffer begging }
  ptFAITransfer.ActiveBuf := 0;               { Single buffer }
  ptFAITransfer.DataType := Smallint(gbDataType);
  ErrCde := DRV_FAITransfer(DeviceHandle, ptFAITransfer);
  If (ChkErr(ErrCde)) Then
  begin
    DRV_DeviceClose(DeviceHandle);
    Exit;
  end;

  { Step 3: Display data }

    p := ptFAITransfer.DataBuffer;

  For i := 0 To ptFAITransfer.Count - 1 do
  begin
    If (gbDataType) Then         { voltage data }
    begin
        tempStr := Format('Buff[%d] = %10.6f',[i+gwStartChl,single(p^)]);
       Longint(p) := Longint(p) + SizeOf(Single);
    end
    else
    begin
        tempStr := Format('Buff[%d] = ',[i+gwStartChl]);
        tempStr := tempStr + IntToHex( Word(p^), 3 );
        Longint(p) := Longint(p) + SizeOf(Word);
    end;
    Datalist.Items.Add(tempStr);
  end;

  { close device }
  DRV_DeviceClose(DeviceHandle);
  Sleep(0);
  GlobalUnlock(hBuf);
  GlobalFree(hBuf);
   cmdRun.Enabled := True;
  cmdStop.Enabled := False;
  cmdExit.Enabled := True;
  DeviceSelect.Enabled := True;
  cmbScanChl.Enabled := True;
  Cyclic.Enabled := True;
  Datafloat.Enabled := True;
  Inprange.Enabled := True;
  Internal.Enabled := True;
  External.Enabled := True;
  bRun := False;
  frmCheck.Close;

end;

procedure TFrmADInt.FormCreate(Sender: TObject);
begin
  gwGainCode := 0;
  gdwPacerRate := 1000;               { pacer rate }
  gulConvNum := 1000;                 { conversion number }
  gwStartChl := 0;                    { scan channels }
  gbDataType := True;                 { display type : FLOAT(1) }
  gbEvtFlag := True;                  { event enable(0) }
  gbFifoEnable := False;              { Fifo disable(0) }
  gwGainList := 0;                    { gain list flag }
  gwExtTrig := 0;                     { external or internal trigger }
  gwCyclicMode := 0;                  { cyclic or non-cyclic mode }
  gwBufferMode := 0;                  { buffer: single or double }
  gwIntrCount := 1;
  gbCyclicCount := False;             { number of cyclic count }
  gwActiveBuf := 0;                   { return by FAICheck }
  gwOverrun := 0;                     { return by FAICheck, FAITransfer }
  gwStopped := 0;                     { return by FAICheck }
  gulRetrieved := 0;                  { return by FAICheck }
  gwHalfReady := 0;                   { return by FAICheck }
  gwFifoSize := 0;
end;

procedure TFrmADInt.cmdExitClick(Sender: TObject);
begin
  GlobalUnlock(hUserbuf);
  GlobalFree(hUserbuf);

  Close();
end;

end.



