unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Driver, WaveSet;

type
  PSHORT = ^SHORT;
  TFrmMain = class(TForm)
    Label1: TLabel;
    Waveform: TButton;
    Run: TButton;
    Stop: TButton;
    WaveCountText: TEdit;
    Tmr: TTimer;
    Label2: TLabel;
    GroupBox1: TGroupBox;
    DevSel: TButton;
    GroupBox2: TGroupBox;
    Label3: TLabel;
    Pacer: TEdit;
    GroupBox3: TGroupBox;
    Conv: TEdit;
    GroupBox4: TGroupBox;
    WaveCount: TEdit;
    GroupBox5: TGroupBox;
    IntTrig: TRadioButton;
    ExtTrig: TRadioButton;
    GroupBox6: TGroupBox;
    EvtEnable: TRadioButton;
    EvtDisable: TRadioButton;
    procedure WaveformClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure RunClick(Sender: TObject);
    procedure StopClick(Sender: TObject);
    procedure TmrTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure DevSelClick(Sender: TObject);
    procedure PacerChange(Sender: TObject);
    procedure ConvChange(Sender: TObject);
    procedure WaveCountChange(Sender: TObject);
    procedure IntTrigClick(Sender: TObject);
    procedure ExtTrigClick(Sender: TObject);
    procedure EvtEnableClick(Sender: TObject);
    procedure EvtDisableClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;
  Function DoesErr(lErrCode: Longint): Integer;
  procedure SetRealBuffer(lpBuf: PSingle; num: Longint; lpWave: LPSWAVE);
  procedure SetMultiToOneBuffer(usEnabledChannel: SHORT; count: Integer);
  procedure MyFreeBuffer();

var
  FrmMain: TFrmMain;
  ErrCde: Integer;             { Return error code }
  DriverHandle: Longint = 0;   { driver handle }
  dwDeviceNum: Longint;        { Device number }
  bRunTimer: Boolean = FALSE;
  ptFAOScale: PT_FAOScale;     { FAOScale table }
  lpBuf: array[0..3] of PSHORT = (NIL, NIL, NIL, NIL);
  lpVoltageBuf: array[0..3] of PSingle = (NIL, NIL, NIL, NIL);
  hBuf: array[0..3] of HGLOBAL;
  hVoltageBuf: array [0..3] of HGLOBAL;
  lpCommonBuf: PSHORT;
  hCommonBuf: HGLOBAL;
  bThreadloop: Boolean = FALSE;
  bThreadflag: Boolean = FALSE;
  tempPtr: Pointer;
  szDescript: array[0..99] of char; { Description device }
  gwSelectDevice: SHORT = 0;      { have never selected "Select Device" button }
  gdwPacerRate: DWORD = 10000;    { pacer rate }
  gulConvNum: ULONG = 2048;       { conversion number }
  gwWaveFormCount: ULONG = 30;    { waveform count }
  gwExtTrig: SHORT = 0;           { external or internal trigger }
  gwEvtFlag: SHORT = 1;           { event enable(0) }

implementation
uses GetEvent;

{$R *.dfm}

Function DoesErr(lErrCode: Longint): Integer;
var
  szErrMsg: string[100];
  pszErrMsg: PChar;
begin
  if (lErrCode <> 0) then
  begin
    pszErrMsg := @szErrMsg;
    DRV_GetErrorMessage(lErrCode, pszErrMsg);
    Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
    DoesErr := 1;
  end
  else
    DoesErr := 0;
end;

procedure SetRealBuffer(lpBuf: PSingle; num: Longint; lpWave: LPSWAVE);
var
  i: Integer;
  r: Integer;
  half: Integer;
  slope: Real;
  x: Integer;
  y: Integer;
  lpBuff: PSingle;
begin
  lpBuff := lpBuf;
  case lpWave.wWaveform of
    0:
    begin
      for i := 0 to num - 1 do
      begin
        lpBuff^ := lpWave.fMagnitude * sin(6.2831853 * i / lpWave.wPeriod) + lpWave.fOffset;
        Inc(lpBuff);
      end;
    end;
    1:
    begin
      slope := lpWave.fMagnitude * 4.0 / lpWave.wPeriod;
      for i := 0 to num -1 do
      begin
        r := i mod lpWave.wPeriod;
        half := lpWave.wPeriod div 2;
        if (r <= half) then
          lpBuff^ := slope * r - lpWave.fMagnitude + lpWave.fOffset
        else
          lpBuff^ := slope * (lpWave.wPeriod - r) - lpWave.fMagnitude + lpWave.fOffset;
        Inc(lpBuff);
      end;
    end;
    2:
    begin
      for i := 0 to num - 1 do
      begin
        r := i mod lpWave.wPeriod;
        half := lpWave.wPeriod div 2;
        if (r <= half) then
          lpBuff^ := lpWave.fOffset - lpWave.fMagnitude
        else
          lpBuff^ := lpWave.fOffset + lpWave.fMagnitude;
        Inc(lpBuff);
      end;
    end;
    3:
    begin
      slope := lpWave.fMagnitude * 4.0 / lpWave.wPeriod;
      for i := 0 to num - 1 do
      begin
	if ((i mod 2) <> 0) then
        begin
	  y := i div 2;
          y := y + 1;
          r := y mod lpWave.wPeriod;
          half := lpWave.wPeriod div 2;
          if (r <= half) then
            lpBuff^ := slope * r - lpWave.fMagnitude + lpWave.fOffset
          else
            lpBuff^ := slope * (lpWave.wPeriod - r) - lpWave.fMagnitude + lpWave.fOffset;
        end
	else
        begin
	  x := i div 2;
	  lpBuff^ := lpWave.fMagnitude * sin(6.28318 * x / lpWave.wPeriod) + lpWave.fOffset;
        end;
        Inc(lpBuff);
      end;
    end;
  end;
end;

procedure SetMultiToOneBuffer(usEnabledChannel: SHORT; count: Integer);
var
  i: Integer;
  lpTemp: PSHORT;
  lpTemp0: PSHORT;
  lpTemp1: PSHORT;
  lpTemp2: PSHORT;
  lpTemp3: PSHORT;
begin
  lpTemp := lpCommonBuf;
  lpTemp0 := lpBuf[0];
  lpTemp1 := lpBuf[1];
  lpTemp2 := lpBuf[2];
  lpTemp3 := lpBuf[3];

  for i := 0 to count - 1 do
  begin
    if (usEnabledChannel and $01 <> 0) then
    begin
      lpTemp^ := lpTemp0^ and $0fff;
      Inc(lpTemp);
    end;
    if (usEnabledChannel and $02 <> 0) then
    begin
      lpTemp^ := (lpTemp1^ or ($01 shl 12)) and $3fff;
      Inc(lpTemp);
    end;
    if (usEnabledChannel and $04 <> 0) then
    begin
      lpTemp^ := (lpTemp2^ or ($02 shl 12)) and $3fff;
      Inc(lpTemp);
    end;
    if (usEnabledChannel and $08 <> 0) then
    begin
      lpTemp^ := (lpTemp3^ or ( $03 shl 12)) and $3fff;
      Inc(lpTemp);
    end;
    Inc(lpTemp0);
    Inc(lpTemp1);
    Inc(lpTemp2);
    Inc(lpTemp3);
  end;
end;

procedure MyFreeBuffer();
var
  i: Integer;
begin
  for i := 0 to 3 do
  begin
    if (hBuf[i] <> 0) then
    begin
      GlobalFree(hBuf[i]);
      hBuf[i] := 0;
    end;
    if(hVoltageBuf[i] <> 0) then
    begin
      GlobalFree(hVoltageBuf[i]);
      hVoltageBuf[i] := 0;
    end;
  end;
end;

procedure TFrmMain.WaveformClick(Sender: TObject);
begin
  Application.CreateForm(TFrmWave, FrmWave);
  FrmWave.Show();
end;

procedure TFrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  MyFreeBuffer();
end;

procedure TFrmMain.RunClick(Sender: TObject);
var
  i: Integer;
  ptEnableEvent: PT_EnableEvent;             { Enable event }
  ptFAOWaveFormStart: PT_FAOWaveFormStart;   { FAODMAStart table }
begin
  { Step 1: Device open }
  ErrCde := DRV_DeviceOpen(dwDeviceNum, DriverHandle);
  if (DoesErr(ErrCde) <> 0) then
    exit;

  { Step 2: Allocate buffer and set data to buffer }
  { Process 4 channel }
  usChannelCount := 0;    { reset channel count }
  usEnabledChannel := 0;
  for i := 0 to 3 do
  begin
    { Allocate memory used by driver }
    hBuf[i] := GlobalAlloc(GPTR, sizeof(SHORT) * gulConvNum);
    if (hBuf[i] = 0) then
    begin
      Application.MessageBox('Not enough memory for buffer', 'High Speed', MB_OK);
      DRV_DeviceClose(DriverHandle);
      exit;
    end;

    { Allocate memory for real voltage }
    hVoltageBuf[i] := GlobalAlloc(GPTR, sizeof(Real) * gulConvNum);
    if(hVoltageBuf[i] = 0) then
    begin
      Application.MessageBox('Not enough memory for buffer', 'High Speed', MB_OK);
      DRV_DeviceClose(DriverHandle);
      exit;
    end;

    lpBuf[i] := GlobalLock(hBuf[i]);
    lpVoltageBuf[i] := GlobalLock(hVoltageBuf[i]);

    case i of
      0:
      begin
        if (sWaveCh0.wWaveform <> 4) then { 4 --> No Waveform }
        begin
          usEnabledChannel := usEnabledChannel or $01;
          { set real voltage to hVoltageBuf }
          SetRealBuffer(lpVoltageBuf[0], gulConvNum, @sWaveCh0);
          usChannelCount := usChannelCount + 1;
        end
        else
          continue;
      end;
      1:
      begin
        if (sWaveCh1.wWaveform <> 4) then
        begin
          usEnabledChannel := usEnabledChannel or $02;
          SetRealBuffer(lpVoltageBuf[1], gulConvNum, @sWaveCh1);
          usChannelCount := usChannelCount + 1;
        end
        else
          continue;
      end;
      2:
      begin
        if (sWaveCh2.wWaveform <> 4) then
        begin
          usEnabledChannel := usEnabledChannel or $04;
          SetRealBuffer(lpVoltageBuf[2], gulConvNum, @sWaveCh2);
          usChannelCount := usChannelCount + 1;
        end
        else
          continue;
      end;
      3:
      begin
        if (sWaveCh3.wWaveform <> 4) then
        begin
          usEnabledChannel := usEnabledChannel or $08;
          SetRealBuffer(lpVoltageBuf[3], gulConvNum, @sWaveCh3);
          usChannelCount := usChannelCount + 1;
        end
        else
          continue;
      end;
    end;

    { call FAOScale for transfer voltage to binary data }
    with ptFAOScale do
    begin
      tempPtr := lpVoltageBuf[i];
      VoltArray := tempPtr;
      tempPtr := lpBuf[i];
      BinArray  := tempPtr;
      chan      := i;
      count     := gulConvNum;
    end;
    ErrCde := DRV_FAOScale(DriverHandle, ptFAOScale);
    if (DoesErr(ErrCde) <> 0) then
    begin
      MyFreeBuffer();
      DRV_DeviceClose(DriverHandle);
      exit;
    end;
  end;

  { Allocate memory for common buffer of 4 channel }
  hCommonBuf := GlobalAlloc(GHND, sizeof(SHORT) * gulConvNum * usChannelCount);
  if(hCommonBuf = 0) then
  begin
    Application.MessageBox('Not enough memory for buffer', 'High Speed', MB_OK);
    MyFreeBuffer();
    DRV_DeviceClose(DriverHandle);
    exit;
  end;

  { Lock down buffer }
  lpCommonBuf := GlobalLock(hCommonBuf);
  if (lpCommonBuf = NIL) then
  begin
    Application.MessageBox('Not enough memory for buffer', 'High Speed', MB_OK);
    MyFreeBuffer();
    GlobalFree(hCommonBuf);
    DRV_DeviceClose(DriverHandle);
    exit;
  end;

  { set 4 buffer to 1 common buffer }
  SetMultiToOneBuffer(usEnabledChannel, gulConvNum);

  { Step 3: Enable event feature }
  with ptEnableEvent do
  begin
    EventType := ADS_EVT_INTERRUPT  or
                 ADS_EVT_BUFCHANGE  or
                 ADS_EVT_TERMINATED or
                 ADS_EVT_OVERRUN;
    Enabled   := gwEvtFlag;
    Count     := 1;
  end;
  ErrCde := DRV_EnableEvent(DriverHandle, ptEnableEvent);
  if (DoesErr(ErrCde) <> 0) then
  begin
    MyFreeBuffer();
    DRV_DeviceClose(DriverHandle);
    exit;
  end;

  { Step 4: call FAOWaveFormStart for start action }
  { Default setting - }
  { conversion number = 2048 * 4 }
  { Enabled Channel = 0,1,2,3 }
  { pacer rate = 1000 Hz }
  { output waveform number = 30 }

  with ptFAOWaveFormStart do
  begin
    TrigSrc    := gwExtTrig;		       { triggering }
    SampleRate := gdwPacerRate;		       { pacer rate = 1K }
    Count      := gulConvNum * usChannelCount; { DA conversion number }
    WaveCount  := gwWaveFormCount;	       { Waveform number }
    tempPtr    := PINT(lpCommonBuf);           { analog output data }
    Buffer     := tempPtr;
    EnabledChannel := usEnabledChannel;        { which channel }
  end;
  ErrCde := DRV_FAOWaveFormStart(DriverHandle, ptFAOWaveFormStart);
  if (DoesErr(ErrCde) <> 0) then
  begin
    MyFreeBuffer();
    GlobalUnlock(hCommonBuf);
    GlobalFree(hCommonBuf);
    DRV_DeviceClose(DriverHandle);
    exit;
  end;

  { if event feature is enabled , then create one thread. }
  if (gwEvtFlag <> 0) then
  begin
    TGetEvent.Create(False);
    bThreadloop := TRUE;
  end;

  Run.Enabled := FALSE;
  Stop.Enabled := TRUE;     { Enable stop buttons }

  { if event feature is enabled }
  { Start up timer }
  if (gwEvtFlag <> 0) then
  begin
    Tmr.Enabled := TRUE;
    bRunTimer := TRUE;
  end;
end;

procedure TFrmMain.StopClick(Sender: TObject);
var
bflag: Boolean;
begin
  bflag := TRUE;
  bThreadloop := FALSE;	{ disable thread loop }
  Sleep(500);

  if (gwExtTrig = 0) then
  begin
    while (bflag) do
    begin
      if (bThreadflag = TRUE) then
        Sleep(10)
      else
        bflag := FALSE;
    end;
  end;

  { Stop Timer }
  if (bRunTimer) then
  begin
    Tmr.Enabled := FALSE;
    bRunTimer := FALSE;
  end;

  Stop.Enabled := FALSE;     { Disable stop buttons }
  Run.Enabled := TRUE;

  { Step 1: Stop D/A conversion }
  ErrCde := DRV_FAOStop(DriverHandle);
  if (ErrCde <> 0) then
  begin
    DoesErr(ErrCde);
    MyFreeBuffer();
    GlobalUnlock(hCommonBuf);
    GlobalFree(hCommonBuf);
    DRV_DeviceClose(DriverHandle);
    exit;
  end;

  { Step 2: Free buffer }
  MyFreeBuffer();
  GlobalUnlock(hCommonBuf);
  GlobalFree(hCommonBuf);

  { Step 3: Close driver }
  DRV_DeviceClose(DriverHandle);
end;

procedure TFrmMain.TmrTimer(Sender: TObject);
var
  ptFAOCheck: PT_FAOCheck;
  gwActiveBuf: SHORT;     { return by FAOCheck }
  gwStopped: SHORT;       { return by FAOCheck }
  ulCurrentCount: ULONG;  { return by FAOCheck }
  gwOverrun: SHORT;       { return by FAOCheck }
  gwHalfReady: SHORT;     { return by FAOCheck }
begin
  gwActiveBuf := 0;     { return by FAOCheck }
  gwStopped := 0;       { return by FAOCheck }
  ulCurrentCount := 0;  { return by FAOCheck }
  gwOverrun := 0;       { return by FAOCheck }
  gwHalfReady := 0;     { return by FAOCheck }

  { Get transfer status }
  ptFAOCheck.ActiveBuf    := @gwActiveBuf;
  ptFAOCheck.stopped      := @gwStopped;
  ptFAOCheck.CurrentCount := @ulCurrentCount;
  ptFAOCheck.overrun      := @gwOverrun;
  ptFAOCheck.HalfReady    := @gwHalfReady;

  ErrCde := DRV_FAOCheck(DriverHandle, ptFAOCheck);
  if (ErrCde <> 0) then
  begin
    DoesErr(ErrCde);
    MyFreeBuffer();
    GlobalUnlock(hCommonBuf);
    GlobalFree(hCommonBuf);
    DRV_DeviceClose(DriverHandle);
    exit;
  end;

  { Display Data }
  WaveCountText.Text := IntToStr(ulCurrentCount);

  { if thread finish run stop instruction }
  if (not bThreadloop) then
  begin
    { if A/O process finish --> call DRV_FAOStop }
    if (gwStopped <> 0) then
    begin
      { Step 1: Stop D/A conversion for high speed }
      ErrCde := DRV_FAOStop(DriverHandle);
      if (ErrCde <> 0) then
        DoesErr(ErrCde);

      { Step 2: Free buffer }
      MyFreeBuffer();
      GlobalUnlock(hCommonBuf);
      GlobalFree(hCommonBuf);

      { Step 3: Close driver }
      DRV_DeviceClose(DriverHandle);

      { Stop Timer }
      if (bRunTimer) then
      begin
        Tmr.Enabled := FALSE;
        bRunTimer := FALSE;
      end;

      Run.Enabled := TRUE;
    end;
  end;
end;

procedure TFrmMain.FormCreate(Sender: TObject);
begin
  if (gwSelectDevice = 0) then
  begin
    { Select Device }
    ErrCde := DRV_SelectDevice(Handle, FALSE, dwDeviceNum, @szDescript[0]);
    if (ErrCde <> SUCCESS) then
    begin
      DoesErr(ErrCde);
      exit;
    end;

    gwSelectDevice := 1;
    FrmMain.Run.Enabled := TRUE;
  end;

  { Show device description to Button text }
  DevSel.Caption := szDescript;

  { Initialize Conversion Number }
  Conv.Text := IntToStr(gulConvNum);

  { Initialize Pacer Rate }
  Pacer.Text := IntToStr(gdwPacerRate);

  { Initialize wave form count }
  if (gwWaveFormCount = INFINITE) then
    WaveCount.Text := 'INFINITE'
  else
    WaveCount.Text := IntToStr(gwWaveFormCount);

  { Initialize Triggering }
  if (gwExtTrig <> 0) then
    ExtTrig.Checked := True
  else
    IntTrig.Checked := True;

  { Initialize Message Flag }
  if (gwEvtFlag <> 0) then
    EvtEnable.Checked := True
  else
    EvtDisable.Checked := True;
end;

procedure TFrmMain.DevSelClick(Sender: TObject);
begin
  { Select Device }
  ErrCde := DRV_SelectDevice(Handle, FALSE, dwDeviceNum, @szDescript[0]);
  if (ErrCde <> SUCCESS) then
  begin
    DoesErr(ErrCde);
    exit;
  end;

  { Show device description to Button text }
  DevSel.Caption := szDescript;
end;

procedure TFrmMain.PacerChange(Sender: TObject);
begin
  gdwPacerRate := StrToInt(Pacer.Text);
end;

procedure TFrmMain.ConvChange(Sender: TObject);
begin
  gulConvNum := StrToInt(Conv.Text);
end;

procedure TFrmMain.WaveCountChange(Sender: TObject);
var
  szBuf: string[40];
begin
  szBuf := WaveCount.Text;
  if(szBuf = 'INFINITE') then
    gwWaveFormCount := INFINITE
  else
    gwWaveFormCount := StrToInt(szBuf);
end;

procedure TFrmMain.IntTrigClick(Sender: TObject);
begin
  gwExtTrig := 0;
end;

procedure TFrmMain.ExtTrigClick(Sender: TObject);
begin
  gwExtTrig := 1;
end;

procedure TFrmMain.EvtEnableClick(Sender: TObject);
begin
  gwEvtFlag := 1;
end;

procedure TFrmMain.EvtDisableClick(Sender: TObject);
begin
  gwEvtFlag := 0;
end;

end.

