Unit FormMain;

Interface

Uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, Driver, ComCtrls;

Type
  TfrmMain = class(TForm)
  GroupBox1: TGroupBox;
  labDeviceName: TStaticText;
  btnSelectDevice: TButton;
  cmdStart  : TButton;
  cmdStop   : TButton;
  btnClose: TBitBtn;
  ScanTimer : TTimer;
  GroupBox2 : TGroupBox;
  optVoltageOut  : TRadioButton;
  optCurrentOut0 : TRadioButton;
  optCurrentOut4 : TRadioButton;
  GroupBox3 : TGroupBox;
  chkSyncAOEnable : TCheckBox;
  Label1          : TLabel;
  txtScanTime     : TEdit;
  GroupBox4 : TGroupBox;
  chkSyncEnable0 : TCheckBox;
  chkSyncEnable1 : TCheckBox;
  chkSyncEnable2 : TCheckBox;
  chkSyncEnable3 : TCheckBox;
  chkSyncEnable4 : TCheckBox;
  chkSyncEnable5 : TCheckBox;
  chkSyncEnable6 : TCheckBox;
  chkSyncEnable7 : TCheckBox;
  GroupBox5 : TGroupBox;
  optAsyncEnable0 : TRadioButton;
  optAsyncEnable1 : TRadioButton;
  optAsyncEnable2 : TRadioButton;
  optAsyncEnable3 : TRadioButton;
  optAsyncEnable4 : TRadioButton;
  optAsyncEnable5 : TRadioButton;
  optAsyncEnable6 : TRadioButton;
  optAsyncEnable7 : TRadioButton;
  editOutput: TEdit;

  procedure btnSelectDeviceClick(Sender: TObject);
  procedure FormCreate(Sender: TObject);
  procedure txtScanTimeChange(Sender: TObject);
  procedure chkSyncAOEnableClick(Sender: TObject);
  procedure cmdStartClick(Sender: TObject);
  procedure cmdStopClick(Sender: TObject);
  procedure ScanTimerTimer(Sender: TObject);

  Private
    { Private declarations }
    lErrCde       : Longint;

  Public
    { Public declarations }
    lDeviceNum    :	Longint;
    lDeviceHandle :	Longint;
    szDescription :	array[0..81] of Char;
    ptDevGetFeatures : PT_DeviceGetFeatures;
    ptDevFeatures    : DEVFEATURES;
    ptAOVoltageOut   : PT_AOVoltageOut;
    ptAOCurrentOut   : PT_AOCurrentOut;
    MaxAOChannel : Integer;
    OutputIndex  : Integer;
    OutputValue  : array[0..3] of Single;
    szErrMsg   : string[100];
    pszErrMsg  : PChar;
End;

Var
  frmMain: TfrmMain;
  const AO_RANGE_SETTING : Smallint = $2000;

Implementation

{$R *.DFM}


{*************************************************************
 * Function: Handle the error code. If the input error code > 0,
 *           it means some error apperent.  This function can
 *           show the error message to a message box and stop
 *           this application.
 * Input:    The error code.
 * return:   none
 ************************************************************* }
Function DoesErr(var lErrCode: LongInt): integer;
var
  szErrMsg   : string[100];
  pszErrMsg  : PChar;
begin
  {Check the pressed error code}
  If (lErrCode <> 0) Then
  Begin
    pszErrMsg := @szErrMsg;
    DRV_GetErrorMessage(lErrCode, pszErrMsg);
    Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
    DoesErr := 1;
  End
  Else
    DoesErr := 0;
end;


Procedure TfrmMain.btnSelectDeviceClick(Sender: TObject);
Var
  i : Integer;
Begin
  DRV_SelectDevice(Handle, True, lDeviceNum, szDescription);
  labDeviceName.Caption := szDescription;
  // Open device to get feature
  lErrCde := DRV_DeviceOpen(lDeviceNum, lDeviceHandle);
  If ( DoesErr(lErrCde) = 1 ) Then
    Exit;

  ptDevGetFeatures.buffer := @ptDevFeatures;
  ptDevGetFeatures.size := sizeof(ptDevFeatures);

  lErrCde := DRV_DeviceGetFeatures(lDeviceHandle, ptDevGetFeatures);
  If ( DoesErr(lErrCde) = 1 ) Then
  Begin
    DRV_DeviceClose(lDeviceHandle);
    Exit;
  End;

  MaxAOChannel := ptDevFeatures.usMaxAOChl;

  DRV_DeviceClose(lDeviceHandle);

  If chkSyncAOEnable.Checked Then
  Begin
    optAsyncEnable0.Enabled := False;
    optAsyncEnable1.Enabled := False;
    optAsyncEnable2.Enabled := False;
    optAsyncEnable3.Enabled := False;
    optAsyncEnable4.Enabled := False;
    optAsyncEnable5.Enabled := False;
    optAsyncEnable6.Enabled := False;
    optAsyncEnable7.Enabled := False;
    For i := 0 To 7 do
    Begin
      If i < MaxAOChannel Then
	Case i of
          0 : chkSyncEnable0.Enabled := True;
          1 : chkSyncEnable1.Enabled := True;
          2 : chkSyncEnable2.Enabled := True;
          3 : chkSyncEnable3.Enabled := True;
          4 : chkSyncEnable4.Enabled := True;
          5 : chkSyncEnable5.Enabled := True;
          6 : chkSyncEnable6.Enabled := True;
          7 : chkSyncEnable7.Enabled := True;
        End
      Else
	Case i of
          0 : chkSyncEnable0.Enabled := False;
          1 : chkSyncEnable1.Enabled := False;
          2 : chkSyncEnable2.Enabled := False;
          3 : chkSyncEnable3.Enabled := False;
          4 : chkSyncEnable4.Enabled := False;
          5 : chkSyncEnable5.Enabled := False;
          6 : chkSyncEnable6.Enabled := False;
          7 : chkSyncEnable7.Enabled := False;
        End;
    End;
  End
  Else
  Begin
    chkSyncEnable0.Enabled := False;
    chkSyncEnable1.Enabled := False;
    chkSyncEnable2.Enabled := False;
    chkSyncEnable3.Enabled := False;
    chkSyncEnable4.Enabled := False;
    chkSyncEnable5.Enabled := False;
    chkSyncEnable6.Enabled := False;
    chkSyncEnable7.Enabled := False;
    For i := 0 To 7 do
    Begin
      If i < MaxAOChannel Then
	Case i of
          0 : optAsyncEnable0.Enabled := True;
          1 : optAsyncEnable1.Enabled := True;
          2 : optAsyncEnable2.Enabled := True;
          3 : optAsyncEnable3.Enabled := True;
          4 : optAsyncEnable4.Enabled := True;
          5 : optAsyncEnable5.Enabled := True;
          6 : optAsyncEnable6.Enabled := True;
          7 : optAsyncEnable7.Enabled := True;
        End
      Else
	Case i of
          0 : optAsyncEnable0.Enabled := False;
          1 : optAsyncEnable1.Enabled := False;
          2 : optAsyncEnable2.Enabled := False;
          3 : optAsyncEnable3.Enabled := False;
          4 : optAsyncEnable4.Enabled := False;
          5 : optAsyncEnable5.Enabled := False;
          6 : optAsyncEnable6.Enabled := False;
          7 : optAsyncEnable7.Enabled := False;
        End;
    End;
  End;
End;

Procedure TfrmMain.FormCreate(Sender: TObject);
Begin
  btnSelectDeviceClick(Sender);
End;


Procedure TfrmMain.txtScanTimeChange(Sender: TObject);
Begin
  ScanTimer.Interval := StrToInt(txtScanTime.Text);
End;


procedure TfrmMain.chkSyncAOEnableClick(Sender: TObject);
var
  i : Integer;
begin
  If chkSyncAOEnable.Checked Then
  Begin
    optAsyncEnable0.Enabled := False;
    optAsyncEnable1.Enabled := False;
    optAsyncEnable2.Enabled := False;
    optAsyncEnable3.Enabled := False;
    optAsyncEnable4.Enabled := False;
    optAsyncEnable5.Enabled := False;
    optAsyncEnable6.Enabled := False;
    optAsyncEnable7.Enabled := False;
    For i := 0 To 7 do
    Begin
      If i < MaxAOChannel Then
	Case i of
          0 : chkSyncEnable0.Enabled := True;
          1 : chkSyncEnable1.Enabled := True;
          2 : chkSyncEnable2.Enabled := True;
          3 : chkSyncEnable3.Enabled := True;
          4 : chkSyncEnable4.Enabled := True;
          5 : chkSyncEnable5.Enabled := True;
          6 : chkSyncEnable6.Enabled := True;
          7 : chkSyncEnable7.Enabled := True;
        End
      Else
	Case i of
          0 : chkSyncEnable0.Enabled := False;
          1 : chkSyncEnable1.Enabled := False;
          2 : chkSyncEnable2.Enabled := False;
          3 : chkSyncEnable3.Enabled := False;
          4 : chkSyncEnable4.Enabled := False;
          5 : chkSyncEnable5.Enabled := False;
          6 : chkSyncEnable6.Enabled := False;
          7 : chkSyncEnable7.Enabled := False;
        End;
    End;
  End
  Else
  Begin
    chkSyncEnable0.Enabled := False;
    chkSyncEnable1.Enabled := False;
    chkSyncEnable2.Enabled := False;
    chkSyncEnable3.Enabled := False;
    chkSyncEnable4.Enabled := False;
    chkSyncEnable5.Enabled := False;
    chkSyncEnable6.Enabled := False;
    chkSyncEnable7.Enabled := False;
    For i := 0 To 7 do
    Begin
      If i < MaxAOChannel Then
	Case i of
          0 : optAsyncEnable0.Enabled := True;
          1 : optAsyncEnable1.Enabled := True;
          2 : optAsyncEnable2.Enabled := True;
          3 : optAsyncEnable3.Enabled := True;
          4 : optAsyncEnable4.Enabled := True;
          5 : optAsyncEnable5.Enabled := True;
          6 : optAsyncEnable6.Enabled := True;
          7 : optAsyncEnable7.Enabled := True;
        End
      Else
	Case i of
          0 : optAsyncEnable0.Enabled := False;
          1 : optAsyncEnable1.Enabled := False;
          2 : optAsyncEnable2.Enabled := False;
          3 : optAsyncEnable3.Enabled := False;
          4 : optAsyncEnable4.Enabled := False;
          5 : optAsyncEnable5.Enabled := False;
          6 : optAsyncEnable6.Enabled := False;
          7 : optAsyncEnable7.Enabled := False;
        End;
    End;
  End;
end;

procedure TfrmMain.cmdStartClick(Sender: TObject);
Var
  ptDevGetPar : PT_DeviceGetParam;
  aosetting : array of AOSET;
  size : Integer;
  errmsg1720 : string[100];
  errtitle1720 : string[20];
  errmsg1720_PChar : PChar;
  errtitle1720_PChar : PChar;
  i : Integer;
  s : Boolean;
begin
  errmsg1720_PChar := @(errmsg1720[1]);
  errmsg1720 := 'For PCI-1720, the precondition of current out is that voltage range is set to 0~5V'+char(0);
  errtitle1720_PChar := @(errtitle1720[1]);

  If optVoltageOut.Checked Then
  begin
    OutputValue[0] := 0.0;
    OutputValue[1] := 1.25;
    OutputValue[2] := 2.5;
    OutputValue[3] := 5.0;
  End;
  If optCurrentOut0.Checked Then
  begin
    OutputValue[0] := 1.0;
    OutputValue[1] := 5.0;
    OutputValue[2] := 10.0;
    OutputValue[3] := 20.0;
  End;
  If optCurrentOut4.Checked Then
  begin
    OutputValue[0] := 4.0;
    OutputValue[1] := 10.0;
    OutputValue[2] := 15.0;
    OutputValue[3] := 20.0;
  End;

  lErrCde := DRV_DeviceOpen( lDeviceNum, lDeviceHandle);
  If ( DoesErr(lErrCde) = 1 ) Then
    Exit;

  { for PCI-1720, the precondition of current out is voltage range is set to 0 ~ 5V }
  If (optVoltageOut.Checked <> True) And (Pos('PCI-1720', labDeviceName.Caption) <> 0) Then
  Begin
    { get device AO settings }
    ptDevGetPar.nID := AO_RANGE_SETTING;
    SetLength(aosetting, MaxAOChannel);
    ptDevGetPar.pData := @aosetting[0];
    ptDevGetPar.Length := Longint(@size);
    lErrCde := DRV_DeviceGetParam(lDeviceHandle, ptDevGetPar);
    If (lErrCde <> 0) Then
    Begin
      DRV_DeviceClose (lDeviceHandle);
      Exit;
    End;
    { check device AO settings }
    If (chkSyncAOEnable.Checked) Then
    Begin
      For i := 0 To MaxAOChannel - 1 Do
      Begin
        If ((aosetting[i].fAOMaxVol <> 5) Or (aosetting[i].fAOMinVol <> 0)) Then
        Begin
          s := False;
          Case i of
            0 : If chkSyncEnable0.Checked Then s := True;
            1 : If chkSyncEnable1.Checked Then s := True;
            2 : If chkSyncEnable2.Checked Then s := True;
            3 : If chkSyncEnable3.Checked Then s := True;
            4 : If chkSyncEnable4.Checked Then s := True;
            5 : If chkSyncEnable5.Checked Then s := True;
            6 : If chkSyncEnable6.Checked Then s := True;
            7 : If chkSyncEnable7.Checked Then s := True;
          End;
          if (s) Then
          Begin
            DRV_DeviceClose (lDeviceHandle);
            errtitle1720 := 'Error: Channel ' + IntToStr(i);
            Application.MessageBox(errmsg1720_PChar, errtitle1720_PChar, MB_OK);
            Exit;
          End;
        End;
      End;
    End
    Else
    Begin
      For i := 0 To MaxAOChannel - 1 Do
      Begin
        If ((aosetting[i].fAOMaxVol <> 5) Or (aosetting[i].fAOMinVol <> 0)) Then
        Begin
          s := False;
          Case i of
            0 : If optAsyncEnable0.Checked Then s := True;
            1 : If optAsyncEnable1.Checked Then s := True;
            2 : If optAsyncEnable2.Checked Then s := True;
            3 : If optAsyncEnable3.Checked Then s := True;
            4 : If optAsyncEnable4.Checked Then s := True;
            5 : If optAsyncEnable5.Checked Then s := True;
            6 : If optAsyncEnable6.Checked Then s := True;
            7 : If optAsyncEnable7.Checked Then s := True;
          End;
          if (s) Then
          Begin
            DRV_DeviceClose (lDeviceHandle);
            errtitle1720 := 'Error: Channel ' + IntToStr(i) + char(0);
            Application.MessageBox(errmsg1720_PChar, errtitle1720_PChar, MB_OK);
            Exit;
          End;
        End;
      End;
    End;
  End;

  // Enable Sync. output feature
  If chkSyncAOEnable.Checked Then
    lErrCde := DRV_EnableSyncAO(lDeviceHandle, 1)
  Else
    lErrCde := DRV_EnableSyncAO(lDeviceHandle, 0);
  If ( DoesErr(lErrCde) = 1 ) Then
  Begin
    DRV_DeviceClose(lDeviceHandle);
    Exit;
  End;

  OutputIndex := 0;

  cmdStart.Enabled  := False;
  cmdStop.Enabled   := True;
  btnClose.Enabled  := False;

  ScanTimer.Enabled := True;
end;

procedure TfrmMain.cmdStopClick(Sender: TObject);
begin
  ScanTimer.Enabled := False;

  lErrCde := DRV_DeviceClose(lDeviceHandle);
  If ( DoesErr(lErrCde) = 1 ) Then
    Exit;

  cmdStart.Enabled  := True;
  cmdStop.Enabled   := False;
  btnClose.Enabled  := True;
end;

procedure TfrmMain.ScanTimerTimer(Sender: TObject);
Var
  i : Integer;
  s : Boolean;
begin
  If chkSyncAOEnable.Checked Then
  Begin
    For i := 0 To MaxAOChannel DO
    Begin
      s := False;
      Case i of
        0 : If chkSyncEnable0.Checked Then s := True;
        1 : If chkSyncEnable1.Checked Then s := True;
        2 : If chkSyncEnable2.Checked Then s := True;
        3 : If chkSyncEnable3.Checked Then s := True;
        4 : If chkSyncEnable4.Checked Then s := True;
        5 : If chkSyncEnable5.Checked Then s := True;
        6 : If chkSyncEnable6.Checked Then s := True;
        7 : If chkSyncEnable7.Checked Then s := True;
      End;
      If s Then
      Begin
	If optVoltageOut.Checked Then
        Begin
	  ptAOVoltageOut.Chan := i;
	  ptAOVoltageOut.OutputValue := OutputValue[OutputIndex];
	  lErrCde := DRV_AOVoltageOut(lDeviceHandle, ptAOVoltageOut);
	End
        Else
        Begin
	  ptAOCurrentOut.Chan := i;
	  ptAOCurrentOut.OutputValue := OutputValue[OutputIndex];
	  lErrCde := DRV_AOCurrentOut(lDeviceHandle, ptAOCurrentOut);
        End;
        If (lErrCde <> 0) Then
        Begin
            ScanTimer.Enabled := false;
            pszErrMsg := @szErrMsg;
            DRV_GetErrorMessage(lErrCde, pszErrMsg);
            Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
            cmdStopClick(nil);
            exit;
        End;
      End;
    end;
    lErrCde := DRV_WriteSyncAO(lDeviceHandle);
    If (lErrCde <> 0) Then
    Begin
        ScanTimer.Enabled := false;
        pszErrMsg := @szErrMsg;
        DRV_GetErrorMessage(lErrCde, pszErrMsg);
        Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
        cmdStopClick(nil);
        exit;
    End;
  End
  Else
  Begin
    For i := 0 To MaxAOChannel Do
    Begin
      s := False;
      Case i of
        0 : If optAsyncEnable0.Checked Then s := True;
        1 : If optAsyncEnable1.Checked Then s := True;
        2 : If optAsyncEnable2.Checked Then s := True;
      	3 : If optAsyncEnable3.Checked Then s := True;
        4 : If optAsyncEnable4.Checked Then s := True;
        5 : If optAsyncEnable5.Checked Then s := True;
        6 : If optAsyncEnable6.Checked Then s := True;
        7 : If optAsyncEnable7.Checked Then s := True;
      End;
      If s Then
      Begin
	If optVoltageOut.Checked Then
        Begin
	  ptAOVoltageOut.Chan := i;
	  ptAOVoltageOut.OutputValue := OutputValue[OutputIndex];
	  lErrCde := DRV_AOVoltageOut(lDeviceHandle, ptAOVoltageOut);
        End
        Else
        Begin
	  ptAOCurrentOut.Chan := i;
	  ptAOCurrentOut.OutputValue := OutputValue[OutputIndex];
	  lErrCde := DRV_AOCurrentOut(lDeviceHandle, ptAOCurrentOut);
        End;
        If (lErrCde <> 0) Then
        Begin
            ScanTimer.Enabled := false;
            pszErrMsg := @szErrMsg;
            DRV_GetErrorMessage(lErrCde, pszErrMsg);
            Application.MessageBox(pszErrMsg, 'Error!!', MB_OK);
            cmdStopClick(nil);
            exit;
        End;
      End;
    End;
  End;
  editOutput.Text := Format('%.3f', [OutputValue[OutputIndex]]);
  OutputIndex := OutputIndex + 1;
  If OutputIndex > 3 Then
    OutputIndex := 0;
end;

End.

