/*
 ******************************************************************************
 *  Program     : ADBMDMA.C                                                   *
 *  Description : Demo program for Windows NT with DMA data transfer          *
 *  Revision    : 2.1                                                         *
 *  Date        : 04/24/2003                  Advantech Beijing R&D Center    *
 ******************************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance()    - Saves instance handle and creates main window
 *
 *   MainWndProc()     - Process main window messages
 *
 *   ShowDataDlgProc() - Process the dialog box messages for acquired data
 *
 *   DisplayDlgProc()  - Process the dialog box messages for display range
 *
 *   ConfigDlgProc()   - Process the dialog box messages for configuration
 *
 *   GainListDlgProc() - Process the dialog box messages for gain list
 */

#include <windows.h>            /* required for all Windows applications */
#include <stdlib.h>
#include <windowsx.h>
#include <commdlg.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\Include\driver.h"
#include "..\..\..\Include\os.h"

#include "resource.h"
#include "AdDmaBm.h"

// ----------------------------------------------------------------------------
// CONSTANT DEFINITION
// ----------------------------------------------------------------------------
#define     MAX_DEVICES     100

// ----------------------------------------------------------------------------
// GLOBAL VARIABLES
// ----------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hWnd;                       // main window handle
HDC         hdc;
RECT        rect;

static      DEVFEATURES        DevFeatures;       // structure for device features
static      PT_FAIDmaExStart   ptFAIDmaExStart;   // FAIIntStart table
static      PT_FAITransfer     ptFAITransfer;     // FAITransfer table
static      PT_FAICheck        ptFAICheck;        // FAICheck table
static      PT_DeviceGetFeatures  ptDevFeatures;  // Devfeatures table
static      PT_AllocateDMABuffer  ptAllocateDMABuffer;  // buffer table

static      PT_EnableEvent     ptEnableEvent;     // Enable event
static      PT_CheckEvent      ptCheckEvent;      // Check event

HANDLE      hThreadHandle;
DWORD       dwThreadID;

char        szErrMsg[80];               // Use for MESSAGEBOX function
char        szBuffer[80];				    // Temperatory buffer
LRESULT     ErrCde;                     // Return error code

//
// used for fixed memory for the installed devices
//
DEVLIST     DeviceList[MAX_DEVICES];
DEVLIST     SubDeviceList[MAX_DEVICES];

BOOL        bThreadflag = FALSE;
BOOL        bThreadloop = FALSE;
BOOL		bNoEventOccur = FALSE;	 

static  USHORT gwGain[16] = {0,0,0,0,0,0,0,0,
                             0,0,0,0,0,0,0,0};
static  USHORT gwGainCode;
static  USHORT gwGainCde[16],ReorderGainCode[16];

static  ULONG  dwDeviceNum;                       // Device number

static  LONG   DriverHandle = (LONG)NULL;         // driver handle
static  USHORT gwDevice = 0, gwSubDevice = 0;     // device index
static  SHORT  gnNumOfDevices, gnNumOfSubdevices; // number of installed devices
static  DWORD  gdwPacerRate = 250000;               // pacer rate
static  ULONG  gulConvNum   = 200000;//40960;               // conversion number
static  USHORT gwStartChl   = 0, gwStopChl = 1;   // scan channels
static  USHORT gwDataType   = 1;                  // display type : FLOAT(1)
static  USHORT gwEvtFlag    = 1;                  // event enable(0)
static  ULONG  gwActualBufSize = 0;               // actual allocate buffer size
static  USHORT gwFifoEnable    = 0;               // Fifo disable(0)

static  USHORT gwGainList = 0;      // gain list flag
static  ULONG  gdwStartPt = 0, gdwStopPt = 99;    // display points
static  USHORT gwExtTrig  = 0;      // external or internal trigger
static  USHORT gwClockSrc = 0;		// Clock source
static  USHORT gwTrigEdge = 0;		// Trigger edge
static  USHORT gwSRCType  = 0;	   // Source type
static  FLOAT  gwTrigVol  = 0.0;		// Trigger voltage 

static  USHORT gwCyclicMode = 0;    // cyclic or non-cyclic mode
static  USHORT gwTrigMode   = 0;    // Trigger Mode
static  USHORT gwIntrCount  = 1;
static  ULONG  gwCyclicCount= 0;    // number of cyclic count

static  USHORT gwActiveBuf = 0;     // return by FAICheck
static  USHORT gwOverrun   = 0;     // return by FAICheck, FAITransfer
static  USHORT gwStopped   = 0;     // return by FAICheck
static  ULONG gulRetrieved = 0;     // return by FAICheck
static  USHORT gwHalfReady = 0;     // return by FAICheck
static  ULONG gwDelayCnt   = 2;

HGLOBAL hUserBuf, hDisplayBuf;

BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FTYPE MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
int  FTYPE WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
static FARPROC lpfnConfigDlgProc, lpfnShowDataDlgProc, lpfnDisplayDlgProc,
    lpfnGainListDlgProc;

static  HWND hwndStopButton, hwndStatusButton;
static  HANDLE  hInstance;
HMENU hMenu;

ULONG gulIntCounter     = 0;  // counter by adInterruptEvent
ULONG gulBfChgCounter   = 0;
ULONG gulOverrunCounter = 0;

ULONG gulOverrunTime  = 0;  // 0 = INFINITE
ULONG gulEventCount   = 1;    
char  szCounter[40];
BOOL  bBufferChange = 1;
BOOL  bInterrupt = 1;
BOOL  bOverrun   = 1;
BOOL  bTerminate = 1;

char szData[32];

void UserThread();
void adInterruptEvent();
void adBufChangeEvent();
void adOverrunEvent();
void adTerminateEvent();

/******************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
******************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                            /* current instance             */
HANDLE hPrevInstance;                        /* previous instance            */
LPSTR lpCmdLine;                             /* command line                 */
int nCmdShow;                                /* show-window type (open/icon) */
{
    MSG msg;                                 /* message                      */

    if (!hPrevInstance)                  /* Other instances of app running?  */
        if (!InitApplication(hInstance)) /* Initialize shared things         */
            return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,        /* message structure                      */
        (HWND)NULL,                /* handle of window receiving the message */
        (UINT)NULL,                /* lowest message to examine              */
        (UINT)NULL))               /* highest message to examine             */
    {
        TranslateMessage(&msg);    /* Translates virtual key codes           */
        DispatchMessage(&msg);     /* Dispatches message to window           */
    }
    return (msg.wParam);           /* Returns the value from PostQuitMessage */
}

/******************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
******************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                              /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW;   /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;   /* window process procedure     */
    wc.cbClsExtra = 0;                    /* No per-class extra data.        */
    wc.cbWndExtra = 0;                    /* No per-window extra data.       */
    wc.hInstance = hInstance;             /* Application that owns the class.*/
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";      /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";     /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));
}

/******************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
******************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
HANDLE          hInstance;          /* Current instance identifier.          */
int             nCmdShow;           /* Param for first ShowWindow() call.    */
{

    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hWnd = CreateWindow(
        "MyClass",                    /* See RegisterClass() call.           */
        "Advantech Driver Demo : Busmaster DMA Data Transfer" ,
        /* Window title bar  */
        WS_OVERLAPPEDWINDOW,          /* Window style.                       */
        CW_USEDEFAULT,                /* Default horizontal position.        */
        CW_USEDEFAULT,                /* Default vertical position.          */
        CW_USEDEFAULT,                /* Default width.                      */
        CW_USEDEFAULT,                /* Default height.                     */
        NULL,                         /* Overlapped windows have no parent.  */
        NULL,                         /* Use the window class menu.          */
        hInstance,                    /* This instance owns this window.     */
        NULL                          /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hWnd, nCmdShow);     /* Show the window                       */
    UpdateWindow(hWnd);             /* Sends WM_PAINT message                */
    return (TRUE);                  /* Returns the value from PostQuitMessage*/
}

/******************************************************************************
    FUNCTION: GainListDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for gain list
******************************************************************************/

BOOL FTYPE GainListDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                                  /* window handle                 */
unsigned message;                           /* type of message               */
WPARAM wParam;                              /* additional information        */
LPARAM lParam;                              /* additional information        */
{
    USHORT i, j, temp;

    switch (message)
    {
    case WM_INITDIALOG :

        // ------------------------------------------------------------
        // Initialize Input Range List Combobox, it needs to get device
        // features for gain list
        // ------------------------------------------------------------

        // Step 1: Open Device
        ErrCde = DRV_DeviceOpen(dwDeviceNum,
                (LONG far *)&DriverHandle);
        if (ErrCde != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }

        // Step 2: Get device features
        ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
        ptDevFeatures.size = sizeof(DEVFEATURES);
        if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
            (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return 0;
        }

        // Step 3: Initialize Input Range List Combobox with device features
        for (i = 0; i < 16; i ++)
            EnableWindow(GetDlgItem(hDlg, IDC_CHL0 + i), FALSE);

        if (DevFeatures.usNumGain != 0)
        {
			int t;
            for(i = (USHORT)gwStartChl; i < (gwStopChl+gwStartChl); i++)
            {
				t =  i;
				if(t>15) t=t-16;

                EnableWindow(GetDlgItem(hDlg, IDC_CHL0+t), TRUE);
                for(j = 0; j < (USHORT)DevFeatures.usNumGain; j ++)
                    SendDlgItemMessage(hDlg, IDC_CHL0+t, CB_ADDSTRING, 0,
                    (LPARAM)((LPSTR)(DevFeatures.glGainList[j].szGainStr)));

                temp = gwGain[t];

                SendDlgItemMessage(hDlg, IDC_CHL0+t,
                    CB_SELECTSTRING, (WPARAM)(-1),
                    (LPARAM)((LPSTR)(DevFeatures.glGainList[temp].szGainStr)));
            }
        }
        // Step 4: Close device
        DRV_DeviceClose((LONG far *)&DriverHandle);

        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

        case IDC_CHL0 :
        case IDC_CHL1 :
        case IDC_CHL2 :
        case IDC_CHL3 :
        case IDC_CHL4 :
        case IDC_CHL5 :
        case IDC_CHL6 :
        case IDC_CHL7 :
        case IDC_CHL8 :
        case IDC_CHL9 :
        case IDC_CHL10 :
        case IDC_CHL11 :
        case IDC_CHL12 :
        case IDC_CHL13 :
        case IDC_CHL14 :
        case IDC_CHL15 :
            if (HIWORD(wParam) == CBN_KILLFOCUS)
            {
                i = (int)SendDlgItemMessage(hDlg, LOWORD(wParam),
                    CB_GETCURSEL, 0, 0) ;
                if (i != CB_ERR)
                    gwGain[((USHORT)LOWORD(wParam)) - IDC_CHL0] = i;
            }
            return TRUE;

        }
        break;
    }
    return FALSE ;
}

/******************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
******************************************************************************/

BOOL FTYPE ConfigDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                          /* additional information          */
LPARAM lParam;                          /* additional information          */
{
    int     i;
    char    szBuffer[80];

    switch (message)
    {
    case WM_INITDIALOG :
       
        // Step 1: Select Device
        ErrCde = DRV_SelectDevice( hDlg, FALSE, &dwDeviceNum, szBuffer);
        if (ErrCde != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }

        // Step 2: Open Device
        ErrCde = DRV_DeviceOpen(dwDeviceNum, (LONG *)&DriverHandle);
        if (ErrCde != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }

        // Step 3: Get device features
        ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
        ptDevFeatures.size = sizeof(DEVFEATURES);
        if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
            (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return 0;
        }

        // Step 4: Initialize Input Range List Combobox with device features
        if (DevFeatures.usNumGain != 0)
        {
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
            for (i = 0; i < (int)DevFeatures.usNumGain; i ++)
                SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_ADDSTRING, 0,
                (LPARAM)((LPSTR)(DevFeatures.glGainList[i].szGainStr)));

            SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_SELECTSTRING, (WPARAM)(-1),
                (LPARAM)((LPSTR)(DevFeatures.glGainList[gwGainCode].szGainStr)));
        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
            SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                0, (LPARAM)((LPSTR)0));
        }

        // Initialize Conversion Number
        ultoa(gulConvNum, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_CONVERSION, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Pacer Rate
        ultoa(gdwPacerRate, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_PACER, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        if (gwClockSrc)
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), FALSE);
        else
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), TRUE);

        // Initialize Start Channel
        itoa((int)gwStartChl, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTART, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Stop Channel
        itoa((int)gwStopChl, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTOP, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Delay Count
        itoa((int)gwDelayCnt, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_DELAYCNT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Event Count
        itoa((int)gulEventCount, szBuffer, 10);
        SendDlgItemMessage(hDlg, IDC_EVENTCOUNT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));
       
        // Initialize Overrun time
        itoa((int)gulOverrunTime, szBuffer, 10);
        SendDlgItemMessage(hDlg, IDC_OVERRUNTIME, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));
        
        // Initialize Clock Source
        CheckRadioButton(hDlg, IDC_INTTRIG, IDC_EXTTRIG_1,
            IDC_INTTRIG + gwClockSrc);

        // Initialize trigger edge
        CheckRadioButton(hDlg, IDC_EDGERISE, IDC_EDGEFALL,
            (gwTrigEdge)?IDC_EDGEFALL:IDC_EDGERISE);

        // Initialize trigger source type
		  CheckRadioButton(hDlg, IDC_DIGTYPE, IDC_ANGTYPE_3,
            IDC_DIGTYPE + gwSRCType );

        if( gwTrigMode )
        {
           	EnableWindow(GetDlgItem(hDlg, IDC_DIGTYPE), TRUE);
			   EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_0), TRUE);
			   EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_1), TRUE);
			   EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_2), TRUE);
			   EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_3), TRUE);
            EnableWindow(GetDlgItem(hDlg, IDC_EDGERISE), TRUE);
			   EnableWindow(GetDlgItem(hDlg, IDC_EDGEFALL), TRUE);
        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_DIGTYPE), FALSE);
			   EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_0), FALSE);
			   EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_1), FALSE);
			   EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_2), FALSE);
			   EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_3), FALSE);
            EnableWindow(GetDlgItem(hDlg, IDC_EDGERISE), FALSE);
			   EnableWindow(GetDlgItem(hDlg, IDC_EDGEFALL), FALSE);
        }
		  if ( 3==gwTrigMode )
        {
		     EnableWindow(GetDlgItem(hDlg,IDC_CYCLIC), FALSE);
           EnableWindow(GetDlgItem(hDlg,IDC_NONCYCLIC), FALSE);
        }
        else
        {
		     EnableWindow(GetDlgItem(hDlg,IDC_CYCLIC), TRUE);
           EnableWindow(GetDlgItem(hDlg,IDC_NONCYCLIC), TRUE);
        }

        EnableWindow(GetDlgItem(hDlg, IDC_TRIGSRC1), FALSE);
        EnableWindow(GetDlgItem(hDlg, IDC_TRIGSRC2), FALSE);
        
        // Initialize Trigger Voltage
        sprintf(szBuffer,"%6.3f" ,gwTrigVol);
        SendDlgItemMessage(hDlg, IDC_TRIGVOLTAGE, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

		if(gwSRCType)
		{
			EnableWindow(GetDlgItem(hDlg, IDC_TRIGVOLTAGE), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_TEXTVOL), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_TEXTV), TRUE);
		}
		else
		{
			EnableWindow(GetDlgItem(hDlg, IDC_TRIGVOLTAGE), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_TEXTVOL), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_TEXTV), FALSE);
		}
        // Initialize Repeated Mode
        CheckRadioButton(hDlg, IDC_CYCLIC, IDC_NONCYCLIC,
            (gwCyclicMode)?IDC_CYCLIC:IDC_NONCYCLIC);

        // Initialize Trigger Mode
        CheckRadioButton(hDlg, IDC_PACERTRIG, IDC_ABOUTTRIG, (IDC_PACERTRIG + gwTrigMode));
		if((gwTrigMode == 2) || (gwTrigMode == 3))
			EnableWindow(GetDlgItem(hDlg,IDC_DELAYCNT), TRUE);
		else
			EnableWindow(GetDlgItem(hDlg,IDC_DELAYCNT), FALSE);

        // Initialize Trigger Source
        CheckRadioButton(hDlg, IDC_TRIGSRC1, IDC_TRIGSRC2,
            (gwExtTrig)?IDC_TRIGSRC2:IDC_TRIGSRC1);

        // Initialize Data Type
        CheckRadioButton(hDlg, IDC_DATAUSHORT, IDC_DATAFLOAT,
            (gwDataType)?IDC_DATAFLOAT:IDC_DATAUSHORT);

        // Initialize Message Flag
		if(gwEvtFlag)
			CheckDlgButton(hDlg,IDC_EnableEvent,TRUE);
		else
			CheckDlgButton(hDlg,IDC_EnableEvent,FALSE);
		
		
		

        SendDlgItemMessage(hDlg, IDC_CONVINT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize gain table option
        if (gwGainList)
        {
            CheckRadioButton(hDlg, IDC_OVERALLGAIN, IDC_GAINARRAY,
                IDC_GAINARRAY);
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
            EnableWindow(GetDlgItem(hDlg, IDC_GAINLIST), TRUE);
        }
        else
        {
            CheckRadioButton(hDlg, IDC_OVERALLGAIN, IDC_GAINARRAY,
                IDC_OVERALLGAIN);
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
            EnableWindow(GetDlgItem(hDlg, IDC_GAINLIST), FALSE);
        }

        // Ininialize event check box
		  EnableWindow(GetDlgItem(hDlg,(IDC_BUFFERCHANGE)),TRUE);
		  if( bBufferChange )
        {
		      CheckDlgButton(hDlg,(IDC_BUFFERCHANGE),TRUE);
        }
		  else
        {
		      CheckDlgButton(hDlg,(IDC_BUFFERCHANGE),FALSE);
        }

		  // EnableWindow(GetDlgItem(hDlg,(IDC_BUFFERCHANGE)),TRUE);
		  // Terminate
        if( bTerminate )
        {
		      CheckDlgButton(hDlg,(IDC_TERMINATE),TRUE);
        }
		  else
        {
		      CheckDlgButton(hDlg,(IDC_TERMINATE),FALSE);
        }
        // Overrun
        if( bOverrun )
        {
		      CheckDlgButton(hDlg,(IDC_OVERRUN),TRUE);
        }
		  else
        {
		      CheckDlgButton(hDlg,(IDC_OVERRUN),FALSE);
        }
        // Interrupt
        if( bInterrupt )
        {
		      CheckDlgButton(hDlg,(IDC_INTERRUPT),TRUE);
		      EnableWindow(GetDlgItem(hDlg,(IDC_EVENTCOUNT)),TRUE);
        }
		  else
        {
		      CheckDlgButton(hDlg,(IDC_INTERRUPT),FALSE);
            EnableWindow(GetDlgItem(hDlg,(IDC_EVENTCOUNT)),FALSE);

        }

        // Step 5: Close device
        DRV_DeviceClose((LONG far *)&DriverHandle);
        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
			{
				for ( i = IDC_PACERTRIG; i <= IDC_ABOUTTRIG; i++)
					if( SendDlgItemMessage(hDlg, i, BM_GETCHECK, 0, 0) == 1)
						gwTrigMode = (i - IDC_PACERTRIG);

				if((gwTrigMode == 2) || (gwTrigMode == 3))
				{
					SendDlgItemMessage(hDlg, IDC_DELAYCNT, WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
					gwDelayCnt = (USHORT)atoi(szBuffer);
				}
				else
					gwDelayCnt = 2;

				if(gwSRCType)
				{
					SendDlgItemMessage(hDlg, IDC_TRIGVOLTAGE, WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
					gwTrigVol = (FLOAT)atof(szBuffer);
				}
			}

        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

        case IDC_GAINLIST :
            DialogBox (hInst, MAKEINTRESOURCE(IDD_GAINLIST),
                hDlg, lpfnGainListDlgProc) ;
            return TRUE;

        case IDC_DeviceSelect:

            // Step 1: Select Device
            ErrCde = DRV_SelectDevice(hDlg, FALSE, &dwDeviceNum, szBuffer);
            if (ErrCde != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                return 0;
            }

            // -------------------------------------------------
            // Initialize Input Range List Combobox, it needs to
            // get device features for gain list
            // -------------------------------------------------

            // Step 2: Open Device
            ErrCde = DRV_DeviceOpen(dwDeviceNum, (LONG far *)&DriverHandle);
            if (ErrCde != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd, (LPCSTR)szErrMsg,
                    "Driver Message", MB_OK);
                return 0;
            }

            // Step 3: Get device features
            ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
            ptDevFeatures.size = sizeof(DEVFEATURES);
            if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd, (LPCSTR)szErrMsg, "Driver Message",
                    MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            // Step 4: Initialize Range Combobox with device features
            gwGainCode = 0;

            if (DevFeatures.usNumGain != 0)
            {
                EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
                SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                    0, (LPARAM)((LPSTR)0));
                for (i = 0; i < (int)DevFeatures.usNumGain; i ++)
                    SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_ADDSTRING, 0,
                    (LPARAM)((LPSTR)(DevFeatures.glGainList[i].szGainStr)));

                SendDlgItemMessage(hDlg, IDC_INPRANGE,
                    CB_SELECTSTRING, (WPARAM)(-1),
                    (LPARAM)((LPSTR)(DevFeatures.glGainList[gwGainCode].szGainStr)));
            }
            else
            {
                EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
                SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                    0, (LPARAM)((LPSTR)0));
            }

            // Step 5: Close device
            DRV_DeviceClose((LONG far *)&DriverHandle);

            return TRUE;

        case IDC_CONVERSION :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_CONVERSION,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_CONVERSION,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gulConvNum = (ULONG)atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_CONVERSION,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;
        case IDC_PACER :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_PACER,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_PACER,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gdwPacerRate = (ULONG)atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_PACER,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;
        case IDC_INPRANGE :
            if (HIWORD(wParam) == CBN_KILLFOCUS)
            {
                i = (int)SendDlgItemMessage(hDlg, IDC_INPRANGE,
                    CB_GETCURSEL, 0, 0) ;
                if (i != CB_ERR)
                    gwGainCode = (USHORT)i;
            }
            return TRUE;
        case IDC_ESTART :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTART,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTART,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwStartChl = (USHORT)atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTART,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;
        case IDC_ESTOP :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTOP,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTOP,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwStopChl = (USHORT)atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTOP,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;

        case IDC_OVERALLGAIN :
            gwGainList = 0;
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
            EnableWindow(GetDlgItem(hDlg, IDC_GAINLIST), FALSE);
            break;
        case IDC_GAINARRAY :
            gwGainList = 1;
            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
            EnableWindow(GetDlgItem(hDlg, IDC_GAINLIST), TRUE);
            break;
        case IDC_INTTRIG :
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), TRUE);
            gwClockSrc = 0;
            break;
        case IDC_EXTTRIG_0:
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), FALSE);
            gwClockSrc = 1;
            break;
        case IDC_EXTTRIG_1:
            EnableWindow(GetDlgItem(hDlg, IDC_PACER), FALSE);
            gwClockSrc = 2;
            break;

        case IDC_CYCLIC :
            gwCyclicMode = 1;
            break;
        case IDC_NONCYCLIC :
            gwCyclicMode = 0;
            break;
        case IDC_DATAUSHORT :
            gwDataType = 0;
            break;
        case IDC_DATAFLOAT :
            gwDataType = 1;
            break;
        case IDC_EVTENABLE :
            gwEvtFlag = 1;
            break;
        case IDC_EVTDISABLE :
            gwEvtFlag = 0;
            break;

		case IDC_PACERTRIG:
			EnableWindow(GetDlgItem(hDlg,IDC_DELAYCNT), FALSE);
         EnableWindow(GetDlgItem(hDlg, IDC_TRIGSRC1), FALSE);
         EnableWindow(GetDlgItem(hDlg, IDC_TRIGSRC2), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_DIGTYPE), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_0), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_1), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_2), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_3), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_EDGERISE), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_EDGEFALL), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_CYCLIC), TRUE);
         EnableWindow(GetDlgItem(hDlg,IDC_NONCYCLIC), TRUE);

         gwExtTrig = 0;
         CheckRadioButton(hDlg, IDC_TRIGSRC1, IDC_TRIGSRC2,
            (gwExtTrig)?IDC_TRIGSRC2:IDC_TRIGSRC1);


			break;
		case IDC_POSTTRIG:
			EnableWindow(GetDlgItem(hDlg,IDC_DELAYCNT), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_TRIGSRC1), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_TRIGSRC2), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_DIGTYPE), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_0), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_1), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_2), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_3), TRUE);
         EnableWindow(GetDlgItem(hDlg,IDC_EDGERISE), TRUE);
         EnableWindow(GetDlgItem(hDlg,IDC_EDGEFALL), TRUE);
         EnableWindow(GetDlgItem(hDlg,IDC_CYCLIC), TRUE);
         EnableWindow(GetDlgItem(hDlg,IDC_NONCYCLIC), TRUE);

         gwExtTrig = 1;
         CheckRadioButton(hDlg, IDC_TRIGSRC1, IDC_TRIGSRC2,
            (gwExtTrig)?IDC_TRIGSRC2:IDC_TRIGSRC1);

		 gwTrigEdge = 1;
        CheckRadioButton(hDlg, IDC_EDGERISE, IDC_EDGEFALL,
            (gwTrigEdge)?IDC_EDGEFALL:IDC_EDGERISE);
		 

			break;

      case IDC_DELAYTRIG:
		case IDC_ABOUTTRIG:
			EnableWindow(GetDlgItem(hDlg,IDC_DELAYCNT), TRUE);
         EnableWindow(GetDlgItem(hDlg,IDC_TRIGSRC1), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_TRIGSRC2), FALSE);
         EnableWindow(GetDlgItem(hDlg,IDC_DIGTYPE), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_0), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_1), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_2), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_ANGTYPE_3), TRUE);
		   EnableWindow(GetDlgItem(hDlg,IDC_EDGERISE), TRUE);
         EnableWindow(GetDlgItem(hDlg,IDC_EDGEFALL), TRUE);
         if ( wParam == IDC_ABOUTTRIG )
         {
             gwCyclicMode = 1;
		     EnableWindow(GetDlgItem(hDlg,IDC_CYCLIC), FALSE);
             EnableWindow(GetDlgItem(hDlg,IDC_NONCYCLIC), FALSE);
             CheckRadioButton(hDlg, IDC_CYCLIC, IDC_NONCYCLIC,
                (gwCyclicMode)?IDC_CYCLIC:IDC_NONCYCLIC);
			 gwTrigEdge = 0;
			CheckRadioButton(hDlg, IDC_EDGERISE, IDC_EDGEFALL,
				(gwTrigEdge)?IDC_EDGEFALL:IDC_EDGERISE);
         }
         else
         {
  		       EnableWindow(GetDlgItem(hDlg,IDC_CYCLIC), TRUE);
             EnableWindow(GetDlgItem(hDlg,IDC_NONCYCLIC), TRUE);
			 gwTrigEdge = 1;
			CheckRadioButton(hDlg, IDC_EDGERISE, IDC_EDGEFALL,
				(gwTrigEdge)?IDC_EDGEFALL:IDC_EDGERISE);
         }
         
         gwExtTrig = 1;
         CheckRadioButton(hDlg, IDC_TRIGSRC1, IDC_TRIGSRC2,
            (gwExtTrig)?IDC_TRIGSRC2:IDC_TRIGSRC1);

			break;

      case IDC_TRIGSRC1:
         gwExtTrig = 0;
         break;

      case IDC_TRIGSRC2:
         gwExtTrig = 1;
         break;
         
		case IDC_EDGERISE:
			gwTrigEdge = 0;
			break;
		case IDC_EDGEFALL:
			gwTrigEdge = 1;
			break;
		case IDC_DIGTYPE:
			EnableWindow(GetDlgItem(hDlg, IDC_TRIGVOLTAGE), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_TEXTVOL), FALSE);
			EnableWindow(GetDlgItem(hDlg, IDC_TEXTV), FALSE);
			gwSRCType = 0;
			break;

		case IDC_ANGTYPE_0:
		case IDC_ANGTYPE_1:
		case IDC_ANGTYPE_2:
		case IDC_ANGTYPE_3:
			EnableWindow(GetDlgItem(hDlg, IDC_TRIGVOLTAGE), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_TEXTVOL), TRUE);
			EnableWindow(GetDlgItem(hDlg, IDC_TEXTV), TRUE);
			gwSRCType = wParam - IDC_DIGTYPE;
			break;

      case IDC_BUFFERCHANGE:
         if(IsDlgButtonChecked(hDlg, IDC_BUFFERCHANGE))
			{ 
             bBufferChange = 1; 		   
			}
         else
         {
            bBufferChange = 0;
         }
         break;
      case IDC_TERMINATE:
         if(IsDlgButtonChecked(hDlg, IDC_TERMINATE))
			{ 
             bTerminate = 1; 		   
			}
         else
         {
             bTerminate = 0;
         }
         break;
     case IDC_OVERRUN:
         if(IsDlgButtonChecked(hDlg, IDC_OVERRUN))
             bOverrun = 1;
         else
             bOverrun = 0;
         break;
	 case IDC_EnableEvent:
         if(IsDlgButtonChecked(hDlg, IDC_EnableEvent))
             gwEvtFlag = 1;
         else
             gwEvtFlag = 0;
         break;
		 
     case IDC_INTERRUPT:
         if(IsDlgButtonChecked(hDlg, IDC_INTERRUPT))
			{ 
             bInterrupt = 1; 	
             EnableWindow(GetDlgItem(hDlg, IDC_EVENTCOUNT), TRUE);
			}
         else
         {
             bInterrupt = 0;
             EnableWindow(GetDlgItem(hDlg, IDC_EVENTCOUNT), FALSE);
         }
         break;

		case IDC_EVENTCOUNT:
         if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_EVENTCOUNT,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_EVENTCOUNT,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer);
                    if ( atol(szBuffer)<=0 || atol(szBuffer)>=32 )
                    {
                       MessageBox(hWnd,"Invalid event count.", "Check Status", MB_OK);
                       break;
                    }
                    gulEventCount = (ULONG)atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_EVENTCOUNT,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
 
         break;

      case IDC_OVERRUNTIME:
         if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_OVERRUNTIME,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_OVERRUNTIME,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gulOverrunTime = (ULONG)atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_OVERRUNTIME,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
         break;

      case IDC_DELAYCNT:
         if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_DELAYCNT,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_DELAYCNT,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwDelayCnt = (ULONG)atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_DELAYCNT,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
         break;
 
        }
        break;
    }
    return FALSE ;
}

/******************************************************************************
    FUNCTION: ShowDataDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for acquired data
******************************************************************************/

BOOL FTYPE ShowDataDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                                  /* window handle                 */
unsigned message;                           /* type of message               */
WPARAM wParam;                              /* additional information        */
LPARAM lParam;                              /* additional information        */
{
    static  LONG  lEndDisplay, lStartDisplay;
    LONG  i;
    LPVOID temp;
    HCURSOR hCursor;

    if (gwDataType == 0)
       temp = (USHORT far *)ptFAITransfer.DataBuffer;
    else
       temp = (FLOAT far *)ptFAITransfer.DataBuffer;

    switch (message)
    {
    case WM_INITDIALOG :
		{
		if(gulConvNum > 4095)
			lEndDisplay = 4095;
		else
			lEndDisplay = gulConvNum-1;

		lStartDisplay = gdwStartPt;

        ultoa(lStartDisplay, szBuffer, 10);
		SetWindowText(GetDlgItem(hDlg,IDC_FROM),(LPCTSTR)szBuffer);

        ultoa(lEndDisplay, szBuffer, 10);
		SetWindowText(GetDlgItem(hDlg,IDC_TO),(LPCTSTR)szBuffer);

        ultoa(gulConvNum, szBuffer, 10);
		SetWindowText(GetDlgItem(hDlg,IDC_TOTAL),(LPCTSTR)szBuffer);

        for(i = lStartDisplay; i <= lEndDisplay; ++i)
        {

            if (gwDataType == 0)
                sprintf(szBuffer, "Buf[%ld] = %4x",i, ((USHORT far *)temp)[i]);
            else
                sprintf(szBuffer, "Buf[%ld] = %10.6f", i, ((FLOAT far *)temp)[i]);

            if(SendDlgItemMessage(hDlg, IDC_DATALIST, LB_ADDSTRING, 0,
                (LPARAM)(LPSTR)szBuffer) == LB_ERRSPACE)
            {   MessageBox(hWnd,(LPCSTR)"Not Enough Memory",
                    "Display Data",MB_OK);
                return TRUE;
            }
        }

		if(gulConvNum<4095)
		{
			EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), FALSE);
		}
		else
		{
			EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), TRUE);
		}

        return TRUE;
		}

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDC_BACKBUTTON :
			{
			hCursor = LoadCursor(NULL,IDC_WAIT);
			SetCursor(hCursor);

			for(i=0; i<8192; i++)
			  SendDlgItemMessage(hDlg, IDC_DATALIST, LB_DELETESTRING , 0, (LPARAM)(LPSTR)0);

			if(gulConvNum > 4095)
				lEndDisplay = 4095;
			else
				lEndDisplay = gulConvNum-1;

			lStartDisplay = gdwStartPt;
	
			ultoa(lStartDisplay, szBuffer, 10);
			SetWindowText(GetDlgItem(hDlg,IDC_FROM),(LPCTSTR)szBuffer);

			ultoa(lEndDisplay, szBuffer, 10);
			SetWindowText(GetDlgItem(hDlg,IDC_TO),(LPCTSTR)szBuffer);

			for(i = lStartDisplay; i <= lEndDisplay; ++i)
			{

				if (gwDataType == 0)
					sprintf(szBuffer, "Buf[%ld] = %4x",i, ((USHORT far *)temp)[i]);
				else
					sprintf(szBuffer, "Buf[%ld] = %10.6f", i, ((FLOAT far *)temp)[i]);

				if(SendDlgItemMessage(hDlg, IDC_DATALIST, LB_ADDSTRING, 0,
					(LPARAM)(LPSTR)szBuffer) == LB_ERRSPACE)
				{   MessageBox(hWnd,(LPCSTR)"Not Enough Memory",
						"Display Data",MB_OK);
                return TRUE;
				}
			}

			if(gulConvNum<4095)
			{
				EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), FALSE);
			}
			else
			{
				EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), TRUE);
			}

			hCursor = LoadCursor(NULL, IDC_ARROW);
			SetCursor(hCursor);
			break;
			}
        case IDC_BACKWARD :
			{
			hCursor = LoadCursor(NULL,IDC_WAIT);
			SetCursor(hCursor);

			for(i=0; i<8192; i++)
			  SendDlgItemMessage(hDlg, IDC_DATALIST, LB_DELETESTRING , 0, (LPARAM)(LPSTR)0);

			lEndDisplay -= 4096;
			lStartDisplay -= 4096;

			if(lStartDisplay < 0)
			{
				lStartDisplay = 0;
				lEndDisplay = 4095;
			}

			ultoa(lStartDisplay, szBuffer, 10);
			SetWindowText(GetDlgItem(hDlg,IDC_FROM),(LPCTSTR)szBuffer);

			ultoa(lEndDisplay, szBuffer, 10);
			SetWindowText(GetDlgItem(hDlg,IDC_TO),(LPCTSTR)szBuffer);

			for(i = lStartDisplay; i <= lEndDisplay; ++i)
			{

				if (gwDataType == 0)
					sprintf(szBuffer, "Buf[%ld] = %4x",i, ((USHORT far *)temp)[i]);
				else
					sprintf(szBuffer, "Buf[%ld] = %10.6f", i, ((FLOAT far *)temp)[i]);

				if(SendDlgItemMessage(hDlg, IDC_DATALIST, LB_ADDSTRING, 0,
					(LPARAM)(LPSTR)szBuffer) == LB_ERRSPACE)
				{   MessageBox(hWnd,(LPCSTR)"Not Enough Memory",
						"Display Data",MB_OK);
                return TRUE;
				}
			}

			if(lStartDisplay == 0)
			{
				EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), TRUE);
			}
			else
			{
				EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), TRUE);
			}

			hCursor = LoadCursor(NULL, IDC_ARROW);
			SetCursor(hCursor);
			break;
			}
        case IDC_FORWARD :
			{
			hCursor = LoadCursor(NULL, IDC_WAIT);
			SetCursor(hCursor);

			for(i=0; i<8192; i++)
			  SendDlgItemMessage(hDlg, IDC_DATALIST, LB_DELETESTRING , 0, (LPARAM)(LPSTR)0);

			lEndDisplay += 4096;
			lStartDisplay += 4096;

			if(lEndDisplay > (LONG)gulConvNum)
				lEndDisplay = gulConvNum-1;

			if(lStartDisplay < 0)
				lStartDisplay = 0;

			ultoa(lStartDisplay, szBuffer, 10);
			SetWindowText(GetDlgItem(hDlg,IDC_FROM),(LPCTSTR)szBuffer);

			ultoa(lEndDisplay, szBuffer, 10);
			SetWindowText(GetDlgItem(hDlg,IDC_TO),(LPCTSTR)szBuffer);

			for(i = lStartDisplay; i <= lEndDisplay; ++i)
			{

				if (gwDataType == 0)
					sprintf(szBuffer, "Buf[%ld] = %4x",i, ((USHORT far *)temp)[i]);
				else
					sprintf(szBuffer, "Buf[%ld] = %10.6f", i, ((FLOAT far *)temp)[i]);

				if(SendDlgItemMessage(hDlg, IDC_DATALIST, LB_ADDSTRING, 0,
					(LPARAM)(LPSTR)szBuffer) == LB_ERRSPACE)
				{   MessageBox(hWnd,(LPCSTR)"Not Enough Memory",
						"Display Data",MB_OK);
                return TRUE;
				}
			}

			if(lEndDisplay == (LONG)gulConvNum-1)
			{
				EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), FALSE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), FALSE);
			}
			else
			{
				EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), TRUE);
				EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), TRUE);
			}

			hCursor = LoadCursor(NULL,IDC_ARROW);
			SetCursor(hCursor);
			break;
			}
        case IDC_FORWARDBUTTON :
			{
			hCursor = LoadCursor(NULL, IDC_WAIT);
			SetCursor(hCursor);

			for(i=0; i<8192; i++)
			  SendDlgItemMessage(hDlg, IDC_DATALIST, LB_DELETESTRING , 0, (LPARAM)(LPSTR)0);

			lEndDisplay = gulConvNum-1;
			lStartDisplay = ((gulConvNum/4095)-1)*4095;
	
			ultoa(lStartDisplay, szBuffer, 10);
			SetWindowText(GetDlgItem(hDlg,IDC_FROM),(LPCTSTR)szBuffer);

			ultoa(lEndDisplay, szBuffer, 10);
			SetWindowText(GetDlgItem(hDlg,IDC_TO),(LPCTSTR)szBuffer);

			for(i = lStartDisplay; i <= lEndDisplay; ++i)
			{

				if (gwDataType == 0)
					sprintf(szBuffer, "Buf[%ld] = %4x",i, ((USHORT far *)temp)[i]);
				else
					sprintf(szBuffer, "Buf[%ld] = %10.6f", i, ((FLOAT far *)temp)[i]);

				if(SendDlgItemMessage(hDlg, IDC_DATALIST, LB_ADDSTRING, 0,
					(LPARAM)(LPSTR)szBuffer) == LB_ERRSPACE)
				{   MessageBox(hWnd,(LPCSTR)"Not Enough Memory",
						"Display Data",MB_OK);
                return TRUE;
				}
			}
			EnableWindow(GetDlgItem(hDlg,IDC_BACKWARD), TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_BACKBUTTON), TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_FORWARD), FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_FORWARDBUTTON), FALSE);

			hCursor = LoadCursor(NULL, IDC_ARROW);
			SetCursor(hCursor);
			break;
			}

        case IDC_UPDATE :
			 break;

        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

        }
        break;
    }
    return FALSE ;
}

/******************************************************************************
    FUNCTION: DisplayDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for display range
******************************************************************************/

BOOL FTYPE DisplayDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                               /* window handle                   */
unsigned message;                        /* type of message                 */
WPARAM wParam;                           /* additional information          */
LPARAM lParam;                           /* additional information          */
{

    switch (message)
    {
    case WM_INITDIALOG :

        // Initialize Start Point
        ultoa(gdwStartPt, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTAPT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Stop Point
        ultoa(gdwStopPt, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTOPT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

        case IDC_ESTAPT :

            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTAPT,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTAPT,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gdwStartPt = (USHORT)atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTAPT,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;

        case IDC_ESTOPT :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTOPT,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTOPT,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gdwStopPt = atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTOPT,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;
        }
        break;
    }
    return FALSE ;
}

/******************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
******************************************************************************/

long FTYPE MainWndProc(hWnd, message, wParam, lParam)
HWND     hWnd;                            /* window handle                   */
unsigned message;                         /* type of message                 */
WPARAM   wParam;                          /* additional information          */
LPARAM   lParam;                          /* additional information          */
{
    int     dec, sign, i;
    TEXTMETRIC tm;
    HDC     hdc;
    static  int cxChar, cyChar;
    char    *szStr, szDest[100], szTemp[50];
    double  ratio;
    USHORT  temp;


    switch (message)
    {
    case WM_CREATE:

        hMenu = GetMenu(hWnd);

        hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

        lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

        lpfnShowDataDlgProc = MakeProcInstance (ShowDataDlgProc, hInstance) ;

        lpfnDisplayDlgProc = MakeProcInstance (DisplayDlgProc, hInstance) ;

        lpfnGainListDlgProc = MakeProcInstance (GainListDlgProc, hInstance) ;

        hdc = GetDC(hWnd);
        SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));
        GetTextMetrics(hdc, &tm);
        cxChar = tm.tmAveCharWidth;
        cyChar = tm.tmHeight + tm.tmExternalLeading;
        ReleaseDC(hWnd, hdc);

        //Create stop button and status button
        hwndStopButton = CreateWindow("button", "Stop",
            (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
            0, 0, 8*cxChar, 3*cyChar/2, hWnd,
            (HMENU)IDC_STOPBUTTON,
            (HANDLE)hInstance, (LPVOID)NULL);

        hwndStatusButton = CreateWindow("button", "Status",
            (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
            8*cxChar, 0, 8*cxChar, 3*cyChar/2, hWnd,
            (HMENU)IDC_STATUSBUTTON,
            (HANDLE)hInstance, (LPVOID)NULL);

        EnableWindow(hwndStopButton, FALSE);
        EnableWindow(hwndStatusButton, FALSE);

        return 0 ;

    case WM_COMMAND:     /* message: from application menu */

        hMenu = GetMenu(hWnd);

        switch(LOWORD(wParam))
        {
            // Show the dialog box of the input parameter
        case IDM_SETTING :

            DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                hWnd, lpfnConfigDlgProc) ;
            return 0;

            // Show the dialog box for the range of data to display
        case IDM_DISPLAY:

            DialogBox (hInstance, MAKEINTRESOURCE(IDD_DISPLAY),
                hWnd, lpfnDisplayDlgProc) ;

            return 0;

        case IDM_RUN :

            // Step 1: Device open
            ErrCde = DRV_DeviceOpen(dwDeviceNum, (LONG far *)&DriverHandle);
            if (ErrCde != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                return 0;
            }

            // Step 2: Get device features
            ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
            ptDevFeatures.size = sizeof(DEVFEATURES);
            if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            // Step 3: Allocate memory for Voltage data or Raw data
            //********Version 2.0B*******************
            //  As with DMA transfers, it's better to start the buffer at
            //  page boundary(4096 Bytes). If the buffers used for the 
            //  DMA operation is aligned on addresses of page boundary, user 
            //  can get exact page change event(just set the Count of 
            //  PT_EnableEvent to 2048, it means 1 page). Once Interrupt
            //  event occurs, it means 2048 samples(4096 byes) are available.
            //  
            //  It is why we changed the memory allocation policy from
            //  Solution A to solution B at Version 2.0B.
            //
            //  At the same time, the buffer that receives the samples must
            //  be a multiple of 4096 bypes(2048 samples). In cycle mode, if
            //  you are interested in buffer change event, the minimal buffer
            //  is 8192 bytes(4096 samples).
            //****************************************
           
            // Solution A:
            if((hUserBuf=(FLOAT far *)GlobalAlloc(GHND,
                sizeof(FLOAT) * gulConvNum)) == 0)
            {
                MessageBox(hWnd,"Not enough memory for buffer ",
                    "High Speed",MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }
            /*
            // Solution B:
            // Win9x seems don't support the routine well.
            // so it's recommended only to use it at WinNT/2K.
            hUserBuf = VirtualAlloc( NULL,                    // region to reserve or commit
                                     2 * gulConvNum,          // size of region
                                     MEM_COMMIT|MEM_RESERVE,  // type of allocation
                                     PAGE_READWRITE );        // type of access protection
            */
            if ( NULL==hUserBuf )                       
            {
                MessageBox( hWnd, "Not enough memory for buffer", "High Speed",MB_OK );
                DRV_DeviceClose((LONG *)&DriverHandle);
                return 0;
            }
            // End of solution B.

            if((hDisplayBuf=(FLOAT far *)GlobalAlloc(GHND,
                sizeof(FLOAT) * gulConvNum)) == 0)
            {
                MessageBox(hWnd,"Not enough memory for buffer ", "High Speed",MB_OK);
                VirtualFree(
                            hUserBuf,                   // base address of block
                            2 * gulConvNum,             // bytes of committed pages
                            MEM_DECOMMIT| MEM_RELEASE); // decommit the pages
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }
            ptFAITransfer.DataBuffer = (FLOAT far  *)GlobalLock(hDisplayBuf);           
          
            // Step 4: Enable event feature
            ptEnableEvent.EventType = 0;
            if( bBufferChange )
            {
                ptEnableEvent.EventType |= ADS_EVT_BUFCHANGE;
            }
            if( bTerminate )
            {
                ptEnableEvent.EventType |= ADS_EVT_TERMINATED;
            }
            if( bOverrun )
            {
                ptEnableEvent.EventType |= ADS_EVT_OVERRUN;
            }
            if( bInterrupt )
            {
                ptEnableEvent.EventType |= ADS_EVT_INTERRUPT;
            }
           
            ptEnableEvent.Enabled = gwEvtFlag;
            ptEnableEvent.Count   = (USHORT)(gulEventCount * 2048);

            if ((ErrCde = DRV_EnableEvent(DriverHandle,
                (LPT_EnableEvent)&ptEnableEvent)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                VirtualFree(
                            hUserBuf,                  
                            2 * gulConvNum,             
                            MEM_DECOMMIT| MEM_RELEASE); 
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }
          
            for (i=0 ; i < 16 ; i++)        
            {
                if (!gwGainList)
                {
                    gwGainCde[i] = DevFeatures.glGainList[gwGainCode].usGainCde;
                }
                else
                {
                    temp = gwGain[i];
                    gwGainCde[i] = DevFeatures.glGainList[temp].usGainCde;
                }
            }
            for(i=0; i<gwStopChl; i++)
            {
				temp = gwStartChl + i;
				if(temp>15)
					temp = temp - 16;

				ReorderGainCode[i] =  gwGainCde[temp];
            }

            // Step 5: Start DMA transfer
            ptFAIDmaExStart.TrigSrc			= gwExtTrig;
            ptFAIDmaExStart.TrigMode		= gwTrigMode;
            ptFAIDmaExStart.ClockSrc		= gwClockSrc;
            ptFAIDmaExStart.TrigEdge		= gwTrigEdge;
            ptFAIDmaExStart.SRCType			= gwSRCType;
            ptFAIDmaExStart.CyclicMode		= gwCyclicMode;
            ptFAIDmaExStart.TrigVol			= gwTrigVol;
            ptFAIDmaExStart.StartChan		= gwStartChl;
            ptFAIDmaExStart.NumChans		= gwStopChl;
            ptFAIDmaExStart.ulDelayCnt		= gwDelayCnt;
            ptFAIDmaExStart.SampleRate		= gdwPacerRate;
            ptFAIDmaExStart.GainList		= &ReorderGainCode[0];           
            ptFAIDmaExStart.count			= gulConvNum;
            ptFAIDmaExStart.buffer0			= (USHORT *)hUserBuf;

            if ((ErrCde = DRV_FAIDmaExStart(DriverHandle,
                (LPT_FAIDmaExStart)&ptFAIDmaExStart)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                VirtualFree(
                            hUserBuf,                  
                            2 * gulConvNum,             
                            MEM_DECOMMIT| MEM_RELEASE); 
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            gwCyclicCount = 0;

            if (gwEvtFlag)
            {
                hThreadHandle = CreateThread(0, 0,
                (LPTHREAD_START_ROUTINE) UserThread,
                0, 0, (LPDWORD)&dwThreadID);
			
			    bThreadloop = TRUE;
				bNoEventOccur = FALSE;	 
            }

            Sleep(0);

            EnableMenuItem(hMenu, IDM_RUN, MF_DISABLED | MF_BYCOMMAND);

            // Enable stop and status buttons
            if( gwCyclicMode )
            {
               EnableWindow( hwndStatusButton, TRUE );
               EnableWindow( hwndStopButton, TRUE );
            }
            else if ( !bTerminate )
            {
               EnableWindow( hwndStatusButton, TRUE );
               EnableWindow( hwndStopButton, TRUE );
            }
            else
            {
               EnableWindow( hwndStatusButton, TRUE );
               EnableWindow( hwndStopButton, TRUE );
            }

            return 0;
            //end IDM_RUN

        case IDC_STOPBUTTON:
        {
	         BOOL bflag = TRUE;
			 DWORD ExitCode;

			 EnableWindow(hwndStopButton, FALSE);
			 EnableWindow(hwndStatusButton, FALSE);
			 EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);
			 // Close thread
			 bThreadloop = FALSE;
             Sleep(100);
			 
			if( !gwExtTrig )
			{

				if(hThreadHandle)
				{
					GetExitCodeThread(hThreadHandle,&ExitCode);
		  		if(ExitCode==STILL_ACTIVE)
					TerminateThread(hThreadHandle,ExitCode);
				}
			}


			
            // Step 1: Stop A/D conversion for high speed
            if ((ErrCde = DRV_FAIStop(DriverHandle)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                VirtualFree(
                            hUserBuf,                  
                            2 * gulConvNum,             
                            MEM_DECOMMIT| MEM_RELEASE); 
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }


			// If no event occurs in thread, skip "Step 2".
			if(	!bNoEventOccur)	 
			{
				// Step 2: Get real voltage of Buffer from driver memory buffer
				ptFAITransfer.ActiveBuf = 0;   // single buffer
				ptFAITransfer.DataType = gwDataType;
				ptFAITransfer.start    = 0;
				ptFAITransfer.count    = gulConvNum;
				ptFAITransfer.overrun  = &gwOverrun;

				if ((ErrCde = DRV_FAITransfer(DriverHandle,
					(LPT_FAITransfer)&ptFAITransfer)) != 0)
				{
					DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
					MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
						GlobalUnlock(hDisplayBuf);
						GlobalFree(hDisplayBuf);
					VirtualFree(
								hUserBuf,                  
								2 * gulConvNum,             
								MEM_DECOMMIT| MEM_RELEASE); 
					DRV_DeviceClose((LONG far *)&DriverHandle);
					return 0;
				}
			
				// Display data
				DialogBox (hInstance, MAKEINTRESOURCE(IDD_DATA), hWnd, lpfnShowDataDlgProc) ;
			}

            // Step 3: Free buffer
            GlobalUnlock(hDisplayBuf);
            GlobalFree(hDisplayBuf);
            GlobalUnlock(hUserBuf);
            GlobalFree(hUserBuf);

            // Step 4: Close driver
            DRV_DeviceClose((LONG *)&DriverHandle);


            return 0;
			}

        case IDC_STATUSBUTTON:

            // Get DMA transfer status
            ptFAICheck.ActiveBuf = &gwActiveBuf;
            ptFAICheck.stopped   = &gwStopped;
            ptFAICheck.retrieved = &gulRetrieved;
            ptFAICheck.overrun   = &gwOverrun;
            ptFAICheck.HalfReady = &gwHalfReady;

            if ((ErrCde = DRV_FAICheck(DriverHandle,
                (LPT_FAICheck)&ptFAICheck)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                VirtualFree(
                            hUserBuf,                  
                            2 * gulConvNum,             
                            MEM_DECOMMIT| MEM_RELEASE); 
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }
            else
            {   // Display the status of the A/D conversion
                sprintf(szDest, "Cyclic Counts: %d\n", gwCyclicCount);
                if (gwHalfReady == 0)
                    strcpy(szTemp, "Buffer : not  ready\n");
                else if (gwHalfReady == 1)
                    strcpy(szTemp, "Buffer : half ready\n");
                else if (gwHalfReady == 2)
                    strcpy(szTemp, "Buffer : full ready\n");
                else
                    strcpy(szTemp, "Buffer : unknow \n");
                strcat(szDest, szTemp);
                ratio = 100.0 * gulRetrieved / gulConvNum;
                szStr = fcvt((double)ratio, 0, &dec, &sign);
                strcat(szDest, "Finish:  ");
                strcat(szDest, szStr);
                strcat(szDest, "%...");
                MessageBox(hWnd,(LPCSTR)szDest, "Check Status", MB_OK);
            }

            return 0;
        }

    case WM_DESTROY:

        PostQuitMessage(0);
        break;

    default:
        return (DefWindowProc(hWnd, message, wParam, lParam));
    }

    return ((LONG)NULL);
}

//------------------------------------------------------------------
void UserThread()
{
    USHORT usEventType;
    LONG  ErrCde;

    gulIntCounter = 0;
    gulBfChgCounter = 0;
    gulOverrunCounter = 0;

    hdc = GetDC( hWnd );
    GetClientRect( hWnd, &rect );
    InvalidateRect( hWnd, NULL, TRUE );
    UpdateWindow( hWnd );

    TextOut( hdc, 300, 15, (LPCSTR) "EVENT COUNTER", 13 );  
    TextOut( hdc, 300, 30, (LPCSTR) "--------------------------------------------------", 50 );  


    TextOut( hdc, 300, 135, (LPCSTR) "AI DATA", 7 );  
    TextOut( hdc, 300, 150, (LPCSTR) "--------------------------------------------------", 50 );  

    while(bThreadloop)
    {

        // Check message
        ptCheckEvent.EventType = &usEventType;
        if ( gulOverrunTime==0 )
        { 
            ptCheckEvent.Milliseconds = INFINITE;
        }
        else
        {
            ptCheckEvent.Milliseconds = gulOverrunTime;
        }

		  bThreadflag = TRUE;
        if ((ErrCde = DRV_CheckEvent(DriverHandle,
            (LPT_CheckEvent)&ptCheckEvent)) != 0)
        {
           MessageBox(hWnd,"Check Event Error !","Thread Message",MB_OK);
		
		   EnableWindow( hwndStopButton, TRUE );  

		   bThreadloop = FALSE;
		   bThreadflag = FALSE;
		   bNoEventOccur = TRUE;	 
			
        }

        // Process interrupt event
        if (usEventType & ADS_EVT_INTERRUPT)
        {
            adInterruptEvent();
        }
        // Process buffer change event
        if (usEventType & ADS_EVT_BUFCHANGE)
            adBufChangeEvent();

        // Process overrun event
        if (usEventType & ADS_EVT_OVERRUN)
        {
            adOverrunEvent();
        }

        // Process terminate event
        if (usEventType & ADS_EVT_TERMINATED)
        {
            adTerminateEvent();
            bThreadloop = FALSE;
            bThreadflag = FALSE;
            return;
        }

		  bThreadflag = FALSE; 
        sprintf( szCounter, "Interrupt Counter = %4d\n", gulIntCounter );
        TextOut( hdc, 300, 50, szCounter, 24 );  
        sprintf( szCounter, "Buffer change Counter = %4d\n", gulBfChgCounter );
        TextOut( hdc, 300, 70, szCounter, 28 );
        sprintf( szCounter, "Overrun Counter = %4d\n", gulOverrunCounter );
        TextOut( hdc, 300, 90, szCounter, 22 );


        UpdateWindow( hWnd );
    }

    bThreadflag = FALSE; 
    ReleaseDC( hWnd, hdc );

}

//-----------------------------------------------------------------------------
void adInterruptEvent( )
{
    gulIntCounter++;
    return;
}

//-----------------------------------------------------------------------------
void adBufChangeEvent()
{
   int i;

   gulBfChgCounter++;
   gwCyclicCount = gulBfChgCounter/2;

   // Get DMA transfer status
   ptFAICheck.ActiveBuf = &gwActiveBuf;
   ptFAICheck.stopped   = &gwStopped;
   ptFAICheck.retrieved = &gulRetrieved;
   ptFAICheck.overrun   = &gwOverrun;
   ptFAICheck.HalfReady = &gwHalfReady;

   if ((ErrCde = DRV_FAICheck(DriverHandle,
       (LPT_FAICheck)&ptFAICheck)) != 0)
   {
       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
       DRV_FAIStop( DriverHandle );
       DRV_DeviceClose((LONG *)&DriverHandle );
       GlobalFree( hUserBuf ); 
       return ;
   }
 
    // When call DRV_FAITransfer( ), the overrun event flag will be cleaned,
    // otherwise, the application will receive overrun event continously.
    // Get real voltage of buffer from driver
    ptFAITransfer.ActiveBuf = 0;   // single buffer
    ptFAITransfer.DataType = gwDataType;

    if ( 2==gwHalfReady )
    {
       ptFAITransfer.start = gulConvNum / 2;
    }
    else
    {
       ptFAITransfer.start = 0;
    }

    if ( gulConvNum >= 20 )
    {
       ptFAITransfer.count = 10; // For demo, we only get few data
    }
    else
    {
       ptFAITransfer.count = 1;  // For demo, we only get few data
    }
    ptFAITransfer.overrun  = &gwOverrun;

    if ((ErrCde = DRV_FAITransfer(DriverHandle,
        (LPT_FAITransfer)&ptFAITransfer)) != 0)
    {
        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
        MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
        return ;
    }

    for ( i=0; i<10; i++ )
    {
        if ( gwDataType )
        {
           sprintf( szData, "Buf[ %07d ]=%8.6f", i+ptFAITransfer.start, 
              ((float*)ptFAITransfer.DataBuffer)[i] );                                 
        }
        else
        {
           sprintf( szData, "Buf[ %07d ]=%   x", i+ptFAITransfer.start, 
              ((USHORT *)ptFAITransfer.DataBuffer)[i] );                                 
        }
        TextOut( hdc, 300, 170+ 20*i, szData, strlen(szData) );  
    }

    return;
}
//-----------------------------------------------------------------------------
void adOverrunEvent()
{
    gulOverrunCounter++; 
    DRV_ClearOverrun(  DriverHandle );
    return;
}
//-----------------------------------------------------------------------------
void adTerminateEvent()
{

	// Disable stop and status buttons
   EnableWindow(hwndStopButton, FALSE);
   EnableWindow(hwndStatusButton, FALSE);

	// Stop A/D conversion
   if ((ErrCde = DRV_FAIStop(DriverHandle)) != 0)
   {
      DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
      MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
      return ;
   }

   // Get real voltage of buffer from driver
   ptFAITransfer.ActiveBuf = 0;   // single buffer
   ptFAITransfer.DataType = gwDataType;
   ptFAITransfer.start    = 0;
   ptFAITransfer.count    = gulConvNum;
   ptFAITransfer.overrun  = &gwOverrun;

   if ((ErrCde = DRV_FAITransfer(DriverHandle,
       (LPT_FAITransfer)&ptFAITransfer)) != 0)
   {
       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
       MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
       return ;
   }

   // Display data
   DialogBox (hInstance, MAKEINTRESOURCE(IDD_DATA), hWnd, lpfnShowDataDlgProc) ;

   // Free buffer
   GlobalUnlock(hDisplayBuf);
   GlobalFree(hDisplayBuf);
   GlobalUnlock(hUserBuf);
   GlobalFree( hUserBuf);
   // Close driver
   DRV_DeviceClose((LONG far *)&DriverHandle);

   EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);
   return;
}


























