/*
 ***********************************************************************
 *  Program     : AIEXP.C                                              *
 *  Description : Demo program for analog input with expansion board   *
 *  Revision    : 1.00                                                 *
 *  Date        : 9/5/1996                     Advantech Co., Ltd.     *
 ***********************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance() - Saves instance handle and creates main window
 *
 *   MainWndProc() - Process main window messages
 *
 *   ConfigDlgProc() - Process the dialog box messages for configuration
 */


#include <windows.h>              /* required for all Windows applications */
#include <stdlib.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
// #include <toolhelp.h>
#include "..\..\..\include\driver.h"
#include "..\..\..\include\os.h"
#include "resource.h"

// ------------------------------------------------------------------------
// CONSTANT DEFINITION
// ------------------------------------------------------------------------
#define     MAX_DEVICES     100


// ------------------------------------------------------------------------
// GLOBAL VARIABLES
// ------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hMainWnd;                   // main window handle


static      DEVFEATURES     DevFeatures;    // structure for device features
static      DEVCONFIG_AI    DevCfg;         // structure for DEVCONFIG_AI table
static      PT_AIConfig     ptAIConfig;     // structure for AIConfig table
static      PT_AIVoltageIn  ptAIVoltageIn;  // structure for AIVoltageIn table
static      PT_AIVoltageInExp  ptAIVoltageInExp;  // structure for AIVoltageInExp table
static      PT_DeviceGetFeatures  ptDevFeatures;  // structure for DeviceGetFeatures
static      PT_AIGetConfig  ptAIGetConfig;  // structure for AIGetConfig table

char        szErrMsg[80];               // Use for MESSAGEBOX function
LRESULT     ErrCde;                     // Return error code

//
// used for dynamic allocation memory for the installed devices
//
// static  HGLOBAL     hDeviceList,hSubDeviceList;
// static  LPDEVLIST   DeviceList,SubDeviceList;
//

//
// used for fixed memory for the installed devices
//
DEVLIST     DeviceList[MAX_DEVICES];
DEVLIST     SubDeviceList[MAX_DEVICES];

LONG        DriverHandle = (LONG)NULL;          // driver handle
BOOL        bRun = FALSE;                       // flag for running
USHORT      gwDevice = 0, gwSubDevice = 0;      // Device index
USHORT      gwChannel = 0;                      // Input channel
USHORT      gwGain = 0;                         // Gain index
SHORT       gnNumOfDevices, gnNumOfSubdevices;  // number of installed devices
USHORT      gwScanTime = 1000;                  // scan time
DWORD       gdwStartTime;                       // start time
DWORD       gdwElapseTime = 0;                  // elapse time
USHORT      gwOverrunCount = 0;                 // overrun count
FARPROC     lpfnConfigDlgProc;                  // config. dialog procedure
FARPROC     lpfnScanDlgProc;                    // scan dialog procedure

USHORT      usExpChan = 0;                      // expansion channel
USHORT      usBoardID = 0;                      // Board ID
USHORT      usMaxChannel = 0;                   // max channel
CHAR        szExpName[15];                      // expansion name
CHAR        szBuf[64];                          // temp buffer


// ------------------------------------------------------------------------
// FUNCTION DECLARATION
// ------------------------------------------------------------------------
BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FTYPE MainWndProc(HWND, UINT, WPARAM, LPARAM);
int  FTYPE WinMain(HANDLE hInstance, HANDLE hPrevInstance,
     LPSTR lpCmdLine, int nCmdShow);


/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                          /* current instance             */
HANDLE hPrevInstance;                      /* previous instance            */
LPSTR lpCmdLine;                           /* command line                 */
int nCmdShow;                              /* show-window type (open/icon) */
{
    MSG msg;                               /* message                      */

    if (!hPrevInstance)                /* Other instances of app running?  */
      if (!InitApplication(hInstance)) /* Initialize shared things         */
          return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,      /* message structure                      */
        (HWND)NULL,              /* handle of window receiving the message */
        (UINT)NULL,              /* lowest message to examine              */
        (UINT)NULL))             /* highest message to examine             */
    {
        TranslateMessage(&msg);  /* Translates virtual key codes           */
        DispatchMessage(&msg);   /* Dispatches message to window           */
    }
    return (msg.wParam);         /* Returns the value from PostQuitMessage */
}

/****************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                            /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW; /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;  /* window process procedure    */
    wc.cbClsExtra = 0;                  /* No per-class extra data.        */
    wc.cbWndExtra = 0;                  /* No per-window extra data.       */
    wc.hInstance = hInstance;           /* Application that owns the class.*/
    wc.hIcon = LoadIcon(hInstance, "IDI_ICON1");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";   /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";  /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));

}

/***************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/

BOOL InitInstance
(
    HANDLE          hInstance,          // Current instance identifier.
    int             nCmdShow            // Param for first ShowWindow() call.
)
{
    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hMainWnd = CreateWindow(
        "MyClass",                 /* See RegisterClass() call.           */
        "Advantech Driver Demo : Software Data Transfer" , /* Window title bar  */
        WS_OVERLAPPEDWINDOW,        /* Window style.                       */
        CW_USEDEFAULT,              /* Default horizontal position.        */
        CW_USEDEFAULT,              /* Default vertical position.          */
        CW_USEDEFAULT,              /* Default width.                      */
        CW_USEDEFAULT,              /* Default height.                     */
        NULL,                       /* Overlapped windows have no parent.  */
        NULL,                       /* Use the window class menu.          */
        hInstance,                  /* This instance owns this window.     */
        NULL                        /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hMainWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hMainWnd, nCmdShow);   // Show the window
    UpdateWindow(hMainWnd);           // Sends WM_PAINT message
    return (TRUE);                // Returns the value from PostQuitMessage
}

/***************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
****************************************************************************/

BOOL FTYPE ConfigDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    int      i;
    DWORD       dwIndex;
    int         nOutEntries;

    switch (message)
    {
    case WM_INITDIALOG :

        // --------------------------------
        // Initialize Device List Combobox
        // --------------------------------

        // get number of the installed devices

        if ((ErrCde = DRV_DeviceGetNumOfList((SHORT far *)&gnNumOfDevices)) !=
            SUCCESS)
        {
             DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
             MessageBox(hMainWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
             return TRUE;
        }


        // ----------------------------------------------------------------
        // used for dynamic allocation memory for these installed devices
        //
        // if ((hDeviceList = GlobalAlloc(GHND, gnNumOfDevices *
        //      sizeof(DEVLIST))) == NULL)
        // {
        //    MessageBox(hMainWnd, (LPCSTR)"Not Enough Memory",
        //        "Memory Allocation", MB_OK);
        //    return TRUE;
        // }
        // DeviceList = (LPDEVLIST)GlobalLock(hDeviceList);
        // ----------------------------------------------------------------


        if (gnNumOfDevices > MAX_DEVICES)
            gnNumOfDevices = MAX_DEVICES;

        // retrieve the information of all installed devices

        if ((ErrCde = DRV_DeviceGetList((DEVLIST far *)&DeviceList[0],
            (SHORT)gnNumOfDevices, (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return TRUE;
        }

        // initialize the Device List Combobox with the retrieved information

        for (i = 0; i < gnNumOfDevices; i ++)
            SendDlgItemMessage(hDlg, IDC_DEVICE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)DeviceList[i].szDeviceName));

        SendDlgItemMessage(hDlg, IDC_DEVICE, CB_SETCURSEL,(WPARAM)gwDevice,
                        (LPARAM)0);


        // -----------------------------------------------------------
        // Initialize Module List Combobox for COM port or CAN devices
        // -----------------------------------------------------------

        // check if there is any device attached on this COM port or CAN

        gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
        if (gnNumOfSubdevices > MAX_DEVICES)
            gnNumOfSubdevices = MAX_DEVICES;


        // --------------------------------------------------------------
        // used for dynamic allocation memory for these installed devices
        //
        // if ((hSubDeviceList = GlobalAlloc(GHND, gnNumOfSubdevices *
        //    sizeof(DEVLIST))) == NULL)
        // {
        //    MessageBox(hMainWnd, (LPCSTR)"Not Enough Memory",
        //        "Memory Allocation", MB_OK);
        //    return TRUE;
        // }
        //
        // SubDeviceList = (LPDEVLIST)GlobalLock(hSubDeviceList);
        // --------------------------------------------------------------


        // retrieve the information of all installed devices
        if (gnNumOfSubdevices != 0)
        {
            if ((ErrCde = DRV_DeviceGetSubList(
                (DWORD)DeviceList[gwDevice].dwDeviceNum,
                (DEVLIST far *)&SubDeviceList[0],
                (SHORT)gnNumOfSubdevices,
                (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                return TRUE;
            }


            // initialize the Module List Combobox with the retrieved
            // information

            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));

            for (i = 0; i < gnNumOfSubdevices; i++)
                SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

            SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
					(WPARAM)gwSubDevice, (LPARAM)0);
        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));
        }

        // ------------------------------------------------------------
        // Initialize Input Range List Combobox, it needs to get device
        // features for gain list
        // ------------------------------------------------------------

        // first : Open Device

        if (gnNumOfSubdevices == 0)
        {
            ErrCde = DRV_DeviceOpen(DeviceList[gwDevice].dwDeviceNum,
               (LONG far *)&DriverHandle);
        }
        else
        {
            ErrCde = DRV_DeviceOpen(
                SubDeviceList[gwSubDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);
        }

        if (ErrCde != SUCCESS)
        {
            strcpy(szErrMsg,"Device open error !");
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }

        // second: get device features

        ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
        ptDevFeatures.size = sizeof(DEVFEATURES);
        if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
            (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return 0;
        }

        // Have AI function support ?
        if ((DevFeatures.usMaxAIDiffChl   == 0) &&
            (DevFeatures.usMaxAISiglChl == 0))
        {
            EnableWindow(GetDlgItem(hDlg, IDC_ECHANNEL), FALSE);
            SendDlgItemMessage(hDlg, IDC_ECHANNEL, CB_RESETCONTENT,
                0, (LPARAM)((LPSTR)0));
            EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), FALSE);
            SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT,
                0, (LPARAM)((LPSTR)0));
            EnableWindow(GetDlgItem(hDlg, IDC_BOARDID), FALSE);
            SendDlgItemMessage(hDlg, IDC_BOARDID, CB_RESETCONTENT,
                0, (LPARAM)((LPSTR)0));
        }
        else
        {
            ptAIGetConfig.buffer = (LPDEVCONFIG_AI)&DevCfg;
            ptAIGetConfig.size = sizeof(DEVCONFIG_AI);

            // call AIGetConfig
            if ((ErrCde = DRV_AIGetConfig(DriverHandle,
                (LPT_AIGetConfig)&ptAIGetConfig)) != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            usMaxChannel = DevFeatures.usMaxAISiglChl;

            EnableWindow(GetDlgItem(hDlg, IDC_ECHANNEL), TRUE);
            EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), TRUE);
            SendDlgItemMessage(hDlg, IDC_ECHANNEL, CB_RESETCONTENT, 0, 0L);
            SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT, 0, 0L);

            // setting dialog box for channel setting
            // fill in the channel listbox with the available channel
            // in the AI subsection (start with channel 0)
            for (i=0; i< (int)usMaxChannel; ++i)
            {
                wsprintf(szBuf, "%d", i);
                if (i == (int)DevCfg.usCjcChannel) // CJC channel
                {
                    lstrcat(szBuf, " (CJC Channel)");
                }
                else if (DevCfg.Daughter[i&0xf].dwBoardID!=0L)// exp. board exist
                {
                    lstrcat(szBuf, " (");
                    DRV_BoardTypeMapBoardName(DevCfg.Daughter[i&0xf].dwBoardID,
                                              (LPSTR)szExpName);
                    lstrcat(szBuf, szExpName);
                    lstrcat(szBuf, ")");
                }
                SendDlgItemMessage(hDlg, IDC_ECHANNEL, CB_ADDSTRING,(WPARAM) -1,
                    (LPARAM)(LPSTR)szBuf);
            }
            // initialize input channel
            SendDlgItemMessage(hDlg, IDC_ECHANNEL, CB_SETCURSEL, gwChannel,0L);

            // if the selected channel is associated with an expansion board,
            // fill in the Exp. channel listbox with the available expansion board
            // channel.
            if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[gwChannel&0xf].dwBoardID
                && (int)gwChannel != (int)DevCfg.usCjcChannel &&
                DevCfg.Daughter[gwChannel&0xf].dwBoardID != (DWORD)BD_PCLD8115)
			{
                EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), TRUE);

                for (i=0; i<(int)DevCfg.Daughter[gwChannel&0xf].usNum; ++i)
				{
					wsprintf(szBuf, "%d", i);
                    SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_ADDSTRING,(WPARAM)-1,
						(LPARAM)(LPSTR)szBuf);
				}

                SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_SETCURSEL,
                    usExpChan,0L);
			}
			else
			{
                usExpChan = 0;
                EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), FALSE);
                SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT, 0, 0L);
			}

            if (DevCfg.Daughter[gwChannel&0xf].dwBoardID == (DWORD)BD_PCLD788)
            {
                EnableWindow(GetDlgItem(hDlg, IDC_BOARDID), TRUE);

                for (i=0; i<(int)DevCfg.Daughter[gwChannel&0xf].usCards; ++i)
                {
                    wsprintf(szBuf, "%d", i);
                    SendDlgItemMessage(hDlg, IDC_BOARDID, CB_ADDSTRING, (WPARAM)-1,
                        (LPARAM)(LPSTR)szBuf);
                }
                SendDlgItemMessage(hDlg, IDC_BOARDID, CB_SETCURSEL,
                    usBoardID,0L);
            }
            else
            {
                usBoardID = 0;
                EnableWindow(GetDlgItem(hDlg,IDC_BOARDID),FALSE);
                SendDlgItemMessage(hDlg, IDC_BOARDID, CB_RESETCONTENT, 0, 0L);
            }

            // third: initialize Input Range List Combobox with device features
            if ((DevFeatures.usNumGain != 0) &&
                (DevCfg.Daughter[gwChannel&0xf].dwBoardID == 0) &&
                (DevCfg.usCjcChannel != gwChannel))
            {
                EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
                SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                    0, (LPARAM)((LPSTR)0));
                for (i = 0; i < (int)DevFeatures.usNumGain; i ++)
                    SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)(DevFeatures.glGainList[i].szGainStr)));

                SendDlgItemMessage(hDlg, IDC_INPRANGE,
                   CB_SELECTSTRING, (WPARAM)(-1),
                   (LPARAM)((LPSTR)(DevFeatures.glGainList[gwGain].szGainStr)));
            }
            else
            {
                gwGain = 0;
                EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
                SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                    0, (LPARAM)((LPSTR)0));
            }
        }


        // fourth: close device

        DRV_DeviceClose((LONG far *)&DriverHandle);

        return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
         {
            case IDOK :

              //
              // get device selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwDevice = (WORD)dwIndex;


              //
              // get sub-device selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_MODULE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwSubDevice = (WORD)dwIndex;


              //
              // get channel number
              //
              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_ECHANNEL,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwChannel = (WORD)dwIndex;
              //
              // get input range selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_INPRANGE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwGain = (WORD)dwIndex;

              //
              // get expansion channel number
              //
              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_EXPCHAN,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   usExpChan = (WORD)dwIndex;

              //
              // get board id for PCLD-788
              //
              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_BOARDID,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   usBoardID = (WORD)dwIndex;

              EndDialog(hDlg, 0);
              return TRUE;


            case IDCANCEL :
                 EndDialog(hDlg, 0);
                 return TRUE;

            case IDC_ECHANNEL :

                //
                // When device selection is changed, it needs to
                // re-initialize sub-device combobox and input range
                // combobox.
                //

                if (HIWORD(wParam) == CBN_SELCHANGE)
                {
                  gwChannel = 0;
                  gwGain = 0;
                  usExpChan = 0;
                  usBoardID = 0;

                  if ((dwIndex = SendDlgItemMessage(hDlg, IDC_ECHANNEL,
                               CB_GETCURSEL, 0, 0)) != CB_ERR)
                       gwChannel = (WORD)dwIndex;

                  // if the selected channel is associated with an expansion board,
                  // fill in the Exp. channel listbox with the available expansion board
                  // channel.
                  if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[gwChannel&0xf].dwBoardID
                      && (int)gwChannel != (int)DevCfg.usCjcChannel &&
                      DevCfg.Daughter[gwChannel&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                  {
                      EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), TRUE);
                      for (i=0; i<(int)DevCfg.Daughter[gwChannel&0xf].usNum; ++i)
                      {
                          wsprintf(szBuf, "%d", i);
                          SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_ADDSTRING,(WPARAM)-1,
                              (LPARAM)(LPSTR)szBuf);
                      }

                      SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_SETCURSEL,
                          usExpChan,0L);
                  }
                  else
                  {
                      EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), FALSE);
                      SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT, 0, 0L);
                  }

                  if (DevCfg.Daughter[gwChannel&0xf].dwBoardID == (DWORD)BD_PCLD788)
                  {
                      EnableWindow(GetDlgItem(hDlg, IDC_BOARDID), TRUE);
                      for (i=0; i<(int)DevCfg.Daughter[gwChannel&0xf].usCards; ++i)
                      {
                          wsprintf(szBuf, "%d", i);
                          SendDlgItemMessage(hDlg, IDC_BOARDID, CB_ADDSTRING, (WPARAM)-1,
                              (LPARAM)(LPSTR)szBuf);
                      }
                      SendDlgItemMessage(hDlg, IDC_BOARDID, CB_SETCURSEL,
                          usBoardID,0L);
                  }
                  else
                  {
                      EnableWindow(GetDlgItem(hDlg,IDC_BOARDID),FALSE);
                      SendDlgItemMessage(hDlg, IDC_BOARDID, CB_RESETCONTENT, 0, 0L);
                  }

                  // third: initialize Input Range List Combobox with device features
                  if ((DevFeatures.usNumGain != 0) &&
                      (DevCfg.Daughter[gwChannel&0xf].dwBoardID == 0) &&
                      (DevCfg.usCjcChannel != gwChannel))
                  {
                      EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
                      for (i = 0; i < (int)DevFeatures.usNumGain; i ++)
                          SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_ADDSTRING, 0,
                              (LPARAM)((LPSTR)(DevFeatures.glGainList[i].szGainStr)));

                      SendDlgItemMessage(hDlg, IDC_INPRANGE,
                         CB_SELECTSTRING, (WPARAM)(-1),
                         (LPARAM)((LPSTR)(DevFeatures.glGainList[gwGain].szGainStr)));
                  }
                  else
                  {
                      EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
                      SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                          0, (LPARAM)((LPSTR)0));
                  }

                }

              return TRUE;

            case IDC_DEVICE :

                //
                // When device selection is changed, it needs to
                // re-initialize sub-device combobox and input range
                // combobox.
                //

                if (HIWORD(wParam) == CBN_SELCHANGE)
                {
                    if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                        CB_GETCURSEL, 0, 0)) != CB_ERR)
                         gwDevice = (WORD)dwIndex;
                    else
                        return TRUE;


                    // ------------------------------------------------------
                    // Initialize Module Combobox for COM port or CAN devices
                    // ------------------------------------------------------

                    // check any device attached on this COM port or CAN

                    gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
                    if (gnNumOfSubdevices > MAX_DEVICES)
                        gnNumOfSubdevices = MAX_DEVICES;


                    // retrieve the information of all installed devices

                    if (gnNumOfSubdevices != 0)
                    {
                        if ((ErrCde = DRV_DeviceGetSubList(
                            (DWORD)DeviceList[gwDevice].dwDeviceNum,
                            (DEVLIST far *)&SubDeviceList[0],
                            (SHORT)gnNumOfSubdevices,
                            (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
                        {
                            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                            MessageBox(hMainWnd, (LPCSTR)szErrMsg,
                                "Driver Message", MB_OK);
                            return TRUE;
                        }


                        // initialize the Module List Combobox with the
                        // retrieved information

                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));

                        for (i = 0; i < gnNumOfSubdevices; i++)
                            SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                                (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

                        gwSubDevice = 0;

                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
							(WPARAM)gwSubDevice, (LPARAM)0);
                    }
                    else
                    {
                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                    }


                    // -------------------------------------------------
                    // Initialize Input Range List Combobox, it needs to
                    // get device features for gain list
                    // -------------------------------------------------

                    // first : Open Device

                    // for non COM port or CAN device
                    if (gnNumOfSubdevices == 0)
                    {
                        ErrCde = DRV_DeviceOpen(
                            DeviceList[gwDevice].dwDeviceNum,
                            (LONG far *)&DriverHandle);
                    }

                    // for COM port or CAN device
                    else
                    {
                        ErrCde = DRV_DeviceOpen(
                            SubDeviceList[gwSubDevice].dwDeviceNum,
                            (LONG far *)&DriverHandle);
                    }

                    if (ErrCde != SUCCESS)
                    {
                        strcpy(szErrMsg,"Device open error !");
                        MessageBox(hMainWnd, (LPCSTR)szErrMsg,
                            "Driver Message", MB_OK);
                        return 0;
                    }

                    // second: get device features

                    ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
                    ptDevFeatures.size = sizeof(DEVFEATURES);
                    if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                        (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hMainWnd, (LPCSTR)szErrMsg, "Driver Message",
                            MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    // initialize value
                    gwChannel = 0;
                    gwGain    = 0;
                    usBoardID = 0;
                    usExpChan = 0;

                    // Have AI function support ?
                    if ((DevFeatures.usMaxAIDiffChl == 0) &&
                        (DevFeatures.usMaxAISiglChl == 0))
                    {
                        EnableWindow(GetDlgItem(hDlg, IDC_ECHANNEL), FALSE);
                        SendDlgItemMessage(hDlg, IDC_ECHANNEL, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                        EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
                        SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                        EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), FALSE);
                        SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                        EnableWindow(GetDlgItem(hDlg, IDC_BOARDID), FALSE);
                        SendDlgItemMessage(hDlg, IDC_BOARDID, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                    }
                    else
                    {
                        ptAIGetConfig.buffer = (LPDEVCONFIG_AI)&DevCfg;
                        ptAIGetConfig.size = sizeof(DEVCONFIG_AI);

                        // call AIGetConfig
                        if ((ErrCde = DRV_AIGetConfig(DriverHandle,
                            (LPT_AIGetConfig)&ptAIGetConfig)) != SUCCESS)
                        {
                            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                            DRV_DeviceClose((LONG far *)&DriverHandle);
                            return 0;
                        }
                       usMaxChannel = DevFeatures.usMaxAISiglChl;

                        EnableWindow(GetDlgItem(hDlg, IDC_ECHANNEL), TRUE);
                        EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), TRUE);
                        EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
                        EnableWindow(GetDlgItem(hDlg, IDC_BOARDID), TRUE);
                        SendDlgItemMessage(hDlg, IDC_ECHANNEL, CB_RESETCONTENT, 0, 0L);
                        SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT, 0, 0L);
                        SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT, 0, 0L);
                        SendDlgItemMessage(hDlg, IDC_BOARDID, CB_RESETCONTENT, 0, 0L);

                        // setting dialog box for channel setting
                        // fill in the channel listbox with the available channel
                        // in the AI subsection (start with channel 0)
                        for (i=0; i< (int)usMaxChannel; ++i)
                        {
                            wsprintf(szBuf, "%d", i);
                            if (i == (int)DevCfg.usCjcChannel) // CJC channel
                            {
                                lstrcat(szBuf, " (CJC Channel)");
                            }
                            else if (DevCfg.Daughter[i&0xf].dwBoardID!=0L)// exp. board exist
                            {
                                lstrcat(szBuf, " (");
                                DRV_BoardTypeMapBoardName(DevCfg.Daughter[i&0xf].dwBoardID,
                                                          (LPSTR)szExpName);
                                lstrcat(szBuf, szExpName);
                                lstrcat(szBuf, ")");
                            }
                            SendDlgItemMessage(hDlg, IDC_ECHANNEL, CB_ADDSTRING,(WPARAM) -1,
                                (LPARAM)(LPSTR)szBuf);
                        }
                        // initialize input channel
                        SendDlgItemMessage(hDlg, IDC_ECHANNEL, CB_SETCURSEL, gwChannel,0L);

                        // if the selected channel is associated with an expansion board,
                        // fill in the Exp. channel listbox with the available expansion board
                        // channel.
                        if (DevCfg.usNumExpChan > 0 && DevCfg.Daughter[gwChannel&0xf].dwBoardID
                            && (int)gwChannel != (int)DevCfg.usCjcChannel &&
                            DevCfg.Daughter[gwChannel&0xf].dwBoardID != (DWORD)BD_PCLD8115)
                        {
                            EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), TRUE);
                            SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT, 0, 0L);

                            for (i=0; i<(int)DevCfg.Daughter[gwChannel&0xf].usNum; ++i)
                            {
                                wsprintf(szBuf, "%d", i);
                                SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_ADDSTRING,(WPARAM)-1,
                                    (LPARAM)(LPSTR)szBuf);
                            }

                            SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_SETCURSEL,
                                usExpChan,0L);
                        }
                        else
                        {
                            EnableWindow(GetDlgItem(hDlg, IDC_EXPCHAN), FALSE);
                            SendDlgItemMessage(hDlg, IDC_EXPCHAN, CB_RESETCONTENT, 0, 0L);
                        }

                        if (DevCfg.Daughter[gwChannel&0xf].dwBoardID == (DWORD)BD_PCLD788)
                        {
                            EnableWindow(GetDlgItem(hDlg, IDC_BOARDID), TRUE);
                            SendDlgItemMessage(hDlg, IDC_BOARDID, CB_RESETCONTENT, 0, 0L);

                            for (i=0; i<(int)DevCfg.Daughter[gwChannel&0xf].usCards; ++i)
                            {
                                wsprintf(szBuf, "%d", i);
                                SendDlgItemMessage(hDlg, IDC_BOARDID, CB_ADDSTRING, (WPARAM)-1,
                                    (LPARAM)(LPSTR)szBuf);
                            }
                            SendDlgItemMessage(hDlg, IDC_BOARDID, CB_SETCURSEL,
                                usBoardID,0L);
                        }
                        else
                        {
                            EnableWindow(GetDlgItem(hDlg,IDC_BOARDID),FALSE);
                            SendDlgItemMessage(hDlg, IDC_BOARDID, CB_RESETCONTENT, 0, 0L);
                        }

                        // third: initialize Range Combobox with device features
                        if ((DevFeatures.usNumGain != 0) &&
                            (DevCfg.Daughter[gwChannel&0xf].dwBoardID == 0) &&
                            (DevCfg.usCjcChannel != gwChannel))
                        {
                            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), TRUE);
                            SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                                0, (LPARAM)((LPSTR)0));
                            for (i = 0; i < (int)DevFeatures.usNumGain; i ++)
                                SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_ADDSTRING, 0,
                                (LPARAM)((LPSTR)(DevFeatures.glGainList[i].szGainStr)));

                            SendDlgItemMessage(hDlg, IDC_INPRANGE,
                                CB_SELECTSTRING, (WPARAM)(-1),
                                (LPARAM)((LPSTR)(DevFeatures.glGainList[gwGain].szGainStr)));
                        }
                        else
                        {
                            EnableWindow(GetDlgItem(hDlg, IDC_INPRANGE), FALSE);
                            SendDlgItemMessage(hDlg, IDC_INPRANGE, CB_RESETCONTENT,
                                0, (LPARAM)((LPSTR)0));
                        }
                    }

                    // fourth: close device
                    DRV_DeviceClose((LONG far *)&DriverHandle);
                }

                return TRUE;
         }
         break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: ScanDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ScanDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    char        szBuffer[40];

    switch (message)
    {
    case WM_INITDIALOG :

        // ---------------------------------------------------
        // Initialize Scan Time Edit, Maximum accuracy: 40 ms
        // ---------------------------------------------------

        itoa(gwScanTime, szBuffer, 10);
        SendDlgItemMessage(hDlg, IDC_SCAN, EM_REPLACESEL, 0,
                        (LPARAM)((LPSTR)szBuffer));

        return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
         {
            case IDOK :

                //
                // get scan time
                //

                if (SendDlgItemMessage(hDlg, IDC_SCAN,
                           EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_SCAN,
                            WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwScanTime = atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_SCAN,
                              EM_SETMODIFY, FALSE, 0) ;
                }

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}


/***************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
****************************************************************************/

long FTYPE MainWndProc
(
    HWND        hWnd,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    static  HANDLE  hInstance ;
    HMENU           hMenu;
    HDC             hdc;
    char            szBuffer[120];
    RECT            rect;
    float           fVoltage;
    DWORD           dwCurrentTime;
    DWORD           dwErrorTime;
    BOOL            bTimerOverrun;

    switch (message)
    {
        case WM_CREATE:

            //
            // initialize Stop Menu
            //

            hMenu = GetMenu(hWnd);
            EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);

            hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

            lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

            lpfnScanDlgProc = MakeProcInstance (ScanDlgProc, hInstance) ;

            return 0 ;

        case WM_COMMAND:     /* message: from application menu */

            hMenu = GetMenu(hWnd);

            switch (LOWORD(wParam))
            {
                case IDM_SETTING :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                         hWnd, lpfnConfigDlgProc) ;
                    return 0;

                case IDM_SCAN :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SCAN),
                         hWnd, lpfnScanDlgProc) ;
                    return 0;

                case IDM_STOP :

                    //
                    // kill timer and close driver
                    //

                    KillTimer(hWnd, 1);
                    DRV_DeviceClose((LONG far *)&DriverHandle);

                    bRun = FALSE;

                    //
                    // reset menu item status
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_ENABLED);

                    return 0;

                case IDM_START :

                    //
                    // Open Device
                    //

                    if (gnNumOfSubdevices == 0)
                        ErrCde = DRV_DeviceOpen(
                            DeviceList[gwDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);
                    else
                        ErrCde = DRV_DeviceOpen(
                            SubDeviceList[gwSubDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);

                    if (ErrCde != SUCCESS)
                    {
                        strcpy(szErrMsg,"Device open error !");
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                        return 0;
                    }

                    //
                    // get gain code
                    //

                    ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
                    ptDevFeatures.size = sizeof(DEVFEATURES);
                    if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                        (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    //
                    // configures the gain for the specifed analog input channel
                    //

                    if(DevCfg.Daughter[gwChannel&0xf].dwBoardID == 0)
                    {
                       ptAIConfig.DasGain = DevFeatures.glGainList[gwGain].usGainCde;
                       ptAIConfig.DasChan = gwChannel;

                       if ((ErrCde = DRV_AIConfig(DriverHandle,
                            (LPT_AIConfig)&ptAIConfig)) != 0)
                       {
                           DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                           MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                           DRV_DeviceClose((LONG far *)&DriverHandle);
                           return 0;
                       }
                    }
                    //
                    // enable Stop Menu item and disable the others
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_ENABLED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_GRAYED);

                    //
                    // Set the new timer type and start it up.
                    //

                    if (SetTimer(hWnd, 1, gwScanTime, NULL) == (UINT)NULL)
                    {
                        MessageBox(hWnd, "Couldn't create timer.", "Notice!",
                            MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    bRun = TRUE;

                    gdwStartTime = GetTickCount();
                    gwOverrunCount = 0;

                    return 0;
            }
            return 0;

        case WM_TIMER:

            //
            // Timer set to dispatch a WM_TIMER message has gone off.
            //

            //
            // check if timer is overrun
            //

            dwCurrentTime = GetTickCount();

            gdwElapseTime = gdwElapseTime + (DWORD)gwScanTime;
            dwErrorTime = dwCurrentTime - gdwElapseTime - gdwStartTime;
            if (dwErrorTime > (DWORD)40)        // maximum 40 ms scan time
            {
                gdwElapseTime = dwCurrentTime - gdwStartTime;
                gwOverrunCount ++;
                if (gwOverrunCount >= 10)
                {
                    bTimerOverrun = TRUE;
                    gwOverrunCount = 0;
                }
                else
                    bTimerOverrun = FALSE;
            }
            else
            {
                bTimerOverrun = FALSE;
                gwOverrunCount = 0;
            }

            //
            // reads an analog input channel
            //

            if(DevCfg.Daughter[gwChannel&0xf].dwBoardID == 0)
            {
               ptAIVoltageIn.chan = gwChannel;
               ptAIVoltageIn.gain = ptAIConfig.DasGain;
               ptAIVoltageIn.TrigMode = 0;               // internal trigger
               ptAIVoltageIn.voltage = (FLOAT far *)&fVoltage;

               if ((ErrCde = DRV_AIVoltageIn(DriverHandle,
                          (LPT_AIVoltageIn)&ptAIVoltageIn)) != 0)
               {
                   DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                   MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                   KillTimer(hWnd, 1);
                   DRV_DeviceClose((LONG far *)&DriverHandle);
                   return 0;
               }

            }
            else
            {
               ptAIVoltageInExp.DasChan = gwChannel;
               ptAIVoltageInExp.DasGain = gwGain;
               ptAIVoltageInExp.ExpChan = usExpChan;
               ptAIVoltageInExp.voltage = (FLOAT far *)&fVoltage;

               if  (DevCfg.Daughter[gwChannel&0xf].dwBoardID == BD_PCLD788)
                  ptAIVoltageInExp.ExpChan = ((usBoardID << 4) | usExpChan);

               if ((ErrCde = DRV_AIVoltageInExp(DriverHandle,
                          (LPT_AIVoltageInExp)&ptAIVoltageInExp)) != 0)
               {
                   DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                   MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                   KillTimer(hWnd, 1);
                   DRV_DeviceClose((LONG far *)&DriverHandle);
                   return 0;
               }

            }
            //
            // Display Data
            //

            hdc = GetDC(hWnd);
            GetClientRect(hWnd, &rect);
            InvalidateRect(hWnd, NULL, TRUE);
            UpdateWindow(hWnd);
            if (bTimerOverrun == FALSE)
                sprintf(szBuffer, "data = %10.6f", fVoltage);
            else
                sprintf(szBuffer, "Timer Overrun!\n data = %10.6f", fVoltage);
            DrawText(hdc, szBuffer, -1, &rect,
                    DT_SINGLELINE | DT_CENTER | DT_VCENTER);
            ReleaseDC(hWnd, hdc);

            return 0;

        case WM_DESTROY:
            if (bRun == TRUE)
            {
                KillTimer(hWnd, 1);
                DRV_DeviceClose((LONG far *)&DriverHandle);
            }
            PostQuitMessage(0);
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }

    return ((LONG)NULL);
}

