/*
 ************************************************************************
 *  Program     : DADMA.C                                               *
 *  Description : Demo program for Windows with DMA data transfer       *
 *  Revision    : 1.00                                                  *
 *  Date        : 12/10/1997                  Advantech Co., Ltd.       *
 ************************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance() - Saves instance handle and creates main window
 *
 *   MainWndProc() - Process main window messages
 *
 *   ConfigDlgProc() - Process the dialog box messages for configuration
 *
 *   WaveDlgProc() - Process the dialog box messages for waveform setting
 */


#include <windows.h>              /* required for all Windows applications */
#include <stdlib.h>
#include <windowsx.h>
#include <commdlg.h>
#include <string.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\..\include\driver.h"
#include "..\..\..\..\include\os.h"
#include "resource.h"
#include "wdadmabm.h"

typedef struct {
    int wWaveform;
    float fMagnitude;
    float fOffset;
    int wPeriod;
} SWAVE, FAR *LPSWAVE;

// ------------------------------------------------------------------------
// GLOBAL VARIABLES
// ------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hWnd;                       // main window handle

static      DEVFEATURES        DevFeatures;       // structure for device features
static      PT_DeviceGetFeatures  ptDevFeatures;  // Devfeatures table
static      PT_FAOWaveFormStart     ptFAOWaveFormStart;     // FAODMAStart table
static      PT_FAOScale        ptFAOScale;        // FAOScale table
static      PT_EnableEvent     ptEnableEvent;     // Enable event
static      PT_CheckEvent      ptCheckEvent;      // Check event
static		PT_FAOCheck		   ptFAOCheck;


static  USHORT gwActiveBuf = 0;     // return by FAOCheck
static  USHORT gwOverrun   = 0;     // return by FAOCheck
static  USHORT gwStopped   = 0;     // return by FAOCheck
static  ULONG  ulCurrentCount = 0;     // return by FAOCheck
static  USHORT gwHalfReady = 0;     // return by FAOCheck

HANDLE      hEvent;
HANDLE      hThreadHandle;
DWORD       dwThreadID;
BOOL        bThreadloop = FALSE;
BOOL        bThreadflag = FALSE;

char        szErrMsg[80];               // Use for MESSAGEBOX function
char        szBuffer[40];               // Temperatory buffer
char        szDescript[100];            // Description device
LRESULT     ErrCde;                     // Return error code

static  char far szWaveform[5][15] = {"SINE","TRIANGLE","SQUARE","SINETRIANGLE","None"};

static  LONG   DriverHandle = (LONG)NULL;         // driver handle
static  ULONG  dwDeviceNum;                       // Device number
static  USHORT gwSelectDevice = FALSE;  // have never selected "Select Device" button
static  USHORT gwDevice = 0;            // device index
static  DWORD  gdwPacerRate = 10000;    // pacer rate
static  ULONG  gulConvNum = 2048;      // conversion number
static  ULONG gwWaveFormCount = 30;  // scan channels
static  USHORT gwExtTrig = 0;           // external or internal trigger
static  USHORT gwCyclicMode = 1;        // cyclic or non-cylic mode
static  USHORT gwEvtFlag = 1;           // event enable(0)
static	BOOL   bRunTimer = FALSE;

static  USHORT  usEnabledChannel=0;
static  USHORT	usChannelCount=0;



HGLOBAL hBuf[4], hVoltageBuf[4];
FLOAT  far * lpVoltageBuf[4]={NULL,NULL,NULL,NULL};
USHORT far * lpBuf[4]={NULL,NULL,NULL,NULL};

HGLOBAL			hCommonBuf;
USHORT far *	lpCommonBuf;	

static  SWAVE sWaveCh0 = {0,          // output waveform A index
                        2.00f,      // the magnitude of output waveform A
                        2.00f,      // the offset of output waveform A
                        2048},       // the points of one period
              sWaveCh1 = {1,          // output waveform B index
                        1.00f,      // the magnitude of output waveform B
                        3.00f,      // the offset of output waveform B
                        2048},       // the points of one period
              sWaveCh2 = {2,          // output waveform C index
                        2.00f,      // the magnitude of output waveform C
                        2.00f,      // the offset of output waveform C
                        2048},       // the points of one period
              sWaveCh3 = {3,          // output waveform C index
                        2.00f,      // the magnitude of output waveform C
                        2.00f,      // the offset of output waveform C
                        2048};       // the points of one period


static  HWND hwndStopButton, hwndStatusButton;
static  HANDLE  hInstance ;
HMENU hMenu;
//LONG pBuffer;


BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FTYPE MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
int  FTYPE WinMain(HANDLE hInstance, HANDLE hPrevInstance,
    LPSTR lpCmdLine, int nCmdShow);
static  FARPROC lpfnConfigDlgProc, lpfnWaveDlgProc;
void SetRealBuffer(float far *lpBuf, long num, LPSWAVE lpWave);
void SetMultiToOneBuffer(USHORT usEnabledChannel, int count);

void UserThread();
void adInterruptEvent();
void adBufChangeEvent();
void adOverrunEvent();
void adTerminateEvent();
void MyFreeBuffer();


/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                          /* current instance             */
HANDLE hPrevInstance;                      /* previous instance            */
LPSTR lpCmdLine;                           /* command line                 */
int nCmdShow;                              /* show-window type (open/icon) */
{
    MSG msg;                               /* message                      */

    if (!hPrevInstance)                /* Other instances of app running?  */
      if (!InitApplication(hInstance)) /* Initialize shared things         */
          return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,      /* message structure                      */
        (HWND)NULL,                    /* handle of window receiving the message */
        (UINT)NULL,                    /* lowest message to examine              */
        (UINT)NULL))                   /* highest message to examine             */
    {
        TranslateMessage(&msg);  /* Translates virtual key codes           */
        DispatchMessage(&msg);   /* Dispatches message to window           */
    }

    return (msg.wParam);         /* Returns the value from PostQuitMessage */
}

/****************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                            /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW; /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;  /* window process procedure    */
    wc.cbClsExtra = 0;                  /* No per-class extra data.        */
    wc.cbWndExtra = 0;                  /* No per-window extra data.       */
    wc.hInstance = hInstance;           /* Application that owns the class.*/
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";   /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";  /* Name used in call to CreateWindow. */


    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));

}

/***************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
HANDLE          hInstance;          /* Current instance identifier.          */
int             nCmdShow;           /* Param for first ShowWindow() call.    */
{

    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hWnd = CreateWindow(
        "MyClass",                 /* See RegisterClass() call.           */
        "Advantech Driver Demo : Analog output with bus master DMA" , /* Window title bar  */
        WS_OVERLAPPEDWINDOW,        /* Window style.                       */
        CW_USEDEFAULT,              /* Default horizontal position.        */
        CW_USEDEFAULT,              /* Default vertical position.          */
        CW_USEDEFAULT,              /* Default width.                      */
        CW_USEDEFAULT,              /* Default height.                     */
        NULL,                       /* Overlapped windows have no parent.  */
        NULL,                       /* Use the window class menu.          */
        hInstance,                  /* This instance owns this window.     */
        NULL                        /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hWnd, nCmdShow);   /* Show the window                       */
    UpdateWindow(hWnd);           /* Sends WM_PAINT message                */
    return (TRUE);                /* Returns the value from PostQuitMessage*/
}

/***************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
****************************************************************************/

BOOL FTYPE ConfigDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                          /* additional information          */
LPARAM lParam;                          /* additional information          */
{

    switch (message)
    {
        case WM_INITDIALOG :

            if (!gwSelectDevice)
            {
                // Step 1: Select Device
                ErrCde = DRV_SelectDevice(hDlg, FALSE, &dwDeviceNum, szDescript);
                if (ErrCde != SUCCESS)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                    return 0;
                }

                gwSelectDevice = TRUE;
            }

			// Show device description to Button text
            SendDlgItemMessage(hDlg, IDC_DeviceSelect, WM_SETTEXT,
               50, (LPARAM)(LPSTR)szDescript);



            // Initialize Conversion Number
            ltoa(gulConvNum, szBuffer, 10);
            SendDlgItemMessage(hDlg, IDC_CONVERSION, EM_REPLACESEL, 0,
                            (LPARAM)((LPSTR)szBuffer));

            // Initialize Pacer Rate
            ltoa(gdwPacerRate, szBuffer, 10);
            SendDlgItemMessage(hDlg, IDC_PACER, EM_REPLACESEL, 0,
                            (LPARAM)((LPSTR)szBuffer));

            // Initialize wave form count
			if(gwWaveFormCount == INFINITE ) 
				strcpy(szBuffer,"INFINITE");
			else
				ltoa(gwWaveFormCount, szBuffer, 10);
            SendDlgItemMessage(hDlg, IDC_WAVEFORMCOUNT, EM_REPLACESEL, 0,
                            (LPARAM)((LPSTR)szBuffer));

            // Initialize Triggering
            CheckRadioButton(hDlg, IDC_INTTRIG, IDC_EXTTRIG,
                                   (gwExtTrig)?IDC_EXTTRIG:IDC_INTTRIG);

            // Initialize Message Flag
            CheckRadioButton(hDlg, IDC_EVTENABLE, IDC_EVTDISABLE,
                (gwEvtFlag)?IDC_EVTENABLE:IDC_EVTDISABLE);

            return TRUE;

        case WM_COMMAND :
             switch (LOWORD(wParam))
             {
                case IDOK :

                     EndDialog(hDlg, 0);
                     return TRUE;


                case IDCANCEL :
                     EndDialog(hDlg, 0);
                     return TRUE;

                case IDC_DeviceSelect:

                    // Step 1: Select Device
                    ErrCde = DRV_SelectDevice(hDlg, FALSE, &dwDeviceNum, szDescript);
                    if (ErrCde != SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                        return 0;
                    }

                    // Show device description to Button text
                    SendDlgItemMessage(hDlg, IDC_DeviceSelect, WM_SETTEXT,
                        50, (LPARAM)(LPSTR)szDescript);

                    gwSelectDevice = TRUE;

                    return TRUE;

                case IDC_CONVERSION :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_CONVERSION,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_CONVERSION,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           gulConvNum = atol(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_CONVERSION,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_PACER :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_PACER,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_PACER,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           gdwPacerRate = atol(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_PACER,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_WAVEFORMCOUNT :

                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_WAVEFORMCOUNT,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_WAVEFORMCOUNT,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;

						   if(!strcmp(szBuffer,"INFINITE"))
								gwWaveFormCount = INFINITE;
						   else
								gwWaveFormCount = atol(szBuffer);

                           SendDlgItemMessage(hDlg, IDC_WAVEFORMCOUNT,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;


                case IDC_INTTRIG :
                     gwExtTrig = 0;
                     break;;
                case IDC_EXTTRIG :
                     gwExtTrig = 1;
                     break;
                case IDC_EVTENABLE :
                    gwEvtFlag = 1;
                    break;
                case IDC_EVTDISABLE :
                    gwEvtFlag = 0;
                    break;
             }
             break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: WaveDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for waveform setting
****************************************************************************/

BOOL FTYPE WaveDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                          /* additional information          */
LPARAM lParam;                          /* additional information          */
{
    int  i;

    switch (message)
    {
        case WM_INITDIALOG :


             // Initialize Output Waveform's Selection
             for (i = 0; i < 5; i ++)
             {
                 SendDlgItemMessage(hDlg, IDC_WAVE_CH0, CB_ADDSTRING, 0,
                       (LPARAM)((LPSTR)szWaveform[i]));
                 SendDlgItemMessage(hDlg, IDC_WAVE_CH1, CB_ADDSTRING, 0,
                       (LPARAM)((LPSTR)szWaveform[i]));
                 SendDlgItemMessage(hDlg, IDC_WAVE_CH2, CB_ADDSTRING, 0,
                       (LPARAM)((LPSTR)szWaveform[i]));
                 SendDlgItemMessage(hDlg, IDC_WAVE_CH3, CB_ADDSTRING, 0,
                       (LPARAM)((LPSTR)szWaveform[i]));

             }

             SendDlgItemMessage(hDlg, IDC_WAVE_CH0, CB_SELECTSTRING, (WPARAM)(-1),
                    (LPARAM)((LPSTR)szWaveform[sWaveCh0.wWaveform]));
             SendDlgItemMessage(hDlg, IDC_WAVE_CH1, CB_SELECTSTRING,
                    (WPARAM)(-1), (LPARAM)((LPSTR)szWaveform[sWaveCh1.wWaveform]));
             SendDlgItemMessage(hDlg, IDC_WAVE_CH2, CB_SELECTSTRING, (WPARAM)(-1),
                    (LPARAM)((LPSTR)szWaveform[sWaveCh2.wWaveform]));
             SendDlgItemMessage(hDlg, IDC_WAVE_CH3, CB_SELECTSTRING,
                    (WPARAM)(-1), (LPARAM)((LPSTR)szWaveform[sWaveCh3.wWaveform]));

             // Initialize Magnitude
             sprintf(szBuffer, "%6.2f", sWaveCh0.fMagnitude);
             SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH0, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%6.2f", sWaveCh1.fMagnitude);
             SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH1, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%6.2f", sWaveCh2.fMagnitude);
             SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH2, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%6.2f", sWaveCh3.fMagnitude);
             SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH3, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));

             // Initialize Offset
             sprintf(szBuffer, "%6.2f", sWaveCh0.fOffset);
             SendDlgItemMessage(hDlg, IDC_OFFSET_CH0, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%6.2f", sWaveCh1.fOffset);
             SendDlgItemMessage(hDlg, IDC_OFFSET_CH1, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%6.2f", sWaveCh2.fOffset);
             SendDlgItemMessage(hDlg, IDC_OFFSET_CH2, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             sprintf(szBuffer, "%6.2f", sWaveCh3.fOffset);
             SendDlgItemMessage(hDlg, IDC_OFFSET_CH3, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));

             // Initialize Period
             itoa(sWaveCh0.wPeriod, szBuffer, 10);
             SendDlgItemMessage(hDlg, IDC_PERIOD_CH0, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             itoa(sWaveCh1.wPeriod, szBuffer, 10);
             SendDlgItemMessage(hDlg, IDC_PERIOD_CH1, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             itoa(sWaveCh2.wPeriod, szBuffer, 10);
             SendDlgItemMessage(hDlg, IDC_PERIOD_CH2, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));
             itoa(sWaveCh3.wPeriod, szBuffer, 10);
             SendDlgItemMessage(hDlg, IDC_PERIOD_CH3, EM_REPLACESEL, 0,
                             (LPARAM)((LPSTR)szBuffer));

             return TRUE;

        case WM_COMMAND :
             switch (LOWORD(wParam))
             {
                case IDOK :
                case IDCANCEL :
					if ( bRunTimer )
					{
						for(i=0; i<4; i++)									
						{
							switch(i)
							{
							case 0 : 
								if(sWaveCh0.wWaveform != 4)  // 4 --> No Waveform
								{
									usEnabledChannel |= ADV_CHANNEL0;
									// set real voltage to hVoltageBuf
									SetRealBuffer(lpVoltageBuf[0], gulConvNum, &sWaveCh0);
									usChannelCount++;
								}
								else
									continue;
								
								break;
							case 1 : 
								if(sWaveCh1.wWaveform != 4)  
								{
									usEnabledChannel |= ADV_CHANNEL1;
									SetRealBuffer(lpVoltageBuf[1], gulConvNum, &sWaveCh1);
									usChannelCount++;
								}
								else
									continue;
								
								break;
							case 2 : 
								if(sWaveCh2.wWaveform != 4) 
								{
									usEnabledChannel |= ADV_CHANNEL2;
									SetRealBuffer(lpVoltageBuf[2], gulConvNum, &sWaveCh2);
									usChannelCount++;
								}
								else
									continue;
								
								break;
							case 3 : 
								if(sWaveCh3.wWaveform != 4) 
								{
									usEnabledChannel |= ADV_CHANNEL3;
									SetRealBuffer(lpVoltageBuf[3], gulConvNum, &sWaveCh3);
									usChannelCount++;
								}
								else
									continue;
								
								break;
							}
							
							// call FAOScale for transfer voltage to binary data
							ptFAOScale.VoltArray = lpVoltageBuf[i];
							ptFAOScale.BinArray  = lpBuf[i];
							ptFAOScale.chan      = i;
							ptFAOScale.count     = gulConvNum;
							if ((ErrCde = DRV_FAOScale(DriverHandle,
								(LPT_FAOScale)&ptFAOScale)) != 0)
							{
								DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
								MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
								MyFreeBuffer();
								DRV_DeviceClose((LONG far *)&DriverHandle);
								return 0;
							}						
						}

						SetMultiToOneBuffer(usEnabledChannel,gulConvNum);
					}					
					EndDialog(hDlg, 0);
					return TRUE;

                case IDC_WAVE_CH0 :
                     if (HIWORD(wParam) == CBN_KILLFOCUS)
                     {
                        i = (int)SendDlgItemMessage(hDlg, IDC_WAVE_CH0,
                                  CB_GETCURSEL, 0, 0) ;
                        if (i != CB_ERR)
                           sWaveCh0.wWaveform = i;
                     }
                     return TRUE;

                case IDC_WAVE_CH1 :
                     if (HIWORD(wParam) == CBN_KILLFOCUS)
                     {
                        i = (int)SendDlgItemMessage(hDlg, IDC_WAVE_CH1,
                                  CB_GETCURSEL, 0, 0) ;
                        if (i != CB_ERR)
                           sWaveCh1.wWaveform = i;
                     }
                     return TRUE;

                case IDC_WAVE_CH2 :
                     if (HIWORD(wParam) == CBN_KILLFOCUS)
                     {
                        i = (int)SendDlgItemMessage(hDlg, IDC_WAVE_CH2,
                                  CB_GETCURSEL, 0, 0) ;
                        if (i != CB_ERR)
                           sWaveCh2.wWaveform = i;
                     }
                     return TRUE;

                case IDC_WAVE_CH3 :
                     if (HIWORD(wParam) == CBN_KILLFOCUS)
                     {
                        i = (int)SendDlgItemMessage(hDlg, IDC_WAVE_CH3,
                                  CB_GETCURSEL, 0, 0) ;
                        if (i != CB_ERR)
                           sWaveCh3.wWaveform = i;
                     }
                     return TRUE;

                case IDC_MAGNITUDE_CH0 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH0,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH0,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh0.fMagnitude = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH0,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_MAGNITUDE_CH1 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH1,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH1,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh1.fMagnitude = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH1,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_MAGNITUDE_CH2 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH2,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH2,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh2.fMagnitude = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH2,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_MAGNITUDE_CH3 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH3,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH3,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh3.fMagnitude = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_MAGNITUDE_CH3,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_OFFSET_CH0 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_OFFSET_CH0,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_OFFSET_CH0,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh0.fOffset = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_OFFSET_CH0,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_OFFSET_CH1 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_OFFSET_CH1,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_OFFSET_CH1,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh1.fOffset = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_OFFSET_CH1,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_OFFSET_CH2 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_OFFSET_CH2,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_OFFSET_CH2,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh2.fOffset = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_OFFSET_CH2,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_OFFSET_CH3 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_OFFSET_CH3,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_OFFSET_CH3,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh3.fOffset = (float)atof(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_OFFSET_CH3,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_PERIOD_CH0 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_PERIOD_CH0,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_PERIOD_CH0,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh0.wPeriod = atoi(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_PERIOD_CH0,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_PERIOD_CH1:
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_PERIOD_CH1,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_PERIOD_CH1,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh1.wPeriod = atoi(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_PERIOD_CH1,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_PERIOD_CH2 :
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_PERIOD_CH2,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_PERIOD_CH2,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh2.wPeriod = atoi(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_PERIOD_CH2,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

                case IDC_PERIOD_CH3:
                     if (HIWORD(wParam) == EN_KILLFOCUS)
                     {
                        if(SendDlgItemMessage(hDlg, IDC_PERIOD_CH3,
                                  EM_GETMODIFY, 0, 0))
                        {
                           SendDlgItemMessage(hDlg, IDC_PERIOD_CH3,
                                  WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                           sWaveCh3.wPeriod = atoi(szBuffer);
                           SendDlgItemMessage(hDlg, IDC_PERIOD_CH3,
                                  EM_SETMODIFY, FALSE, 0) ;
                        }
                     }
                     return TRUE;

             }
             break;
    }
    return FALSE ;
}

/***************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
****************************************************************************/

long FTYPE MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                            /* additional information          */
LPARAM lParam;                            /* additional information          */
{
    TEXTMETRIC tm;
    HDC     hdc;
    static  int cxChar, cyChar;
    char    szBuffer[120];
    RECT    rect;
	int		i;
	BOOL	bflag = TRUE;



    switch (message)
      {
       case WM_CREATE:

            hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

            lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

            lpfnWaveDlgProc = MakeProcInstance (WaveDlgProc, hInstance) ;

            hdc = GetDC(hWnd);
            SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));
            GetTextMetrics(hdc, &tm);
            cxChar = tm.tmAveCharWidth;
            cyChar = tm.tmHeight + tm.tmExternalLeading;
            ReleaseDC(hWnd, hdc);

            //Create stop button and status button
			        //Create stop button and status button
            hwndStopButton = CreateWindow("button", "Stop",
                (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
                0, 0, 8*cxChar, 3*cyChar/2, hWnd,
                (HMENU)IDC_STOPBUTTON,
                (HANDLE)hInstance, (LPVOID)NULL);

            EnableWindow(hwndStopButton, FALSE);

            return 0 ;

       case WM_CLOSE:

		   MyFreeBuffer();
		   DestroyWindow( hWnd ); 
 

		   return 0;
			

       case WM_COMMAND:     /* message: from application menu */

         hMenu = GetMenu(hWnd);

         switch(LOWORD(wParam))
          {
            // Show the dialog box of the input parameter
            case IDM_SETTING :

                DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                         hWnd, lpfnConfigDlgProc) ;
                return 0;

            case IDM_WAVEFORM :

                DialogBox (hInstance, MAKEINTRESOURCE(IDD_WAVEFORM),
                         hWnd, lpfnWaveDlgProc) ;
                return 0;

           case IDM_RUN :

                // Step 1: Device open
                ErrCde = DRV_DeviceOpen(dwDeviceNum,
                    (LONG far *)&DriverHandle);
                if (ErrCde != SUCCESS)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                    return 0;
                }

				// Step 2: Allocate buffer and set data to buffer
				// Process 4 channel 				
				usChannelCount = 0;    // reset channel count
				usEnabledChannel = 0;
				for(i=0; i<4; i++)
				{

					// Allocate memory used by driver
					if((hBuf[i]=(USHORT far *)GlobalAlloc(GPTR,
						sizeof(USHORT) * gulConvNum)) == 0)
					{
						MessageBox(hWnd,"Not enough memory for buffer ",
							"High Speed",MB_OK);
						DRV_DeviceClose((LONG far *)&DriverHandle);
						return 0;
					}

					// Allocate memory for real voltage
					if((hVoltageBuf[i]=(FLOAT far *)GlobalAlloc(GPTR,
						sizeof(FLOAT) * gulConvNum)) == 0)
					{
						MessageBox(hWnd,"Not enough memory for buffer ",
							"High Speed",MB_OK);
						DRV_DeviceClose((LONG far *)&DriverHandle);
						return 0;
					}


					lpBuf[i] = (USHORT far *)hBuf[i];
					lpVoltageBuf[i] = (FLOAT far  *)hVoltageBuf[i];

					switch(i)
					{
					case 0 : 
							 if(sWaveCh0.wWaveform != 4)  // 4 --> No Waveform
							 {
								 usEnabledChannel |= ADV_CHANNEL0;
								 // set real voltage to hVoltageBuf
								 SetRealBuffer(lpVoltageBuf[0], gulConvNum, &sWaveCh0);
								 usChannelCount++;
							 }
							 else
								 continue;

							 break;
					case 1 : 
							 if(sWaveCh1.wWaveform != 4)  
							 {
								 usEnabledChannel |= ADV_CHANNEL1;
								 SetRealBuffer(lpVoltageBuf[1], gulConvNum, &sWaveCh1);
								 usChannelCount++;
							 }
							 else
								 continue;

							 break;
					case 2 : 
							 if(sWaveCh2.wWaveform != 4) 
							 {
								 usEnabledChannel |= ADV_CHANNEL2;
								 SetRealBuffer(lpVoltageBuf[2], gulConvNum, &sWaveCh2);
								 usChannelCount++;
							 }
							 else
								 continue;

							 break;
					case 3 : 
							 if(sWaveCh3.wWaveform != 4) 
							 {
								 usEnabledChannel |= ADV_CHANNEL3;
								 SetRealBuffer(lpVoltageBuf[3], gulConvNum, &sWaveCh3);
								 usChannelCount++;
							 }
							 else
								 continue;

							 break;
					}
				
					// call FAOScale for transfer voltage to binary data
					ptFAOScale.VoltArray = lpVoltageBuf[i];
					ptFAOScale.BinArray  = lpBuf[i];
					ptFAOScale.chan      = i;
					ptFAOScale.count     = gulConvNum;
					if ((ErrCde = DRV_FAOScale(DriverHandle,
						(LPT_FAOScale)&ptFAOScale)) != 0)
					{
						DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
						MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
						MyFreeBuffer();
						DRV_DeviceClose((LONG far *)&DriverHandle);
						return 0;
					}
				}

				// Allocate memory for common buffer of 4 channel
				if((hCommonBuf=(USHORT far *)GlobalAlloc(GHND, 
						sizeof(USHORT) * gulConvNum * usChannelCount)) == 0)
				{
					MessageBox(hWnd,"Not enough memory for buffer ",
						"High Speed",MB_OK);
					MyFreeBuffer();
					DRV_DeviceClose((LONG far *)&DriverHandle);
					return 0;
				}

			    // Lock down buffer
			    if((lpCommonBuf = (USHORT far *)GlobalLock(hCommonBuf)) == 0)
				{
					MessageBox(hWnd,"Not enough memory for buffer ",
						"High Speed",MB_OK);
					MyFreeBuffer();
					GlobalFree(hCommonBuf);
					DRV_DeviceClose((LONG far *)&DriverHandle);
					return 0;
				}

				// set 4 buffer to 1 common buffer
				SetMultiToOneBuffer(usEnabledChannel,gulConvNum);


                // Step 3: Enable event feature
                ptEnableEvent.EventType = ADS_EVT_INTERRUPT  |
                    ADS_EVT_BUFCHANGE  |
                    ADS_EVT_TERMINATED |
                    ADS_EVT_OVERRUN;
                ptEnableEvent.Enabled = gwEvtFlag;
                ptEnableEvent.Count   = 1;
                if ((ErrCde = DRV_EnableEvent(DriverHandle,
                    (LPT_EnableEvent)&ptEnableEvent)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
					MyFreeBuffer(); 
					DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Step 4: call FAOWaveFormStart for start action
                // Default steting - 
				// conversion number = 2048 * 4
                // Enabled Channel = 0,1,2,3
				// pacer rate = 1000 Hz
				// output waveform number = 30

		        ptFAOWaveFormStart.TrigSrc    = gwExtTrig;					// triggering
			    ptFAOWaveFormStart.SampleRate = gdwPacerRate;				// pacer rate = 1K
				ptFAOWaveFormStart.Count      = gulConvNum * usChannelCount;// DA conversion number
				ptFAOWaveFormStart.WaveCount  = gwWaveFormCount;			// Waveform number
				ptFAOWaveFormStart.Buffer     = (USHORT far *)lpCommonBuf;  // analog output data
				ptFAOWaveFormStart.EnabledChannel     = usEnabledChannel;   // which channel

                if ((ErrCde = DRV_FAOWaveFormStart(DriverHandle,
                    (LPT_FAOWaveFormStart)&ptFAOWaveFormStart)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
					MyFreeBuffer();
					GlobalUnlock(hCommonBuf);
					GlobalFree(hCommonBuf);
					DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }


                // if event feature is enabled , then create one thread.
                if (gwEvtFlag)
                {
                    hThreadHandle = CreateThread(0, 0,
						                        (LPTHREAD_START_ROUTINE) UserThread,
												0, 0, (LPDWORD)&dwThreadID);
					bThreadloop = TRUE;
                }



                EnableMenuItem(hMenu, IDM_RUN, MF_DISABLED | MF_BYCOMMAND);
                EnableWindow(hwndStopButton, TRUE);	// Enable stop buttons


                // if event feature is enabled 
                // Set the new timer type and start it up.
                //
                if (gwEvtFlag)
                {			
					if (SetTimer(hWnd, 1, 50, NULL) == (UINT)NULL)
					{
						MessageBox(hWnd, "Couldn't create timer.", "Notice!", MB_OK);
						DRV_DeviceClose((LONG far *)&DriverHandle);
						return 0;
					}
					bRunTimer = TRUE;
				}
                return 0;

                //end IDM_RUN


           case IDC_STOPBUTTON:
			   
			    bThreadloop = FALSE;	// disable thread loop
				Sleep(500);

				if(!gwExtTrig)
				{
					while(bflag)
					{
						if(bThreadflag==TRUE)
							Sleep(10);
						else
							bflag = FALSE;
					}
				}

				// Stop Timer
				if(bRunTimer)  
				{
					KillTimer(hWnd, 1);
					bRunTimer = FALSE;  
				}

                EnableWindow(hwndStopButton, FALSE);    // Disable stop buttons
				CloseHandle(hThreadHandle);				// close thread handle
                EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);


                // Step 1: Stop D/A conversion 
                if ((ErrCde = DRV_FAOStop(DriverHandle)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
					MyFreeBuffer();
					GlobalUnlock(hCommonBuf);
					GlobalFree(hCommonBuf);
					DRV_DeviceClose((LONG far *)&DriverHandle);
                    return 0;
                }

                // Step 2: Free buffer
				MyFreeBuffer();
				GlobalUnlock(hCommonBuf);
				GlobalFree(hCommonBuf);

                // Step 3: Close driver
                DRV_DeviceClose((LONG far *)&DriverHandle);

                return 0;
          }


    case WM_TIMER:

	    // Get transfer status
		ptFAOCheck.ActiveBuf    = &gwActiveBuf;
		ptFAOCheck.stopped      = &gwStopped;
		ptFAOCheck.CurrentCount = &ulCurrentCount;
		ptFAOCheck.overrun      = &gwOverrun;
		ptFAOCheck.HalfReady    = &gwHalfReady;

		if ((ErrCde = DRV_FAOCheck(DriverHandle,(LPT_FAOCheck)&ptFAOCheck)) != 0)
		{
			DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
			MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
			MyFreeBuffer();
			GlobalUnlock(hCommonBuf);
			GlobalFree(hCommonBuf);
			DRV_DeviceClose((LONG far *)&DriverHandle);
			return 0;
		}

        // Display Data
        hdc = GetDC(hWnd);
        GetClientRect(hWnd, &rect);
        InvalidateRect(hWnd, NULL, TRUE);
        UpdateWindow(hWnd);
        wsprintf((LPSTR)szBuffer, "Wave Count  = %ld", ulCurrentCount);
        DrawText(hdc, szBuffer, -1, &rect,
               DT_SINGLELINE | DT_CENTER | DT_VCENTER);
        ReleaseDC(hWnd, hdc);


		// if thread finish run stop instruction 
		if(!bThreadloop)
		{
			// if A/O process finish --> call DRV_FAOStop
			if(gwStopped) 
			{
                // Step 1: Stop D/A conversion for high speed
				if ((ErrCde = DRV_FAOStop(DriverHandle)) != 0)
				{
					DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
					MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
				}

                // Step 2: Free buffer
				MyFreeBuffer();
				GlobalUnlock(hCommonBuf);
				GlobalFree(hCommonBuf);

			    // Step 3: Close driver
				DRV_DeviceClose((LONG far *)&DriverHandle);


				// Stop Timer
				if(bRunTimer)  
				{
					KillTimer(hWnd, 1);
					bRunTimer = FALSE;  
				}
			    EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);
				CloseHandle(hThreadHandle);
			}
		}

        return 0;


    case WM_DESTROY:

        PostQuitMessage(0);
        break;

    default:
         return (DefWindowProc(hWnd, message, wParam, lParam));
      }

   return ((LONG)NULL);
}

void SetRealBuffer(float far *lpBuf, long num, LPSWAVE lpWave)
{
    int i, r, half;
    float slope;

    switch (lpWave->wWaveform)
    {
        case WAVE_SINE:
             for (i = 0; i < num; i ++)
			 {
               *(lpBuf+i) = (float)(lpWave->fMagnitude
                            * sin(6.2831853*(double)i/(double)lpWave->wPeriod)
                            + lpWave->fOffset);
			 }
             break;
        case WAVE_TRIANGLE:
             slope = lpWave->fMagnitude * 4.0f / (float)lpWave->wPeriod;

             for (i = 0; i < num; i ++)
                {
                    r = i % lpWave->wPeriod;
                    half = (int)((float)lpWave->wPeriod / 2.0f);
                    if (r <= half)
                       *(lpBuf + i) = slope * r - lpWave->fMagnitude
                                      + lpWave->fOffset;
                    else
                       *(lpBuf + i) = slope * (lpWave->wPeriod - r)
                                      - lpWave->fMagnitude + lpWave->fOffset;
                }

             break;
        case WAVE_SQUARE:
             for (i = 0; i < num; i ++)
                {
                    r = i % lpWave->wPeriod;
                    half = (int)((float)lpWave->wPeriod / 2.0f);
                    if (r <= half)
                       *(lpBuf + i) = lpWave->fOffset - lpWave->fMagnitude;
                    else
                       *(lpBuf + i) = lpWave->fOffset + lpWave->fMagnitude;
                }
             break;
        case SINE_TRIANGLE:
			{
			 ULONG x;
			 double y;

             slope = lpWave->fMagnitude * 4.0f / (float)lpWave->wPeriod;
             for (i = 0; i < num; i ++)
			 {
				if(i%2)
				{
					modf(((double)i/2.0),(double *)&y);
					y = y + 1;
                    r = ((USHORT)y) % lpWave->wPeriod;
                    half = (int)((float)lpWave->wPeriod / 2.0f);
                    if (r <= half)
                       *(lpBuf + i) = slope * r - lpWave->fMagnitude
                                      + lpWave->fOffset;
                    else
                       *(lpBuf + i) = slope * (lpWave->wPeriod - r)
                                      - lpWave->fMagnitude + lpWave->fOffset;
				}
				else
				{
					x = i/2;
					*(lpBuf+i) = (float)(lpWave->fMagnitude
                            * sin(6.28318*(double)x/(double)lpWave->wPeriod)
                            + lpWave->fOffset);
				}
			 }
             break;
			}
    }

}

void SetMultiToOneBuffer(USHORT usEnabledChannel, int count)
{
	int i,TotalCount;

	TotalCount=0;

    for (i = 0; i < count; i ++)
	{
		if(usEnabledChannel & ADV_CHANNEL0)
		{
			((USHORT far *)lpCommonBuf)[TotalCount] = 
				((USHORT)((USHORT far *) lpBuf[0])[i]) & 0x0fff;
			TotalCount++;
		}
		if(usEnabledChannel & ADV_CHANNEL1)
		{
			((USHORT far *)lpCommonBuf)[TotalCount] = 
				((USHORT)((USHORT far *) lpBuf[1])[i] | ( 0x01 << 12)) & 0x3fff;
			TotalCount++;
		}
		if(usEnabledChannel & ADV_CHANNEL2)
		{
			((USHORT far *)lpCommonBuf)[TotalCount] = 
				((USHORT)((USHORT far *) lpBuf[2])[i] | ( 0x02 << 12)) & 0x3fff;
			TotalCount++;
		}
		if(usEnabledChannel & ADV_CHANNEL3)
		{
			((USHORT far *)lpCommonBuf)[TotalCount] = 
				((USHORT)((USHORT far *) lpBuf[3])[i] | ( 0x03 << 12)) & 0x3fff;
			TotalCount++;
		}
	}

}


//------------------------------------------------------------------
void UserThread()
{
    USHORT usEventType;
    

    while(bThreadloop)
    {
        // Check message
        ptCheckEvent.EventType = &usEventType;
        ptCheckEvent.Milliseconds = 100;

		bThreadflag = TRUE;
        DRV_CheckEvent(DriverHandle, (LPT_CheckEvent)&ptCheckEvent);

        // Process interrupt event
        if (usEventType & ADS_EVT_INTERRUPT)
            adInterruptEvent();

        // Process buffer change event
        if (usEventType & ADS_EVT_BUFCHANGE)
            adBufChangeEvent();

        // Process overrun event
        if (usEventType & ADS_EVT_OVERRUN)
            adOverrunEvent();

        // Process terminate event
        if (usEventType & ADS_EVT_TERMINATED)
            adTerminateEvent();

		bThreadflag = FALSE;
    }

}

//------------------------------------------------------------------
void adInterruptEvent()
{
    return;
}

//------------------------------------------------------------------
void adBufChangeEvent()
{
    return;
}
//------------------------------------------------------------------
void adOverrunEvent()
{

    return;
}
//------------------------------------------------------------------
void adTerminateEvent()
{

    bThreadloop = FALSE;
    // Disable stop and status buttons
    EnableWindow(hwndStopButton, FALSE);
    EnableWindow(hwndStatusButton, FALSE);

    return;
}


void MyFreeBuffer()
{
	int i;

	for(i=0; i<4; i++)
	{

		if(hBuf[i]) 
		{
			GlobalFree(hBuf[i]);
			hBuf[i] = NULL;
		}
		if(hVoltageBuf[i])
		{
			GlobalFree(hVoltageBuf[i]);
			hVoltageBuf[i] = NULL;
		}
	}

}