/*
 ******************************************************************************
 *  Program     : DIBMDMA.C                                                   *
 *  Description : Demo program for Windows with DMA data transfer             *
 *  Revision    : 2.0B                                                        *
 *  Date        : 04/18/2003                  Advantech Beijing R&D Center    *
 ******************************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance()    - Saves instance handle and creates main window
 *
 *   MainWndProc()     - Process main window messages
 *
 *   ShowDataDlgProc() - Process the dialog box messages for acquired data
 *
 *   DisplayDlgProc()  - Process the dialog box messages for display range
 *
 *   ConfigDlgProc()   - Process the dialog box messages for configuration
 */

#include <windows.h>            /* required for all Windows applications */
#include <stdlib.h>
#include <windowsx.h>
#include <commdlg.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\Include\driver.h"
#include "resource.h"
#include "DIdmaBm.h"

// ----------------------------------------------------------------------------
// CONSTANT DEFINITION
// ----------------------------------------------------------------------------
#define     MAX_DEVICES     100
#define     Mega            0x100000


// ----------------------------------------------------------------------------
// GLOBAL VARIABLES
// ----------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hWnd;                       // main window handle
HDC         hdc;
RECT        rect;

static      DEVFEATURES           DevFeatures;       // structure for device features
static      PT_DeviceGetFeatures  ptDevFeatures;  // Devfeatures table

static      PT_EnableEvent     ptEnableEvent;     // Enable event
static      PT_CheckEvent      ptCheckEvent;      // Check event

HANDLE      hThreadHandle;
DWORD       dwThreadID;

char        szErrMsg[80];               // Use for MESSAGEBOX function
char        szBuffer[80];				    // Temperatory buffer
LRESULT     ErrCde;                     // Return error code

//
// used for fixed memory for the installed devices
//
DEVLIST     DeviceList[MAX_DEVICES];
DEVLIST     SubDeviceList[MAX_DEVICES];

BOOL        bThreadflag = FALSE;
BOOL        bThreadloop = FALSE;

static  ULONG  dwDeviceNum;                       // Device number
char        szDeviceName[80];				    // Temperatory buffer

static  LONG   DriverHandle = (LONG)NULL;         // driver handle
static  USHORT gwDevice = 0, gwSubDevice = 0;     // device index
static  SHORT  gnNumOfDevices, gnNumOfSubdevices; // number of installed devices
static  ULONG  gdwStartPt = 0, gdwStopPt = 99;    // display points

// global 
DWORD  gdwDataWidth;
DWORD  gdwOperationMode;
DWORD  gdwPatternMatchValue;
DWORD  gdwDIStartMode;
DWORD  gdwDITriggerSource;
DWORD  gdwDIStopMode;
DWORD  gdwCounterValue[3];
DWORD  gdwCount;
DWORD  gdwUserBufferSize = 1;
SHORT  gdwCyclicMode = 0;    // cyclic or non-cyclic mode
HGLOBAL hDisplayBuf;
LPVOID  hUserBuf;

BOOL InitApplication(HANDLE hInstance);
BOOL InitInstance(HANDLE hInstance, int nCmdShow);
long FTYPE MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
int  FTYPE WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow);
static FARPROC lpfnConfigDlgProc, lpfnShowDataDlgProc, lpfnDisplayDlgProc,
    lpfnGainListDlgProc;

static  HWND hwndStopButton, hwndStatusButton;
static  HANDLE  hInstance;
HMENU hMenu;

ULONG gdwDIBufferChangeCounter = 0;  // counter by UserThread
ULONG gdwDIOverRunCounter = 0;
ULONG gdwDITerminateCounter = 0;
BOOL  gbConfigured = 0;

ULONG gulOverrunTime  = 10000;  // 0 = INFINITE
char  szCounter[40];

void UserThread();
void adTerminateEvent();

/******************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
******************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                            /* current instance             */
HANDLE hPrevInstance;                        /* previous instance            */
LPSTR lpCmdLine;                             /* command line                 */
int nCmdShow;                                /* show-window type (open/icon) */
{
    MSG msg;                                 /* message                      */

    if (!hPrevInstance)                  /* Other instances of app running?  */
        if (!InitApplication(hInstance)) /* Initialize shared things         */
            return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,        /* message structure                      */
        (HWND)NULL,                /* handle of window receiving the message */
        (UINT)NULL,                /* lowest message to examine              */
        (UINT)NULL))               /* highest message to examine             */
    {
        TranslateMessage(&msg);    /* Translates virtual key codes           */
        DispatchMessage(&msg);     /* Dispatches message to window           */
    }
    return (msg.wParam);           /* Returns the value from PostQuitMessage */
}

/******************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
******************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                              /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW;   /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;   /* window process procedure     */
    wc.cbClsExtra = 0;                    /* No per-class extra data.        */
    wc.cbWndExtra = 0;                    /* No per-window extra data.       */
    wc.hInstance = hInstance;             /* Application that owns the class.*/
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";      /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";     /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));
}

/******************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
******************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
HANDLE          hInstance;          /* Current instance identifier.          */
int             nCmdShow;           /* Param for first ShowWindow() call.    */
{

    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hWnd = CreateWindow(
        "MyClass",                    /* See RegisterClass() call.           */
        "Advantech Driver Demo : FDI Busmaster DMA Data Transfer" ,
        /* Window title bar  */
        WS_OVERLAPPEDWINDOW,          /* Window style.                       */
        CW_USEDEFAULT,                /* Default horizontal position.        */
        CW_USEDEFAULT,                /* Default vertical position.          */
        CW_USEDEFAULT,                /* Default width.                      */
        CW_USEDEFAULT,                /* Default height.                     */
        NULL,                         /* Overlapped windows have no parent.  */
        NULL,                         /* Use the window class menu.          */
        hInstance,                    /* This instance owns this window.     */
        NULL                          /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hWnd, nCmdShow);     /* Show the window                       */
    UpdateWindow(hWnd);             /* Sends WM_PAINT message                */
    return (TRUE);                  /* Returns the value from PostQuitMessage*/
}

/******************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
******************************************************************************/

BOOL FTYPE ConfigDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                              /* window handle                   */
unsigned message;                       /* type of message                 */
WPARAM wParam;                          /* additional information          */
LPARAM lParam;                          /* additional information          */
{
    int     i;
    char    szBuffer[80];
    char    szTemp[80];
	 BOOL bEnable;

    switch (message)
    {
    case WM_INITDIALOG :
       
        // Step 1: Select Device
		if(!gbConfigured)
		{
			ErrCde = DRV_SelectDevice( hDlg, FALSE, &dwDeviceNum, szDeviceName);
			if (ErrCde != SUCCESS)
			{
				DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
				MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
				return 0;
			}
		}
		SetDlgItemText(hDlg,IDC_DEVICE_NAME,szDeviceName);

        // Step 2: Open Device
        ErrCde = DRV_DeviceOpen(dwDeviceNum, (LONG far *)&DriverHandle);
        if (ErrCde != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }


        // Step 3: Get device features
        ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
        ptDevFeatures.size = sizeof(DEVFEATURES);
        if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
            (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return 0;
        }

        if ( !gbConfigured )
        {
           gdwCount = sizeof(DWORD);
		      // 32DI(0)/32DO(1)/ 16 DIO(2) / 8DIO(3)
		      ErrCde = DRV_DeviceGetProperty(DriverHandle, CFG_DioFdioDirection, &gdwDataWidth, &gdwCount);
		      if ( gdwDataWidth == 1 )
		      {
			      gdwDataWidth = 0; // 32DI
		      }
			  //Get the Operation Mode Normal(0) 8255(1) Burst(2)
              ErrCde = DRV_DeviceGetProperty(DriverHandle, CFG_DiPatternMatchValue, &gdwPatternMatchValue, &gdwCount);
		      ErrCde = DRV_DeviceGetProperty(DriverHandle, CFG_DiOperationMode, &gdwOperationMode, &gdwCount);	
		      // Software(1), External trigger(2) Pattern Match(3)
		      ErrCde = DRV_DeviceGetProperty(DriverHandle, CFG_DiStartMethod, &gdwDIStartMode, &gdwCount);	
		      // 30MHz(1), 15Mhz(2), 10MHz(3), Counter 0 OUT (4), External (5).
		      ErrCde = DRV_DeviceGetProperty(DriverHandle, CFG_DiPacerSource, &gdwDITriggerSource, &gdwCount);
		      // Software(1), External trigger(2) Pattern Match(3)
		      ErrCde = DRV_DeviceGetProperty(DriverHandle, CFG_DiStopMethod, &gdwDIStopMode, &gdwCount);
		      // 82C54 Counter 0-2 value
		      gdwCount = sizeof(gdwCounterValue);
		      ErrCde = DRV_DeviceGetProperty(DriverHandle, CFG_CounterCountValue, &gdwCounterValue, &gdwCount);
   
		      if ( ErrCde != SUCCESS )
            {
               DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
               MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
               DRV_DeviceClose((LONG far *)&DriverHandle);
               return 0;
            }
            gbConfigured = TRUE;
        }


        // Step 4: Initialize Input Range List Combobox with device features

		  // initialize Operation mode
		switch(gdwOperationMode)
		{
		case 0:
		    CheckRadioButton(hDlg, IDC_NORMAL, IDC_BURST,IDC_NORMAL);
			break;
		case 1:
		    CheckRadioButton(hDlg, IDC_NORMAL, IDC_BURST,IDC_8255);
			break;
		case 2:
		    CheckRadioButton(hDlg, IDC_NORMAL, IDC_BURST,IDC_BURST);
			break;
		default:
			break;
		}
		switch(gdwDIStartMode)
		{
		case 1:
		    CheckRadioButton(hDlg, IDC_SOFT_START, IDC_MATCH_START,IDC_SOFT_START);
			break;
		case 2:
		    CheckRadioButton(hDlg, IDC_SOFT_START, IDC_MATCH_START,IDC_EXTERNAL_START);
			break;
		case 3:
		    CheckRadioButton(hDlg, IDC_SOFT_START, IDC_MATCH_START,IDC_MATCH_START);
			break;
		default:
			break;
		}
		  // initialize stop type
		switch(gdwDIStopMode)
		{
		case 1:
		    CheckRadioButton(hDlg, IDC_SOFT_STOP, IDC_MATCH_STOP,IDC_SOFT_STOP);
			break;
		case 2:
		    CheckRadioButton(hDlg, IDC_SOFT_STOP, IDC_MATCH_STOP,IDC_EXTERNAL_STOP);
			break;
		case 3:
		    CheckRadioButton(hDlg, IDC_SOFT_STOP, IDC_MATCH_STOP,IDC_MATCH_STOP);
			break;
		default:
			break;
		}

		  // initialize trigger source combo
		  // Add trigger source control selection strings
		  // The trigger source X'tal 30MHz - 10Mhz
		  SendDlgItemMessage(hDlg, IDC_LIST_TRIGGER_SOURCE, LB_ADDSTRING, 0,(LPARAM)((LPSTR)"30 MHz"));
		  SendDlgItemMessage(hDlg, IDC_LIST_TRIGGER_SOURCE, LB_ADDSTRING, 1,(LPARAM)((LPSTR)"15 MHz"));
		  SendDlgItemMessage(hDlg, IDC_LIST_TRIGGER_SOURCE, LB_ADDSTRING, 2,(LPARAM)((LPSTR)"10 MHz"));
		  SendDlgItemMessage(hDlg, IDC_LIST_TRIGGER_SOURCE, LB_ADDSTRING, 3,(LPARAM)((LPSTR)"Counter 0"));
		  SendDlgItemMessage(hDlg, IDC_LIST_TRIGGER_SOURCE, LB_ADDSTRING, 4,(LPARAM)((LPSTR)"External"));

		  // 30MHz(1), 15Mhz(2), 10Mhz(3), Counter 0(4), External (5)
		  SendDlgItemMessage(hDlg, IDC_LIST_TRIGGER_SOURCE, LB_SETCURSEL, gdwDITriggerSource-1, 0);

        // Initialize Counter0 value
        itoa((int)gdwCounterValue[0], szBuffer, 10);
        SendDlgItemMessage(hDlg, IDC_EDIT_COUNTER0_VALUE, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));
		  if ( gdwDITriggerSource != 4 )
		  {
			  EnableWindow( GetDlgItem(hDlg, IDC_EDIT_COUNTER0_VALUE), FALSE);
		  }
        // Initialize pattern Match value
		  sprintf(szBuffer,"%X",gdwPatternMatchValue);
         SetDlgItemText(hDlg,IDC_PATTERN_MATCH,szBuffer);
        // Initialize Repeated Mode
        CheckRadioButton(hDlg, IDC_CYCLIC, IDC_NONCYCLIC,
           (gdwCyclicMode)?IDC_CYCLIC:IDC_NONCYCLIC);

        // Initialize Data Width
        CheckRadioButton(hDlg, IDC_32DI, IDC_8DIO, (IDC_32DI + gdwDataWidth) );

		  // initialize User Buffer
		  // Add trigger source control selection strings		  
		  SendDlgItemMessage(hDlg, IDC_LIST_USER_BUFFER, LB_ADDSTRING, 0,(LPARAM)((LPSTR)"1  Mega"));
		  SendDlgItemMessage(hDlg, IDC_LIST_USER_BUFFER, LB_ADDSTRING, 1,(LPARAM)((LPSTR)"2  Mega"));
		  SendDlgItemMessage(hDlg, IDC_LIST_USER_BUFFER, LB_ADDSTRING, 2,(LPARAM)((LPSTR)"4  Mega"));
		  SendDlgItemMessage(hDlg, IDC_LIST_USER_BUFFER, LB_ADDSTRING, 3,(LPARAM)((LPSTR)"8  Mega"));
		  SendDlgItemMessage(hDlg, IDC_LIST_USER_BUFFER, LB_ADDSTRING, 4,(LPARAM)((LPSTR)"16 Mega "));
		  SendDlgItemMessage(hDlg, IDC_LIST_USER_BUFFER, LB_SETCURSEL, gdwUserBufferSize, 0);

        
        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
			{
				// save the data width
				for ( i = IDC_32DI; i <= IDC_8DIO; i++)
				{
					if( SendDlgItemMessage(hDlg, i, BM_GETCHECK, 0, 0) == 1)
					{
						gdwDataWidth = (i - IDC_32DI);
					}
				}
				
				// save the start type
				for ( i = IDC_SOFT_START; i <= IDC_MATCH_START; i++ )
				{
					if ( SendDlgItemMessage( hDlg, i, BM_GETCHECK, 0, 0 ) == 1 )
					{
						gdwDIStartMode = i - IDC_SOFT_START + 1;
					}
				}
            
				// save the operation mode
				for ( i = IDC_NORMAL; i <= IDC_BURST; i++ )
				{
					if ( SendDlgItemMessage( hDlg, i, BM_GETCHECK, 0, 0 ) == 1 )
					{
						gdwOperationMode = i - IDC_NORMAL ;
					}
				}
            
				// save the stop type
				for ( i = IDC_SOFT_STOP; i <= IDC_MATCH_STOP; i++ )
				{
					if ( SendDlgItemMessage( hDlg, i, BM_GETCHECK, 0, 0 ) == 1 )
					{
						gdwDIStopMode = i - IDC_SOFT_STOP + 1;
					}
				}

				// save the trigger source
				// 30MHz(1), 15Mhz(2), 10Mhz(3), Counter 0(4), External (5)
			   gdwDITriggerSource = SendDlgItemMessage(hDlg, IDC_LIST_TRIGGER_SOURCE, LB_GETCURSEL, 0, 0) + 1;

				// save the counter0 value 
				SendDlgItemMessage(hDlg, IDC_EDIT_COUNTER0_VALUE, WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
				gdwCounterValue[0] = (DWORD)atoi(szBuffer);

 				// save the pattern match value 
 				sprintf(szBuffer,"%s","0x");
                GetDlgItemText(hDlg,IDC_PATTERN_MATCH,&(szBuffer[2]),10);
				gdwPatternMatchValue=strtol(szBuffer,&szTemp,16);

				// save user buffer size
				// 1M(0), 2M(1), 4M(2), 8M(3), 16M(4)
				gdwUserBufferSize = SendDlgItemMessage(hDlg, IDC_LIST_USER_BUFFER, LB_GETCURSEL, 0, 0);

				DRV_DeviceClose((LONG far *)&DriverHandle);
			}

         case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

		  case  IDC_LIST_TRIGGER_SOURCE:
			  {
				  if( HIWORD(wParam) == LBN_SELCHANGE )
				  {

					  if( SendDlgItemMessage(hDlg, IDC_LIST_TRIGGER_SOURCE, LB_GETCURSEL ,0,0) == 3 )
						 bEnable = TRUE;
					  else 
						 bEnable = FALSE;

					  EnableWindow( GetDlgItem(hDlg, IDC_EDIT_COUNTER0_VALUE), bEnable);
				  }

				}
			  break;

        case IDC_DeviceSelect:

            // Step 1: Select Device
            ErrCde = DRV_SelectDevice(hDlg, FALSE, &dwDeviceNum, szDeviceName);
            if (ErrCde != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                return 0;
            }
	        SetDlgItemText(hDlg,IDC_DEVICE_NAME,szDeviceName);

            // -------------------------------------------------
            // Initialize Input Range List Combobox, it needs to
            // get device features for gain list
            // -------------------------------------------------

            // Step 2: Open Device
            ErrCde = DRV_DeviceOpen(dwDeviceNum, (LONG far *)&DriverHandle);
            if (ErrCde != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd, (LPCSTR)szErrMsg,
                    "Driver Message", MB_OK);
                return 0;
            }

            // Step 3: Get device features
            ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
            ptDevFeatures.size = sizeof(DEVFEATURES);
            if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd, (LPCSTR)szErrMsg, "Driver Message",
                    MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }



            // Step 5: Close device
            DRV_DeviceClose((LONG far *)&DriverHandle);

            return TRUE;

        case IDC_CYCLIC :
            gdwCyclicMode = 1;
            break;
        case IDC_NONCYCLIC :
            gdwCyclicMode = 0;
            break;
        
		
        }
        break;
    }
    return FALSE ;
}

/******************************************************************************
    FUNCTION: ShowDataDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for acquired data
******************************************************************************/

BOOL FTYPE ShowDataDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                                  /* window handle                 */
unsigned message;                           /* type of message               */
WPARAM wParam;                              /* additional information        */
LPARAM lParam;                              /* additional information        */
{
    static  LONG  lEndDisplay, lStartDisplay;
    LONG  i;
    DWORD  dwUserBufferSize, dwCount;
    
    switch (message)
    {
    case WM_INITDIALOG :
		{         
         dwUserBufferSize = (DWORD)(Mega * pow((double)2, (double)gdwUserBufferSize));
         // 32DI(0)/32DO(1)/ 16 DIO(2) / 8DIO(3)
         switch( gdwDataWidth )
         {
         case 3:
            dwCount = dwUserBufferSize;
            break;

         case 2:
            dwCount = dwUserBufferSize / 2;
            break;

         default:
            dwCount = dwUserBufferSize / 4;
            break;
         }
         
         lStartDisplay = gdwStartPt;
         ultoa(lStartDisplay, szBuffer, 10);
         SetWindowText(GetDlgItem(hDlg,IDC_FROM),(LPCTSTR)szBuffer);

         lEndDisplay = gdwStopPt;
         ultoa(lEndDisplay, szBuffer, 10);
         SetWindowText(GetDlgItem(hDlg,IDC_TO),(LPCTSTR)szBuffer);
         
         ultoa(dwCount, szBuffer, 10);
         SetWindowText(GetDlgItem(hDlg,IDC_TOTAL),(LPCTSTR)szBuffer);
         
         for(i = lStartDisplay; i <= lEndDisplay; ++i)
         {
            if ( gdwDataWidth == 3 )     // 8-bit
            {
               sprintf(szBuffer, "Buf[%ld] = %2x", i, ((BYTE far *)hUserBuf)[i]);
            }
            else if ( gdwDataWidth == 2 )  // 16-bit
            {
               sprintf(szBuffer, "Buf[%ld] = %4x",i, ((USHORT far *)hUserBuf)[i]);
            }
            else                           // 32-bit
            {
               sprintf(szBuffer, "Buf[%ld] = %8x",i, ((ULONG far *)hUserBuf)[i]);
            }


            if(SendDlgItemMessage(hDlg, IDC_DATALIST, LB_ADDSTRING, 0,
               (LPARAM)(LPSTR)szBuffer) == LB_ERRSPACE)
            {
               MessageBox(hWnd,(LPCSTR)"Not Enough Memory",
                  "Display Data",MB_OK);
               return TRUE;
            }
         }

        return TRUE;
		}

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDC_BACKBUTTON :
           break;			
        case IDC_BACKWARD :
			  break;
		  case IDC_FORWARD :
           break;			
        case IDC_FORWARDBUTTON :
           break;
        case IDC_UPDATE :
			 break;
        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;
        }
        break;
    }
    return FALSE ;
}

/******************************************************************************
    FUNCTION: DisplayDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for display range
******************************************************************************/

BOOL FTYPE DisplayDlgProc(hDlg, message, wParam, lParam)
HWND hDlg;                               /* window handle                   */
unsigned message;                        /* type of message                 */
WPARAM wParam;                           /* additional information          */
LPARAM lParam;                           /* additional information          */
{

    switch (message)
    {
    case WM_INITDIALOG :

        // Initialize Start Point
        ultoa(gdwStartPt, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTAPT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        // Initialize Stop Point
        ultoa(gdwStopPt, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ESTOPT, EM_REPLACESEL, 0,
            (LPARAM)((LPSTR)szBuffer));

        return TRUE;

    case WM_COMMAND :
        switch (LOWORD(wParam))
        {
        case IDOK :
        case IDCANCEL :
            EndDialog(hDlg, 0);
            return TRUE;

        case IDC_ESTAPT :

            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTAPT,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTAPT,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gdwStartPt = (USHORT)atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTAPT,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;

        case IDC_ESTOPT :
            if (HIWORD(wParam) == EN_KILLFOCUS)
            {
                if(SendDlgItemMessage(hDlg, IDC_ESTOPT,
                    EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_ESTOPT,
                        WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gdwStopPt = atol(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ESTOPT,
                        EM_SETMODIFY, FALSE, 0) ;
                }
            }
            return TRUE;
        }
        break;
    }
    return FALSE ;
}

/******************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
******************************************************************************/

long FTYPE MainWndProc(hWnd, message, wParam, lParam)
HWND     hWnd;                            /* window handle                   */
unsigned message;                         /* type of message                 */
WPARAM   wParam;                          /* additional information          */
LPARAM   lParam;                          /* additional information          */
{

    TEXTMETRIC tm;
    HDC     hdc;
    static  int cxChar, cyChar;
    DWORD  dwUserBufferSize;
    DWORD  dwCount;
    ULONG  dwFdiStatus, dwRetrieved;
    char    *szStr, szDest[100], szTemp[50];
    double  ratio;
    int     dec, sign;
//    USHORT  temp;


    switch (message)
    {
    case WM_CREATE:

        hMenu = GetMenu(hWnd);

        hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

        lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

        lpfnShowDataDlgProc = MakeProcInstance (ShowDataDlgProc, hInstance) ;

        lpfnDisplayDlgProc = MakeProcInstance (DisplayDlgProc, hInstance) ;
        
        hdc = GetDC(hWnd);
        SelectObject(hdc, GetStockObject(SYSTEM_FIXED_FONT));
        GetTextMetrics(hdc, &tm);
        cxChar = tm.tmAveCharWidth;
        cyChar = tm.tmHeight + tm.tmExternalLeading;
        ReleaseDC(hWnd, hdc);

        //Create stop button and status button
        hwndStopButton = CreateWindow("button", "Stop",
            (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
            0, 0, 8*cxChar, 3*cyChar/2, hWnd,
            (HMENU)IDC_STOPBUTTON,
            (HANDLE)hInstance, (LPVOID)NULL);

        hwndStatusButton = CreateWindow("button", "Status",
            (DWORD)(WS_CHILD | WS_VISIBLE | BS_PUSHBUTTON),
            8*cxChar, 0, 8*cxChar, 3*cyChar/2, hWnd,
            (HMENU)IDC_STATUSBUTTON,
            (HANDLE)hInstance, (LPVOID)NULL);

        EnableWindow(hwndStopButton, FALSE);
        EnableWindow(hwndStatusButton, FALSE);

        return 0 ;

    case WM_COMMAND:     /* message: from application menu */

        hMenu = GetMenu(hWnd);

        switch(LOWORD(wParam))
        {
            // Show the dialog box of the input parameter
        case IDM_SETTING :

            DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                hWnd, lpfnConfigDlgProc) ;
            return 0;

            // Show the dialog box for the range of data to display
        case IDM_DISPLAY:

            DialogBox (hInstance, MAKEINTRESOURCE(IDD_DISPLAY),
                hWnd, lpfnDisplayDlgProc) ;

            return 0;

        case IDM_RUN :

            // Step 1: Device open
            ErrCde = DRV_DeviceOpen(dwDeviceNum, (LONG far *)&DriverHandle);
            if (ErrCde != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                return 0;
            }

            // Step 2: Get device features
            ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
            ptDevFeatures.size = sizeof(DEVFEATURES);
            if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            // Step 3: Allocate memory for Fast DI data
            //********Version 2.0B*******************
            //  As with DMA transfers, it's better to start the buffer at
            //  page boundary(4096 Bytes). If the buffers used for the 
            //  DMA operation is aligned on addresses of page boundary, user 
            //  can get exact page change event(just set the Count of 
            //  PT_EnableEvent to 2048, it means 1 page). Once Interrupt
            //  event occurs, it means 2048 samples(4096 byes) are available.
            //  
            //  It is why we changed the memory allocation policy from
            //  Solution A to solution B at Version 2.0B.
            //
            //  At the same time, the buffer that receives the samples must
            //  be a multiple of 4096 bypes(2048 samples). In cycle mode, if
            //  you are interested in buffer change event, the minimal buffer
            //  is 8192 bytes(4096 samples).
            //****************************************
            
            //*************** 2.0B note *********************
            // Device driver's ReadFile will Error(System resource insufficient
            // when use GlobalAlloc function to allocate more than 16M byte memory.
            // So we use VirtualAlloc, but not test in Win9x platform.

            dwUserBufferSize = (DWORD)(Mega * pow((double)2, (double)gdwUserBufferSize));

            /*
            // Solution A:
            if((hUserBuf = GlobalAlloc(GPTR,
                 dwUserBufferSize )) == 0)
            {
                MessageBox(hWnd,"Not enough memory for buffer ",
                    "High Speed",MB_OK);
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }
            */
               
            // Solution B:
            // Win9x seems don't support the routine well.
            // so it's recommended only to use it at WinNT/2K.
            hUserBuf = VirtualAlloc( NULL,                    // region to reserve or commit
                                     dwUserBufferSize,          // size of region
                                     MEM_COMMIT,  // type of allocation
                                     PAGE_READWRITE );        // type of access protection
            
            if ( NULL==hUserBuf )                       
            {
                MessageBox( hWnd, "Not enough memory for buffer", "High Speed",MB_OK );
                DRV_DeviceClose((LONG *)&DriverHandle);
                return 0;
            }
            // End of solution B.
            

            // Step 4: Enable event feature
            ptEnableEvent.Enabled = 1;
            ptEnableEvent.Count = 1;

            ptEnableEvent.EventType = ADS_EVT_DI_OVERRUN;    // Overrun event 
            ErrCde = DRV_EnableEvent(DriverHandle, &ptEnableEvent);
            ptEnableEvent.EventType = ADS_EVT_DI_LOBUFREADY; // Low buffer ready event
            ErrCde = DRV_EnableEvent(DriverHandle, &ptEnableEvent);
            ptEnableEvent.EventType = ADS_EVT_DI_HIBUFREADY; // High buffer ready event.
            ErrCde = DRV_EnableEvent(DriverHandle, &ptEnableEvent);
            ptEnableEvent.EventType = ADS_EVT_DI_TERMINATED; // Fast DI function terminated
            ErrCde = DRV_EnableEvent(DriverHandle, &ptEnableEvent);

            if (ErrCde != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                // GlobalFree( hUserBuf );                
                // Decommit first, then release the memory
                VirtualFree( hUserBuf, dwUserBufferSize, MEM_DECOMMIT ); 
                VirtualFree( hUserBuf, 0, MEM_RELEASE);
                            
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }
            // 32DI(0)/32DO(1)/ 16 DIO(2) / 8DIO(3)
            switch( gdwDataWidth )
            {
            case 3:
               dwCount = dwUserBufferSize;
               break;

            case 2:
               dwCount = dwUserBufferSize / 2;
               break;

            default:
               dwCount = dwUserBufferSize / 4;
               break;
            }
            ErrCde = DRV_DeviceSetProperty(DriverHandle, CFG_DiPatternMatchValue, &gdwPatternMatchValue, sizeof(DWORD));
  			ErrCde = DRV_DeviceSetProperty(DriverHandle, CFG_CounterCountValue, &gdwCounterValue, 3*sizeof(DWORD));
  			ErrCde = DRV_DeviceSetProperty(DriverHandle, CFG_DiOperationMode, &gdwOperationMode, sizeof(DWORD));
            ErrCde = DRV_DeviceSetProperty(DriverHandle, CFG_DioFdioDirection, &gdwDataWidth, sizeof(DWORD));
			ErrCde = DRV_DeviceSetProperty(DriverHandle, CFG_DiStartMethod, &gdwDIStartMode, sizeof(DWORD));
			ErrCde = DRV_DeviceSetProperty(DriverHandle, CFG_DiStopMethod, &gdwDIStopMode, sizeof(DWORD));				
			ErrCde = DRV_DeviceSetProperty(DriverHandle, CFG_DiPacerSource, &gdwDITriggerSource, sizeof(DWORD));
            if (ErrCde != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                // GlobalFree( hUserBuf );
                // Decommit first, then release the memory
                VirtualFree( hUserBuf, dwUserBufferSize, MEM_DECOMMIT ); 
                VirtualFree( hUserBuf, 0, MEM_RELEASE);
                            
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }


            // Step 5: Start fast DI
            ErrCde = DRV_FDIStart(
               DriverHandle,   //Device handle
               gdwCyclicMode,  //Cyclic(1)/non cyclic
               dwCount,        //Data counts 
               hUserBuf );     //Data buffer.
                          
            if (ErrCde != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                // GlobalFree( hUserBuf );
                // Decommit first, then release the memory
                VirtualFree( hUserBuf, dwUserBufferSize, MEM_DECOMMIT ); 
                VirtualFree( hUserBuf, 0, MEM_RELEASE);
                
                DRV_DeviceClose((LONG far *)&DriverHandle);
                return 0;
            }

            
            hThreadHandle = CreateThread(0, 0,
                (LPTHREAD_START_ROUTINE) UserThread,
                0, 0, (LPDWORD)&dwThreadID);
			
			   bThreadloop = TRUE;
            

            Sleep(0);

            EnableMenuItem(hMenu, IDM_RUN, MF_DISABLED | MF_BYCOMMAND);

            // Enable stop and status buttons
            if( gdwCyclicMode )
            {
               EnableWindow( hwndStatusButton, TRUE );
               EnableWindow( hwndStopButton, TRUE );
            }
            else
            {
               EnableWindow( hwndStatusButton, FALSE );
               EnableWindow( hwndStopButton, TRUE );
            }

            return 0;
            //end IDM_RUN

        case IDC_STOPBUTTON:
        {
           BOOL bflag = TRUE;
           // Close thread
           bThreadloop = FALSE;
           Sleep(100);

           // stop fdi function 
           adTerminateEvent();
                      
           EnableWindow(hwndStopButton, FALSE);
           EnableWindow(hwndStatusButton, FALSE);
			 
           // Step 3: Free buffer
           
           // GlobalFree(hUserBuf);     
           
           // Decommit first, then release the memory
           VirtualFree( hUserBuf, (DWORD)(Mega * pow((double)2, (double)gdwUserBufferSize)), MEM_DECOMMIT ); 
           VirtualFree( hUserBuf, 0, MEM_RELEASE);
           
           // Step 4: Close driver
           if ( DriverHandle )
           {
              DRV_DeviceClose((LONG far *)&DriverHandle);
           }
           
           EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);

           return 0;
        }

        case IDC_STATUSBUTTON:
		  {
           ErrCde = DRV_FDICheck( DriverHandle, &dwFdiStatus, &dwRetrieved); 

           // Display the status of the A/D conversion
           sprintf(szDest, "Cyclic Counts: %d\n", gdwDIBufferChangeCounter / 2);
           
           if ( dwFdiStatus & 0x1 )       // Running
           {
              strcpy(szTemp, "Running\n");
           }
           else                           // Stopped
           {
              strcpy(szTemp, "FDI: Stopped\n");
           } 
           strcat(szDest, szTemp);  
           dwUserBufferSize = (DWORD)(Mega * pow((double)2, (double)gdwUserBufferSize));
           
           // 32DI(0)/32DO(1)/ 16 DIO(2) / 8DIO(3)
           switch( gdwDataWidth )
           {
           case 3:
              dwCount = dwUserBufferSize;
              break;
           case 2:
              dwCount = dwUserBufferSize / 2;
              break;
           default:
              dwCount = dwUserBufferSize / 4;
              break;
           }
        
           ratio = 100.0 * dwRetrieved / dwCount;
           szStr = fcvt((double)ratio, 0, &dec, &sign);           
           strcat(szDest, "Finish:  ");    
           strcat(szDest, szStr);         
           strcat(szDest, "%...");
           MessageBox(hWnd,(LPCSTR)szDest, "Check Status", MB_OK);
           
        }
        return 0;
      }						
    case WM_CLOSE:
		if( IsWindowEnabled(hwndStopButton))
		{
			MessageBox(hWnd,"The program is running, please push the stop button to stop. ",
				"Close", MB_ICONEXCLAMATION | MB_ICONWARNING);
			return 0;
		}
	      
    case WM_DESTROY:

        PostQuitMessage(0);
        break;

    default:
        return (DefWindowProc(hWnd, message, wParam, lParam));
    }

    return ((LONG)NULL);
}

//------------------------------------------------------------------
void UserThread()
{
    USHORT usEventType;
    LONG  ErrCde;

    gdwDIBufferChangeCounter = 0; 
    gdwDIOverRunCounter = 0;
    gdwDITerminateCounter = 0;

    hdc = GetDC( hWnd );
    GetClientRect( hWnd, &rect );
    InvalidateRect( hWnd, NULL, TRUE );
    UpdateWindow( hWnd );

    while(bThreadloop)
    {
        // Check message
        ptCheckEvent.EventType = &usEventType;
        ptCheckEvent.Milliseconds = gulOverrunTime;

		  bThreadflag = TRUE;
         DRV_CheckEvent(DriverHandle,(LPT_CheckEvent)&ptCheckEvent);

        if ( usEventType ==  ADS_EVT_DI_LOBUFREADY )  // Driver finish the fill low buffer data 
        {            
           DRV_ClearFlag( DriverHandle, ADS_EVT_DI_OVERRUN);
		   sprintf(szBuffer,"%s","Transfer Buffer = Low half buffer      ");
           gdwDIBufferChangeCounter++;            
        }
        else if( usEventType ==  ADS_EVT_DI_HIBUFREADY )
        {         
           DRV_ClearFlag( DriverHandle, ADS_EVT_DI_OVERRUN);
		   sprintf(szBuffer,"%s","Transfer Buffer = Higher half buffer    ");
           gdwDIBufferChangeCounter++;           
        }
        else if ( usEventType == ADS_EVT_DI_TERMINATED )
        {
           gdwDITerminateCounter++;
           bThreadloop = FALSE;             // Stop the thread
           bThreadflag = FALSE; 
           sprintf( szCounter, "Buffer change Counter = %4d\n", gdwDIBufferChangeCounter );
           TextOut( hdc, 300, 50, szBuffer, 38 );
           TextOut( hdc, 300, 70, szCounter, 28 );
           sprintf( szCounter, "Overrun Counter = %4d\n", gdwDIOverRunCounter );
           TextOut( hdc, 300, 90, szCounter, 22 );
           sprintf( szCounter, "Terminate Counter = %4d\n",gdwDITerminateCounter );
           TextOut( hdc, 300, 110, szCounter, 24 );

           UpdateWindow( hWnd );
           adTerminateEvent();           
           EnableWindow(hwndStopButton, FALSE);
			  EnableWindow(hwndStatusButton, FALSE);           
        }
        else if ( usEventType == ADS_EVT_DI_OVERRUN )
        {
           DRV_ClearFlag( DriverHandle, ADS_EVT_DI_OVERRUN);
           gdwDIOverRunCounter++;           
        }    

		  bThreadflag = FALSE; 
        sprintf( szCounter, "Buffer change Counter = %4d\n", gdwDIBufferChangeCounter );
           TextOut( hdc, 300, 50, szBuffer, 38 );
        TextOut( hdc, 300, 70, szCounter, 28 );
        sprintf( szCounter, "Overrun Counter = %4d\n", gdwDIOverRunCounter );
        TextOut( hdc, 300, 90, szCounter, 22 );
        sprintf( szCounter, "Terminate Counter = %4d\n",gdwDITerminateCounter );
        TextOut( hdc, 300, 110, szCounter, 24 );

        UpdateWindow( hWnd );
    }

    bThreadflag = FALSE; 
    ReleaseDC( hWnd, hdc );
}

//-----------------------------------------------------------------------------
void adTerminateEvent()
{
	DWORD ExitCode;
   if ( !DriverHandle )
   {
      return;
   }

   ErrCde = DRV_FDIStop(DriverHandle);
   if (ErrCde != 0)
   {
      DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
      MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
      return ;
   }

   //
   // Disable open Event
   //
   ptEnableEvent.Enabled = 0;
   ptEnableEvent.EventType = ADS_EVT_DI_OVERRUN;    // Overrun event 
   DRV_EnableEvent(DriverHandle, &ptEnableEvent);
   ptEnableEvent.EventType = ADS_EVT_DI_LOBUFREADY; // Low buffer ready event
   DRV_EnableEvent(DriverHandle, &ptEnableEvent);
   ptEnableEvent.EventType = ADS_EVT_DI_HIBUFREADY; // High buffer ready event.
   DRV_EnableEvent(DriverHandle, &ptEnableEvent);
   ptEnableEvent.EventType = ADS_EVT_DI_TERMINATED; // Fast DI function terminated
   DRV_EnableEvent(DriverHandle, &ptEnableEvent);

   GetExitCodeThread(hThreadHandle,&ExitCode);
   if(ExitCode==STILL_ACTIVE)
	   TerminateThread(hThreadHandle,ExitCode);
   ErrCde = DRV_DeviceClose(&DriverHandle);   
   if (ErrCde != 0)
   {
      DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
      MessageBox(hWnd,(LPCSTR)szErrMsg,"Thread Message",MB_OK);
      return ;
   }

    // Display data
    DialogBox (hInstance, MAKEINTRESOURCE(IDD_DATA), hWnd, lpfnShowDataDlgProc) ;

    // Free buffer
    // GlobalFree(hUserBuf);

    // Decommit first, then release the memory
    VirtualFree( hUserBuf, (DWORD)(Mega * pow((double)2, (double)gdwUserBufferSize)), MEM_DECOMMIT ); 
    VirtualFree( hUserBuf, 0, MEM_RELEASE);

    
    ErrCde = GetLastError();

    // Close driver
    if ( DriverHandle )
    {
       DRV_DeviceClose((LONG far *)&DriverHandle);
    }

    EnableMenuItem(hMenu, IDM_RUN, MF_ENABLED | MF_BYCOMMAND);

    return;
}
