/*
 ***********************************************************************
 *  Program     : DIGIN.C                                              *
 *  Description : Demo program for digital input                       *
 *  Revision    : 1.00                                                 *
 *  Date        : 9/5/1996                     Advantech Co., Ltd.     *
 ***********************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance() - Saves instance handle and creates main window
 *
 *   MainWndProc() - Process main window messages
 *
 *   ConfigDlgProc() - Process the dialog box messages for configuration
 */


#include <windows.h>
#include <stdlib.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\..\include\driver.h"

#include "resource.h"

// ------------------------------------------------------------------------
// CONSTANT DEFINITION
// ------------------------------------------------------------------------
#define     MAX_DEVICES     100


// ------------------------------------------------------------------------
// GLOBAL VARIABLES
// ------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hMainWnd;                   // main window handle

static      DEVFEATURES             DevFeatures;    // structure for device features
static      PT_DeviceGetFeatures    ptDevFeatures;
static      PT_DioReadPortByte      ptDioReadPortByte;
static      PT_EnableEventEx        ptEnableEventEx;     // Enable event
static      PT_EnableEvent          ptEnableEvent;       // Enable event
static      PT_FAICheck             ptFAICheck;          // FAICheck table
static      PT_CheckEvent           ptCheckEvent;        // Check event
static      PT_FDITransfer          ptFDITransfer;
static      ThreadhWnd;    

HANDLE      hThreadHandle;
DWORD       dwThreadID;

char        szErrMsg[80];               // Use for MESSAGEBOX function
LRESULT     ErrCde;                     // Return error code

//
// used for dynamic allocation memory for the installed devices
//
// static  HGLOBAL     hDeviceList,hSubDeviceList;
// static  LPDEVLIST   DeviceList,SubDeviceList;
//

//
// used for fixed memory for the installed devices
//
DEVLIST     DeviceList[MAX_DEVICES];
DEVLIST     SubDeviceList[MAX_DEVICES];

LONG        DriverHandle = (LONG)NULL;          // driver handle
BOOL        bRun = FALSE;                       // flag for running
USHORT      gwDevice = 0, gwSubDevice = 0;      // device index
USHORT      gwChannel = 0;                      // input channel
USHORT      gwValue;                            // input value
SHORT       gnNumOfDevices, gnNumOfSubdevices;  // number of installed devices
USHORT      gwScanTime = 1000;                  // scan time
DWORD       gdwStartTime;                       // start time
DWORD       gdwElapseTime = 0;                  // elapse time
USHORT      gwOverrunCount = 0;                 // overrun count
FARPROC     lpfnConfigDlgProc;                  // config. dialog procedure
FARPROC     lpfnScanDlgProc;                    // scan dialog procedure
FARPROC     lpfnConfigPatternProc;              // Pattern dialog procedure
FARPROC     lpfnConfigStatusProc;               // Status dialog procedure
FARPROC     lpfnConfigExpPatternProc;           // Expansion board Pattern dialog procedure
FARPROC     lpfnConfigExpStatusProc;            // Expansion board Status dialog procedure

USHORT      usPEnable = 0, usPValue = 0;
USHORT      usSEnable = 0, usIEnable = 0;
USHORT      m_bContinue = FALSE;
USHORT      usSCount=0,usPCount=0, usICount=0;
BOOL        bThreadflag = FALSE;

// ------------------------------------------------------------------------
// FUNCTION DECLARATION
// ------------------------------------------------------------------------
BOOL InitApplication(HANDLE);
BOOL InitInstance(HANDLE, int);
long FTYPE MainWndProc(HWND, UINT, WPARAM, LPARAM);
int  FTYPE WinMain(HINSTANCE, HINSTANCE, LPSTR, int);

void UserThread();
void adPatternMatchEvent();
void adInterruptEvent();
void adStatusChangeEvent();


/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HINSTANCE hInstance;                          /* current instance             */
HINSTANCE hPrevInstance;                      /* previous instance            */
LPSTR lpCmdLine;                           /* command line                 */
int nCmdShow;                              /* show-window type (open/icon) */
{
    MSG msg;                               /* message                      */

    if (!hPrevInstance)                /* Other instances of app running?  */
      if (!InitApplication(hInstance)) /* Initialize shared things         */
          return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,      /* message structure                      */
        (HWND)NULL,              /* handle of window receiving the message */
        (UINT)NULL,              /* lowest message to examine              */
        (UINT)NULL))             /* highest message to examine             */
    {
        TranslateMessage(&msg);  /* Translates virtual key codes           */
        DispatchMessage(&msg);   /* Dispatches message to window           */
    }
    return (msg.wParam);         /* Returns the value from PostQuitMessage */
}

/****************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                            /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW; /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;   /* window process procedure   */
    wc.cbClsExtra = 0;                  /* No per-class extra data.        */
    wc.cbWndExtra = 0;                  /* No per-window extra data.       */
    wc.hInstance = hInstance;           /* Application that owns the class.*/
    wc.hIcon = LoadIcon(hInstance, "IDI_ICON1");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";   /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";  /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));

}

/***************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/

BOOL InitInstance
(
    HANDLE          hInstance,          // Current instance identifier.
    int             nCmdShow            // Param for first ShowWindow() call.
)
{
    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hMainWnd = CreateWindow(
        "MyClass",                 /* See RegisterClass() call.           */
        "Advantech Driver Demo : Digital Input with Interrupt for PCI-1753" , /* Window title bar  */
        WS_OVERLAPPEDWINDOW,        /* Window style.                       */
        CW_USEDEFAULT,              /* Default horizontal position.        */
        CW_USEDEFAULT,              /* Default vertical position.          */
        CW_USEDEFAULT,              /* Default width.                      */
        CW_USEDEFAULT,              /* Default height.                     */
        NULL,                       /* Overlapped windows have no parent.  */
        NULL,                       /* Use the window class menu.          */
        hInstance,                  /* This instance owns this window.     */
        NULL                        /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hMainWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hMainWnd, nCmdShow);   // Show the window
    UpdateWindow(hMainWnd);           // Sends WM_PAINT message
    return (TRUE);                // Returns the value from PostQuitMessage
}

/***************************************************************************
    FUNCTION: ConfigPatternProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/
BOOL FTYPE ConfigPatternProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    USHORT      i;

    switch (message)
    {
    case WM_INITDIALOG :
	  for(i=0; i<8; i++)
	  {
		 EnableWindow(GetDlgItem(hDlg,(IDC_Pattern0 + i)),TRUE);

		 if(usPEnable & (0x01 << i))
		 {
		    CheckDlgButton(hDlg,(IDC_Pattern0 + i),TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_PB0HI + (i*2)),TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_PB0LOW + (i*2)),TRUE);
			if(usPValue & (0x01 << i))
			   CheckDlgButton(hDlg,(IDC_PB0HI + (i*2)),1);
			else
			   CheckDlgButton(hDlg,(IDC_PB0LOW + (i*2)),1);
		 } else	 {
		    CheckDlgButton(hDlg,(IDC_Pattern0 + i),FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_PB0HI + i*2),FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_PB0LOW + i*2),FALSE);
		 }
	  }

        return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
         {
		    case IDC_Pattern0:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern0))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB0HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB0LOW),TRUE);
		            CheckDlgButton(hDlg,IDC_PB0HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB0LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB0HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB0LOW),FALSE);
				 }
				 break;
		    case IDC_Pattern1:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern1))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB1HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB1LOW),TRUE);
		            CheckDlgButton(hDlg,IDC_PB1HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB1LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB1HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB1LOW),FALSE);
				 }
				 break;
		    case IDC_Pattern2:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern2))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB2HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB2LOW),TRUE);
		            CheckDlgButton(hDlg,IDC_PB2HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB2LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB2HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB2LOW),FALSE);
				 }
				 break;
		    case IDC_Pattern3:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern3))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB3HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB3LOW),TRUE);
		            CheckDlgButton(hDlg,IDC_PB3HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB3LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB3HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB3LOW),FALSE);
				 }
				 break;
		    case IDC_Pattern4:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern4))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB4HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB4LOW),TRUE);
		            CheckDlgButton(hDlg,IDC_PB4HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB4LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB4HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB4LOW),FALSE);
				 }
				 break;
		    case IDC_Pattern5:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern5))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB5HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB5LOW),TRUE);
		            CheckDlgButton(hDlg,IDC_PB5HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB5LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB5HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB5LOW),FALSE);
				 }
				 break;
		    case IDC_Pattern6:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern6))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB6HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB6LOW),TRUE);
		            CheckDlgButton(hDlg,IDC_PB6HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB6LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB6HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB6LOW),FALSE);
				 }
				 break;
		    case IDC_Pattern7:
                 if(IsDlgButtonChecked(hDlg, IDC_Pattern7))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB7HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB7LOW),TRUE);
		            CheckDlgButton(hDlg,IDC_PB7HI,TRUE);	 
			        CheckDlgButton(hDlg,IDC_PB7LOW,FALSE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_PB7HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_PB7LOW),FALSE);
				 }
				 break;

			case IDC_PTNOPTION:
                 DialogBox (hInst, MAKEINTRESOURCE(IDD_EXPPATTERN),
                            hDlg, lpfnConfigExpPatternProc) ;
				 break;

            case IDOK :
				 usPEnable &= 0xff00;
				 usPValue &= 0xff00;
		         for(i=0; i<8; i++)
		            if(IsDlgButtonChecked(hDlg,IDC_Pattern0 + i))
					{
					 usPEnable |= (0x01 << i);
					 if(IsDlgButtonChecked(hDlg,IDC_PB0HI + (i*2)))
						 usPValue |= (0x01 << i);
					}

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}
/***************************************************************************
    FUNCTION: ConfigExpPatternProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ConfigExpPatternProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    USHORT      i;

    switch (message)
    {
    case WM_INITDIALOG :
	  for(i=0; i<8; i++)
	  {
		 EnableWindow(GetDlgItem(hDlg,(IDC_ExpPattern0 + i)),TRUE);

		 if(usPEnable & (0x01 << (i+8)))
		 {
		    CheckDlgButton(hDlg,(IDC_ExpPattern0 + i),TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_ExpPB0HI + i),TRUE);
			EnableWindow(GetDlgItem(hDlg,IDC_ExpPB0LOW + i),TRUE);
			if(usPValue & (0x01 << i))
			   CheckDlgButton(hDlg,(IDC_ExpPB0HI + i),1);
			else
			   CheckDlgButton(hDlg,(IDC_ExpPB0LOW + i),1);
		 } else	 {
		    CheckDlgButton(hDlg,(IDC_ExpPattern0 + i),FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_ExpPB0HI + i),FALSE);
			EnableWindow(GetDlgItem(hDlg,IDC_ExpPB0LOW + i),FALSE);
		 }
	  }

        return TRUE;

    case WM_COMMAND :

         switch (LOWORD(wParam))
         {
		    case IDC_ExpPattern0:
                 if(IsDlgButtonChecked(hDlg, IDC_ExpPattern0))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB0HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB0LOW),TRUE);
			        CheckDlgButton(hDlg,IDC_ExpPB0HI,TRUE);	 
				 }
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB0HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB0LOW),FALSE);}
				 break;
		    case IDC_ExpPattern1:
                 if(IsDlgButtonChecked(hDlg, IDC_ExpPattern1))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB1HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB1LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_ExpPB1HI,TRUE);}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB1HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB1LOW),FALSE);}
				 break;
		    case IDC_ExpPattern2:
                 if(IsDlgButtonChecked(hDlg, IDC_ExpPattern2))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB2HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB2LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_ExpPB2HI,TRUE);}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB2HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB2LOW),FALSE);}
				 break;
		    case IDC_ExpPattern3:
                 if(IsDlgButtonChecked(hDlg, IDC_ExpPattern3))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB3HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB3LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_ExpPB3HI,TRUE);}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB3HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB3LOW),FALSE);}
				 break;
		    case IDC_ExpPattern4:
                 if(IsDlgButtonChecked(hDlg, IDC_ExpPattern4))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB4HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB4LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_ExpPB4HI,TRUE);}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB4HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB4LOW),FALSE);}
				 break;
		    case IDC_ExpPattern5:
                 if(IsDlgButtonChecked(hDlg, IDC_ExpPattern5))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB5HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB5LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_ExpPB5HI,TRUE);}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB5HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB5LOW),FALSE);}
				 break;
		    case IDC_ExpPattern6:
                 if(IsDlgButtonChecked(hDlg, IDC_ExpPattern6))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB6HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB6LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_ExpPB6HI,TRUE);}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB6HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB6LOW),FALSE);}
				 break;
		    case IDC_ExpPattern7:
                 if(IsDlgButtonChecked(hDlg, IDC_ExpPattern7))
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB7HI ),TRUE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB7LOW),TRUE);
				    CheckDlgButton(hDlg,IDC_ExpPB7HI,TRUE);}
		         else
				 {  EnableWindow(GetDlgItem(hDlg,IDC_ExpPB7HI ),FALSE);
			        EnableWindow(GetDlgItem(hDlg,IDC_ExpPB7LOW),FALSE);}
				 break;
            case IDOK :
				 usPEnable &= 0x00ff;
				 usPValue &= 0x00ff;
		         for(i=0; i<8; i++)
		            if(IsDlgButtonChecked(hDlg,IDC_ExpPattern0 + i))
					{
					 usPEnable |= (0x01 << (i+8));
					 if(IsDlgButtonChecked(hDlg,IDC_ExpPB0HI + (i*2)))
						 usPValue |= (0x01 << (i+8));
					}

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}
/***************************************************************************
    FUNCTION: ConfigStatusProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/
BOOL FTYPE ConfigStatusProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    USHORT   i;

    switch (message)
    {
    case WM_INITDIALOG :
	  for(i=0; i<8; i++)
	  {
		 EnableWindow(GetDlgItem(hDlg,(IDC_Status0 + i)),TRUE);
		 if(usSEnable & (0x01 << i))
		    CheckDlgButton(hDlg,(IDC_Status0 + i),TRUE);
		 else
		    CheckDlgButton(hDlg,(IDC_Status0 + i),FALSE);
	  }
      return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
         {
		    case IDC_STAOPTION:
                 DialogBox (hInst, MAKEINTRESOURCE(IDD_EXPSTATUS),
                            hDlg, lpfnConfigExpStatusProc) ;
				 break;

            case IDOK :
				 usSEnable &= 0xff00;// usSRiseEdge &= 0xff00; usSFallEdge &= 0xff00;
		         for(i=0; i<8; i++)
				 {
		           if(IsDlgButtonChecked(hDlg, (IDC_Status0 + i)))
                      usSEnable |= (0x01 << i);
				 }


            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}
/***************************************************************************
    FUNCTION: ConfigExpStatusProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ConfigExpStatusProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    USHORT   i;

    switch (message)
    {
    case WM_INITDIALOG :
      for(i=0; i<8; i++)
	  {
		 EnableWindow(GetDlgItem(hDlg,(IDC_ExpStatus0 + i)),TRUE);
		 if(usSEnable & (0x01 << (i+8)))
		    CheckDlgButton(hDlg,(IDC_ExpStatus0 + i),TRUE);
		 else
		    CheckDlgButton(hDlg,(IDC_ExpStatus0 + i),FALSE);
	  }
      return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
         {
            case IDOK :
				 usSEnable &= 0x00ff;
		         for(i=0; i<8; i++)
				 {
		           if(IsDlgButtonChecked(hDlg, (IDC_ExpStatus0 + i)))
                      usSEnable |= (0x01 << (i+8));
				 }

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
      break;
    }
    return FALSE ;
}
/***************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
****************************************************************************/
BOOL FTYPE ConfigDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    int         i;
    DWORD       dwIndex;
    char        szBuffer[40];
    int         nOutEntries;

    switch (message)
    {
    case WM_INITDIALOG :

        // --------------------------------
        // Initialize Device List Combobox
        // --------------------------------

        // get number of the installed devices


		if ((ErrCde = DRV_DeviceGetNumOfList((SHORT far *)&gnNumOfDevices)) !=
            SUCCESS)
        {
             DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
             MessageBox(hMainWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
             return TRUE;
        }


        // ----------------------------------------------------------------
        // used for dynamic allocation memory for these installed devices
        //
        // if ((hDeviceList = GlobalAlloc(GHND, gnNumOfDevices *
        //      sizeof(DEVLIST))) == NULL)
        // {
        //    MessageBox(hMainWnd, (LPCSTR)"Not Enough Memory",
        //        "Memory Allocation", MB_OK);
        //    return TRUE;
        // }
        // DeviceList = (LPDEVLIST)GlobalLock(hDeviceList);
        // ----------------------------------------------------------------


        if (gnNumOfDevices > MAX_DEVICES)
            gnNumOfDevices = MAX_DEVICES;

        // retrieve the information of all installed devices

        if ((ErrCde = DRV_DeviceGetList((DEVLIST far *)&DeviceList[0],
            (SHORT)gnNumOfDevices, (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return TRUE;
        }

        // initialize the Device List Combobox with the retrieved information

        for (i = 0; i < gnNumOfDevices; i ++)
            SendDlgItemMessage(hDlg, IDC_DEVICE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)DeviceList[i].szDeviceName));

        SendDlgItemMessage(hDlg, IDC_DEVICE, CB_SETCURSEL,(WPARAM)gwDevice,
                        (LPARAM)0);


        // -----------------------------------------------------------
        // Initialize Module List Combobox for COM port or CAN devices
        // -----------------------------------------------------------

        // check if there is any device attached on this COM port or CAN

        gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
        if (gnNumOfSubdevices > MAX_DEVICES)
            gnNumOfSubdevices = MAX_DEVICES;


        // --------------------------------------------------------------
        // used for dynamic allocation memory for these installed devices
        //
        // if ((hSubDeviceList = GlobalAlloc(GHND, gnNumOfSubdevices *
        //    sizeof(DEVLIST))) == NULL)
        // {
        //    MessageBox(hMainWnd, (LPCSTR)"Not Enough Memory",
        //        "Memory Allocation", MB_OK);
        //    return TRUE;
        // }
        //
        // SubDeviceList = (LPDEVLIST)GlobalLock(hSubDeviceList);
        // --------------------------------------------------------------


        // retrieve the information of all installed devices
        if (gnNumOfSubdevices != 0)
        {
            if ((ErrCde = DRV_DeviceGetSubList(
                (DWORD)DeviceList[gwDevice].dwDeviceNum,
                (DEVLIST far *)&SubDeviceList[0],
                (SHORT)gnNumOfSubdevices,
                (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                return TRUE;
            }


            // initialize the Module List Combobox with the retrieved
            // information

            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));

            for (i = 0; i < gnNumOfSubdevices; i++)
                SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

            SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
					(WPARAM)gwSubDevice, (LPARAM)0);
        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));
        }


        //
        // Initialize Input Channel
        //

        itoa(gwChannel, szBuffer, 10);

        SendDlgItemMessage(hDlg, IDC_ECHANNEL, EM_REPLACESEL, 0,
                        (LPARAM)((LPSTR)szBuffer));

	    if(usIEnable)
		   CheckDlgButton(hDlg,IDC_CGINTENABLE,TRUE);
		else
		   CheckDlgButton(hDlg,IDC_CGINTDISABLE,TRUE);

        if (gnNumOfSubdevices == 0)
        {
            ErrCde = DRV_DeviceOpen(DeviceList[gwDevice].dwDeviceNum,
               (LONG far *)&DriverHandle);
        }
        else
        {
            ErrCde = DRV_DeviceOpen(
                SubDeviceList[gwSubDevice].dwDeviceNum,
                (LONG far *)&DriverHandle);
        }

        if (ErrCde != SUCCESS)
        {
            strcpy(szErrMsg,"Device open error !");
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return 0;
        }

        ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
        ptDevFeatures.size = sizeof(DEVFEATURES);
        if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
            (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            DRV_DeviceClose((LONG far *)&DriverHandle);
            return 0;
        }


        DRV_DeviceClose((LONG far *)&DriverHandle);

        return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
        {
            case IDC_FUNPATTERN :
                 DialogBox (hInst, MAKEINTRESOURCE(IDD_PATTERN),
                            hDlg, lpfnConfigPatternProc) ;
                 return 0;

            case IDC_FUNSTATUS :
                 DialogBox (hInst, MAKEINTRESOURCE(IDD_STATUS),
                            hDlg, lpfnConfigStatusProc) ;
                 return 0;

            case IDOK :

              //
              // get device selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwDevice = (WORD)dwIndex;


              //
              // get sub-device selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_MODULE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwSubDevice = (WORD)dwIndex;


              //
              // get channel number
              //

              if (SendDlgItemMessage(hDlg, IDC_ECHANNEL,
                           EM_GETMODIFY, 0, 0))
              {
                    SendDlgItemMessage(hDlg, IDC_ECHANNEL,
                            WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwChannel = atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_ECHANNEL,
                              EM_SETMODIFY, FALSE, 0) ;
              }

              if(IsDlgButtonChecked(hDlg,IDC_CGINTENABLE))
				  usIEnable = 1;
			  else
				  usIEnable = 0;
			  
			  EndDialog(hDlg, 0);
              return TRUE;

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

            case IDC_DEVICE :

                //
                // When device selection is changed, it needs to
                // re-initialize sub-device combobox and input range
                // combobox.
                //

                if (HIWORD(wParam) == CBN_SELCHANGE)
                {
                    if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                        CB_GETCURSEL, 0, 0)) != CB_ERR)
                         gwDevice = (WORD)dwIndex;
                    else
                        return TRUE;


                    // ------------------------------------------------------
                    // Initialize Module Combobox for COM port or CAN devices
                    // ------------------------------------------------------

                    // check any device attached on this COM port or CAN

                    gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
                    if (gnNumOfSubdevices > MAX_DEVICES)
                        gnNumOfSubdevices = MAX_DEVICES;


                    // retrieve the information of all installed devices

                    if (gnNumOfSubdevices != 0)
                    {
                        if ((ErrCde = DRV_DeviceGetSubList(
                            (DWORD)DeviceList[gwDevice].dwDeviceNum,
                            (DEVLIST far *)&SubDeviceList[0],
                            (SHORT)gnNumOfSubdevices,
                            (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
                        {
                            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                            MessageBox(hMainWnd, (LPCSTR)szErrMsg,
                                "Driver Message", MB_OK);
                            return TRUE;
                        }


                        // initialize the Module List Combobox with the
                        // retrieved information

                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));

                        for (i = 0; i < gnNumOfSubdevices; i++)
                            SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                                (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

                        gwSubDevice = 0;

                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
							(WPARAM)gwSubDevice, (LPARAM)0);
                    }
                    else
                    {
                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                    }
                }

                return TRUE;
        }
        break;
    }

    return FALSE ;
}


/***************************************************************************
    FUNCTION: ScanDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for scan time settings
****************************************************************************/

BOOL FTYPE ScanDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    char        szBuffer[40];

    switch (message)
    {
    case WM_INITDIALOG :

        // ---------------------------------------------------
        // Initialize Scan Time Edit, Maximum accuracy: 40 ms
        // ---------------------------------------------------

        itoa(gwScanTime, szBuffer, 10);
        SendDlgItemMessage(hDlg, IDC_SCAN, EM_REPLACESEL, 0,
                        (LPARAM)((LPSTR)szBuffer));

        return TRUE;

    case WM_COMMAND :

         switch (LOWORD(wParam))
         {
            case IDOK :

                //
                // get scan time
                //

                if (SendDlgItemMessage(hDlg, IDC_SCAN,
                           EM_GETMODIFY, 0, 0))
                {
                    SendDlgItemMessage(hDlg, IDC_SCAN,
                            WM_GETTEXT, 10, (LPARAM)(LPSTR)szBuffer) ;
                    gwScanTime = atoi(szBuffer);
                    SendDlgItemMessage(hDlg, IDC_SCAN,
                              EM_SETMODIFY, FALSE, 0) ;
                }

            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

         }
         break;
    }
    return FALSE ;
}


/***************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
****************************************************************************/

long FTYPE MainWndProc
(
    HWND        hWnd,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    static  HANDLE  hInstance ;
    HMENU           hMenu;
    HDC             hdc;
    char            szBuffer[120];
    RECT            rect;
    DWORD           dwCurrentTime;
    DWORD           dwErrorTime;
    BOOL            bTimerOverrun;
	DWORD           dwExitCode;
    PT_EnableEventEx      Zero = {0};

    switch (message)
    {
        case WM_CREATE:

            //
            // initialize Stop Menu
            //

            hMenu = GetMenu(hWnd);
            EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);

            hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

            lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

            lpfnScanDlgProc = MakeProcInstance (ScanDlgProc, hInstance) ;
            
		    lpfnConfigPatternProc = MakeProcInstance (ConfigPatternProc, hInstance) ;
            
		    lpfnConfigStatusProc = MakeProcInstance (ConfigStatusProc, hInstance) ;

		    lpfnConfigExpPatternProc = MakeProcInstance (ConfigExpPatternProc, hInstance) ;
            
		    lpfnConfigExpStatusProc = MakeProcInstance (ConfigExpStatusProc, hInstance) ;
			return 0 ;

       case WM_COMMAND:     /* message: from application menu */

            hMenu = GetMenu(hWnd);

            switch (LOWORD(wParam))
            {
                case IDM_SETTING :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                         hWnd, lpfnConfigDlgProc) ;
                    return 0;

                case IDM_SCAN :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SCAN),
                         hWnd, lpfnScanDlgProc) ;
                    return 0;

                case IDM_STOP :
					{
                    BOOL bflag = TRUE;
                    // kill timer and close driver
                    //

                    // Close thread
                    m_bContinue = FALSE;
                    Sleep(100);

					if(hThreadHandle)
					{
						GetExitCodeThread(hThreadHandle,&dwExitCode);
						if(dwExitCode==STILL_ACTIVE)
							TerminateThread(hThreadHandle,dwExitCode);
					}


                    KillTimer(hWnd, 1);

					bRun = FALSE;
					if(usPEnable || usSEnable)
					{
					  ptEnableEventEx = Zero;
                      if ((ErrCde = DRV_EnableEventEx(DriverHandle,
                                                (LPT_EnableEventEx)&ptEnableEventEx)) != 0)
					  {
                       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                       DRV_DeviceClose((LONG far *)&DriverHandle);
                       return 0;
					  }
					}
					if(usIEnable)
					{
                       // Disable event feature
                       ptEnableEvent.EventType = ADS_EVT_INTERRUPT;
                       ptEnableEvent.Enabled = FALSE;
                       ptEnableEvent.Count   = 1;
                       if ((ErrCde = DRV_EnableEvent(DriverHandle,
                           (LPT_EnableEvent)&ptEnableEvent)) != 0)
					   {
                         DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                         MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                         DRV_DeviceClose((LONG far *)&DriverHandle);
                         return 0;
					   }
					}
                    
					DRV_DeviceClose((LONG far *)&DriverHandle);

                    //
                    // reset menu item status
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_ENABLED);

                    return 0;
					}

                case IDM_START :

					 usSCount=0;
					 usPCount=0;
					 usICount=0;
                    //
                    // Open Device
                    //

                    if (gnNumOfSubdevices == 0)
                        ErrCde = DRV_DeviceOpen(
                            DeviceList[gwDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);
                    else
                        ErrCde = DRV_DeviceOpen(
                            SubDeviceList[gwSubDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);

                    if (ErrCde != SUCCESS)
                    {
                        strcpy(szErrMsg,"Device open error !");
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                        return 0;
                    }

                    //
                    // enable Stop Menu item and disable the others
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_ENABLED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_GRAYED);

                   // Enable Pattern Match event feature
                    if(usPEnable)
					{
				      ptEnableEventEx.Pattern.usEventType = ADS_EVT_PATTERNMATCH;
                      ptEnableEventEx.Pattern.usEventEnable = TRUE;
                      ptEnableEventEx.Pattern.usCount   = 1;
					  ptEnableEventEx.Pattern.usEnable = usPEnable;
					  ptEnableEventEx.Pattern.usValue = usPValue;
                      if ((ErrCde = DRV_EnableEventEx(DriverHandle,
                                                (LPT_EnableEventEx)&ptEnableEventEx)) != 0)
					  {
                       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                       DRV_DeviceClose((LONG far *)&DriverHandle);
                       return 0;
					  }
					}

                   // Enable Status Change event feature
                    if(usSEnable)
					{
				      ptEnableEventEx.Status.usEventType = ADS_EVT_STATUSCHANGE;
                      ptEnableEventEx.Status.usEventEnable = TRUE;
                      ptEnableEventEx.Status.usCount   = 1;
					  ptEnableEventEx.Status.usEnable = usSEnable;
//					  ptEnableEventEx.Status.usRisingedge = 0;
//					  ptEnableEventEx.Status.usFallingedge = 0;
                      if ((ErrCde = DRV_EnableEventEx(DriverHandle,
                                                (LPT_EnableEventEx)&ptEnableEventEx)) != 0)
					  {
                       DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                       MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                       DRV_DeviceClose((LONG far *)&DriverHandle);
                       return 0;
					  }
					}
                    if(usIEnable)
					{
                       // Enable event feature
                       ptEnableEvent.EventType = ADS_EVT_INTERRUPT;
                       ptEnableEvent.Enabled = TRUE;
                       ptEnableEvent.Count   = 1;
                       if ((ErrCde = DRV_EnableEvent(DriverHandle,
                           (LPT_EnableEvent)&ptEnableEvent)) != 0)
					   {
                         DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                         MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                         DRV_DeviceClose((LONG far *)&DriverHandle);
                         return 0;
					   }
					}
					// create thread
                    hThreadHandle = CreateThread(0, 0, (LPTHREAD_START_ROUTINE) UserThread,
                                                 0, 0, (LPDWORD)&dwThreadID);
                    m_bContinue = TRUE;
					//
                    // Set the new timer type and start it up.
                    //

                    if (SetTimer(hWnd, 1, gwScanTime, NULL) == (UINT)NULL)
                    {
                        MessageBox(hWnd, "Couldn't create timer.", "Notice!",
                            MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    bRun = TRUE;

                    gdwStartTime = GetTickCount();
                    gwOverrunCount = 0;

                    return 0;
            }
            return 0;

        case WM_TIMER:

            //
            // Timer set to dispatch a WM_TIMER message has gone off.
            //

            //
            // check if timer is overrun
            //

            dwCurrentTime = GetTickCount();

            gdwElapseTime = gdwElapseTime + (DWORD)gwScanTime;
            dwErrorTime = dwCurrentTime - gdwElapseTime - gdwStartTime;
            if (dwErrorTime > (DWORD)40)        // maximum 40 ms scan time
            {
                gdwElapseTime = dwCurrentTime - gdwStartTime;
                gwOverrunCount ++;
                if (gwOverrunCount >= 10)
                {
                    bTimerOverrun = TRUE;
                    gwOverrunCount = 0;
                }
                else
                    bTimerOverrun = FALSE;
            }
            else
            {
                bTimerOverrun = FALSE;
                gwOverrunCount = 0;
            }


            //
            // read data
            //

            ptDioReadPortByte.port = gwChannel;
            ptDioReadPortByte.value = (USHORT far *)&gwValue;

            if ((ErrCde = DRV_DioReadPortByte(DriverHandle,
                (LPT_DioReadPortByte)&ptDioReadPortByte)) != 0)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                KillTimer(hWnd, 1);
                MessageBox(hWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
                return TRUE;
            }

            //
            // Display Data
            //

            hdc = GetDC(hWnd);
            GetClientRect(hWnd, &rect);
            InvalidateRect(hWnd, NULL, TRUE);
            UpdateWindow(hWnd);
            if (bTimerOverrun == FALSE)
                sprintf(szBuffer, "Data (Hex) = %2X ", gwValue);
            else
                sprintf(szBuffer, "Timer Overrun!");

            TextOut(hdc, 200, 150, szBuffer, strlen(szBuffer));
            
			sprintf(szBuffer,"Interrupt count=%d", usICount);
            TextOut(hdc, 100, 180, szBuffer, strlen(szBuffer));

			sprintf(szBuffer,"Pattern Match count=%d", usPCount);
            TextOut(hdc, 100, 210, szBuffer, strlen(szBuffer));

		    sprintf(szBuffer,"Status Change  count=%d", usSCount);
            TextOut(hdc, 100, 240, szBuffer, strlen(szBuffer));

            ReleaseDC(hWnd, hdc);

            return 0;

        case WM_DESTROY:

            if (bRun == TRUE)
            {
                KillTimer(hWnd, 1);
                DRV_DeviceClose((LONG far *)&DriverHandle);
            }
            PostQuitMessage(0);
            break;

      default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
  }

   return ((LONG)NULL);
}

//------------------------------------------------------------------
void UserThread()
{
    USHORT usEventType;
    LONG  ErrCde;
    
	while(m_bContinue)
    {
        // Check message
        ptCheckEvent.EventType = &usEventType;
        ptCheckEvent.Milliseconds = 5000;

		bThreadflag = TRUE;
        if ((ErrCde = DRV_CheckEvent(DriverHandle,
            (LPT_CheckEvent)&ptCheckEvent)) != 0)
        {
            MessageBox(hMainWnd,"Check Event Error !","Thread Message",MB_OK);
            return ;
        }

        // Process counter match event
        if (usEventType == ADS_EVT_INTERRUPT)
            adInterruptEvent();

		// Process pattern match event
        if (usEventType == ADS_EVT_PATTERNMATCH)
            adPatternMatchEvent();

        // Process status change event
        if (usEventType == ADS_EVT_STATUSCHANGE)
            adStatusChangeEvent();

		bThreadflag = FALSE;
    }

    return ;
}

//------------------------------------------------------------------
void adPatternMatchEvent()
{
    if(usPCount>10000)
       usPCount = 0;
	else
       usPCount++;
    return;
}

//------------------------------------------------------------------
void adInterruptEvent()
{
    if(usICount>10000)
       usICount = 0;
	else
       usICount++;
    return;
}
//------------------------------------------------------------------
void adStatusChangeEvent()
{
    if(usSCount>10000)
       usSCount = 0;
	else
       usSCount++;
    return;
}
