/*
 ***********************************************************************
 *  Program     : PULSE.C                                              *
 *  Description : Demo program for pulse output                        *
 *  Revision    : 1.00                                                 *
 *  Date        : 9/5/1996                     Advantech Co., Ltd.     *
 ***********************************************************************
 *
 * FUNCTIONS:
 *
 *   InitApplication() - Initializes window data and registers window class
 *
 *   InitInstance() - Saves instance handle and creates main window
 *
 *   MainWndProc() - Process main window messages
 *
 *   ConfigDlgProc() - Process the dialog box messages for configuration
 */


#include <windows.h>
#include <stdlib.h>
#include <dos.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "..\..\..\..\include\driver.h"
#include "..\..\..\..\include\os.h"
#include "resource.h"

// ------------------------------------------------------------------------
// CONSTANT DEFINITION
// ------------------------------------------------------------------------
#define     MAX_DEVICES     100


// ------------------------------------------------------------------------
// GLOBAL VARIABLES
// ------------------------------------------------------------------------
HANDLE      hInst;                      // current instance
HWND        hMainWnd;                   // main window handle


static      DEVFEATURES     DevFeatures;    // structure for device features
//static      PT_CounterPulseStart ptCounterPulseStart;
static      PT_DeviceGetFeatures  ptDevFeatures;
static      PT_CounterPWMSetting ptCounterPWMSetting;

char        szErrMsg[80];               // Use for MESSAGEBOX function
LRESULT     ErrCde;                     // Return error code

//
// used for dynamic allocation memory for the installed devices
//
// static  HGLOBAL     hDeviceList,hSubDeviceList;
// static  LPDEVLIST   DeviceList,SubDeviceList;
//

//
// used for fixed memory for the installed devices
//
DEVLIST     DeviceList[MAX_DEVICES];
DEVLIST     SubDeviceList[MAX_DEVICES];

LONG        DriverHandle = (LONG)NULL;          // driver handle
BOOL        bRun = FALSE;                       // flag for running
USHORT      gwDevice = 0, gwSubDevice = 0;      // device index
USHORT      gwChannel = 0;                      // input channel

USHORT      gwGateMode[2]  = {0,0};                    // gate mode
FLOAT       gfGatePeriod[2]={0.0,0.0} ;                // gate period
FLOAT       gfUpCycle[2]={0.0,0.0} ;                   // UpCycle
ULONG       gdwReading = 0;                     // read counter data
USHORT      gusOutCount[2] = {0,0};                    // PWM pulse out count
SHORT       gnNumOfDevices, gnNumOfSubdevices;  // number of installed devices
FARPROC     lpfnConfigDlgProc;                  // config. dialog procedure


// ------------------------------------------------------------------------
// FUNCTION DECLARATION
// ------------------------------------------------------------------------
BOOL InitApplication(HANDLE);
BOOL InitInstance(HANDLE, int);
long FTYPE MainWndProc(HWND, UINT, WPARAM, LPARAM);
int  FTYPE WinMain(HANDLE, HANDLE, LPSTR, int);


/***************************************************************************
    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop
****************************************************************************/

int FTYPE WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;                          /* current instance             */
HANDLE hPrevInstance;                      /* previous instance            */
LPSTR lpCmdLine;                           /* command line                 */
int nCmdShow;                              /* show-window type (open/icon) */
{
    MSG msg;                               /* message                      */

    if (!hPrevInstance)                /* Other instances of app running?  */
      if (!InitApplication(hInstance)) /* Initialize shared things         */
          return (FALSE);              /* Exits if unable to initialize    */

    // Perform initializations that apply to a specific instance
    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    // Acquire and dispatch messages until a WM_QUIT message is received.
    while (GetMessage(&msg,      /* message structure                      */
        (HWND)NULL,              /* handle of window receiving the message */
        (UINT)NULL,              /* lowest message to examine              */
        (UINT)NULL))             /* highest message to examine             */
    {
        TranslateMessage(&msg);  /* Translates virtual key codes           */
        DispatchMessage(&msg);   /* Dispatches message to window           */
    }
    return (msg.wParam);         /* Returns the value from PostQuitMessage */
}

/****************************************************************************
    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class
****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                            /* current instance           */
{
    WNDCLASS  wc;

    // Fill in window class structure with parameters that describe the
    // main window.
    wc.style = CS_HREDRAW | CS_VREDRAW; /* Class style(s).                 */
    wc.lpfnWndProc = (WNDPROC)MainWndProc;   /* window process procedure   */
    wc.cbClsExtra = 0;                  /* No per-class extra data.        */
    wc.cbWndExtra = 0;                  /* No per-window extra data.       */
    wc.hInstance = hInstance;           /* Application that owns the class.*/
    wc.hIcon = LoadIcon(hInstance, "IDI_ICON1");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  "MyMenu";   /* Name of menu resource in .RC file.  */
    wc.lpszClassName = "MyClass";  /* Name used in call to CreateWindow. */

    // Register the window class and return success/failure code.
    return (RegisterClass(&wc));

}

/***************************************************************************
    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window
****************************************************************************/

BOOL InitInstance
(
    HANDLE          hInstance,          // Current instance identifier.
    int             nCmdShow            // Param for first ShowWindow() call.
)
{
    // Save the instance handle in static variable, which will be used in
    // many subsequence calls from this application to Windows.
    hInst = hInstance;

    // Create a main window for this application instance.
    hMainWnd = CreateWindow(
        "MyClass",                 /* See RegisterClass() call.           */
        "Advantech Driver Demo : PWM Pulse Output" , /* Window title bar  */
        WS_OVERLAPPEDWINDOW,        /* Window style.                       */
        CW_USEDEFAULT,              /* Default horizontal position.        */
        CW_USEDEFAULT,              /* Default vertical position.          */
        CW_USEDEFAULT,              /* Default width.                      */
        CW_USEDEFAULT,              /* Default height.                     */
        NULL,                       /* Overlapped windows have no parent.  */
        NULL,                       /* Use the window class menu.          */
        hInstance,                  /* This instance owns this window.     */
        NULL                        /* Pointer not needed.                 */
        );

    // If window could not be created, return "failure"
    if (!hMainWnd)
        return (FALSE);

    // Make the window visible; update its client area; and return "success"

    ShowWindow(hMainWnd, nCmdShow);   // Show the window
    UpdateWindow(hMainWnd);           // Sends WM_PAINT message
    return (TRUE);                // Returns the value from PostQuitMessage
}

/***************************************************************************
    FUNCTION: ConfigDlgProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes dialog box messages for configuration
****************************************************************************/

BOOL FTYPE ConfigDlgProc
(
    HWND        hDlg,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    int         i;
    DWORD       dwIndex;
    char        szBuffer[40];
    int         nOutEntries;
    char        szTmp[20];

    switch (message)
    {
    case WM_INITDIALOG :

        // --------------------------------
        // Initialize Device List Combobox
        // --------------------------------

        // get number of the installed devices

        if ((ErrCde = DRV_DeviceGetNumOfList((SHORT far *)&gnNumOfDevices)) !=
            SUCCESS)
        {
             DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
             MessageBox(hMainWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
             return TRUE;
        }


        // ----------------------------------------------------------------
        // used for dynamic allocation memory for these installed devices
        //
        // if ((hDeviceList = GlobalAlloc(GHND, gnNumOfDevices *
        //      sizeof(DEVLIST))) == NULL)
        // {
        //    MessageBox(hMainWnd, (LPCSTR)"Not Enough Memory",
        //        "Memory Allocation", MB_OK);
        //    return TRUE;
        // }
        // DeviceList = (LPDEVLIST)GlobalLock(hDeviceList);
        // ----------------------------------------------------------------


        if (gnNumOfDevices > MAX_DEVICES)
            gnNumOfDevices = MAX_DEVICES;

        // retrieve the information of all installed devices

        if ((ErrCde = DRV_DeviceGetList((DEVLIST far *)&DeviceList[0],
            (SHORT)gnNumOfDevices, (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
        {
            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
            MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
            return TRUE;
        }

        // initialize the Device List Combobox with the retrieved information

        for (i = 0; i < gnNumOfDevices; i ++)
            SendDlgItemMessage(hDlg, IDC_DEVICE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)DeviceList[i].szDeviceName));

        SendDlgItemMessage(hDlg, IDC_DEVICE, CB_SETCURSEL,(WPARAM)gwDevice,
                        (LPARAM)0);


        // -----------------------------------------------------------
        // Initialize Module List Combobox for COM port or CAN devices
        // -----------------------------------------------------------

        // check if there is any device attached on this COM port or CAN

        gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
        if (gnNumOfSubdevices > MAX_DEVICES)
            gnNumOfSubdevices = MAX_DEVICES;


        // --------------------------------------------------------------
        // used for dynamic allocation memory for these installed devices
        //
        // if ((hSubDeviceList = GlobalAlloc(GHND, gnNumOfSubdevices *
        //    sizeof(DEVLIST))) == NULL)
        // {
        //    MessageBox(hMainWnd, (LPCSTR)"Not Enough Memory",
        //        "Memory Allocation", MB_OK);
        //    return TRUE;
        // }
        //
        // SubDeviceList = (LPDEVLIST)GlobalLock(hSubDeviceList);
        // --------------------------------------------------------------


        // retrieve the information of all installed devices
        if (gnNumOfSubdevices != 0)
        {
            if ((ErrCde = DRV_DeviceGetSubList(
                (DWORD)DeviceList[gwDevice].dwDeviceNum,
                (DEVLIST far *)&SubDeviceList[0],
                (SHORT)gnNumOfSubdevices,
                (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
            {
                DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                MessageBox(hMainWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                return TRUE;
            }


            // initialize the Module List Combobox with the retrieved
            // information

            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));

            for (i = 0; i < gnNumOfSubdevices; i++)
                SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                        (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

            SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
					(WPARAM)gwSubDevice, (LPARAM)0);
        }
        else
        {
            EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
            SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT, 0,
                    (LPARAM)((LPSTR)0));
        }


        //
        // Initialize Input Channel Edit
        //
		
        itoa(gwChannel, szBuffer, 10);
        SendDlgItemMessage(hDlg, IDC_ECHANNEL, EM_REPLACESEL, 0,
                        (LPARAM)((LPSTR)szBuffer));

        // Initialize Pulse Out count

        // J.L mod
		//if(gwChannel & 0x01)
		//if(!(gwChannel )) //commented by yingsong.huang at 08/29/02
		if(gwChannel & 0x01)
		{
           CheckDlgButton(hDlg, IDC_PWMEnableCh0, 1);
           EnableWindow(GetDlgItem(hDlg, IDC_GATEPERIOD0),TRUE);
           EnableWindow(GetDlgItem(hDlg, IDC_UPCYCLE0),TRUE);
           EnableWindow(GetDlgItem(hDlg, IDC_OutCountEnable0),TRUE);
		   sprintf(szBuffer,"%5.1f",gfGatePeriod[0]);
           SendDlgItemMessage(hDlg, IDC_GATEPERIOD0, EM_REPLACESEL, 0,
                              (LPARAM)((LPSTR)szBuffer));
		   sprintf(szBuffer,"%5.1f",gfUpCycle[0]);
           SendDlgItemMessage(hDlg, IDC_UPCYCLE0, EM_REPLACESEL, 0,
                              (LPARAM)((LPSTR)szBuffer));
		   if(gusOutCount[0])
		   {
             CheckDlgButton(hDlg, IDC_OutCountEnable0, 1);
             EnableWindow(GetDlgItem(hDlg, IDC_ED_OutCount0),TRUE);
		     sprintf(szBuffer,"%d",gusOutCount[0]);
             SendDlgItemMessage(hDlg, IDC_ED_OutCount0, EM_REPLACESEL, 0,
                              (LPARAM)((LPSTR)szBuffer));
		   }
		   else
		   {
             CheckDlgButton(hDlg, IDC_OutCountEnable0, 0);
             EnableWindow(GetDlgItem(hDlg, IDC_ED_OutCount0),FALSE);
		   }
		}
		else
		{
           CheckDlgButton(hDlg, IDC_PWMEnableCh0, 0);
           EnableWindow(GetDlgItem(hDlg, IDC_GATEPERIOD0),FALSE);
           EnableWindow(GetDlgItem(hDlg, IDC_UPCYCLE0),FALSE);
           EnableWindow(GetDlgItem(hDlg, IDC_OutCountEnable0),FALSE);
           EnableWindow(GetDlgItem(hDlg, IDC_ED_OutCount0),FALSE);
		}
  
        // J.L mod
		//if(gwChannel & 0x02)
		//if(gwChannel & 0x01)     //commented by yingsong.huang
		if(gwChannel & 0x02)
		{
           CheckDlgButton(hDlg, IDC_PWMEnableCh1, 1);
           EnableWindow(GetDlgItem(hDlg, IDC_GATEPERIOD1),TRUE);
           EnableWindow(GetDlgItem(hDlg, IDC_UPCYCLE1),TRUE);
           EnableWindow(GetDlgItem(hDlg, IDC_OutCountEnable1),TRUE);
		   sprintf(szBuffer,"%5.1f",gfGatePeriod[1]);
           SendDlgItemMessage(hDlg, IDC_GATEPERIOD1, EM_REPLACESEL, 0,
                              (LPARAM)((LPSTR)szBuffer));
		   sprintf(szBuffer,"%5.1f",gfUpCycle[1]);
           SendDlgItemMessage(hDlg, IDC_UPCYCLE1, EM_REPLACESEL, 0,
                              (LPARAM)((LPSTR)szBuffer));
		   if(gusOutCount[1])
		   {
             CheckDlgButton(hDlg, IDC_OutCountEnable1, 1);
             EnableWindow(GetDlgItem(hDlg, IDC_ED_OutCount1),TRUE);
		     sprintf(szBuffer,"%d",gusOutCount[1]);
             SendDlgItemMessage(hDlg, IDC_ED_OutCount1, EM_REPLACESEL, 0,
                              (LPARAM)((LPSTR)szBuffer));
		   }
		   else
		   { 
             CheckDlgButton(hDlg, IDC_OutCountEnable1, 0);
             EnableWindow(GetDlgItem(hDlg, IDC_ED_OutCount1),FALSE);
		   }
		}
		else
		{
           CheckDlgButton(hDlg, IDC_PWMEnableCh1, 0);
           EnableWindow(GetDlgItem(hDlg, IDC_GATEPERIOD1),FALSE);
           EnableWindow(GetDlgItem(hDlg, IDC_UPCYCLE1),FALSE);
           EnableWindow(GetDlgItem(hDlg, IDC_OutCountEnable1),FALSE);
           EnableWindow(GetDlgItem(hDlg, IDC_ED_OutCount1),FALSE);
		}

        return TRUE;

    case WM_COMMAND :
         switch (LOWORD(wParam))
         {
            case IDOK :

              //
              // get device selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwDevice = (WORD)dwIndex;


              //
              // get sub-device selection
              //

              if ((dwIndex = SendDlgItemMessage(hDlg, IDC_MODULE,
                           CB_GETCURSEL, 0, 0)) != CB_ERR)
                   gwSubDevice = (WORD)dwIndex;


               //
               // get gate period
               //
               if(IsDlgButtonChecked(hDlg, IDC_PWMEnableCh0))
			   {
                // J.L mod
				//gwChannel = 0x01;
                //gwChannel = 0x0;   //commented by yingsong.huang at 08/29/02
                gwChannel |= 0x01;   //add by yingsong.huang at 08/29/02
				SendDlgItemMessage(hDlg, IDC_GATEPERIOD0, WM_GETTEXT, 13,
                                   (LPARAM)(LPSTR)szBuffer) ;
                gfGatePeriod[0] = (float)atof(szBuffer);

                SendDlgItemMessage(hDlg, IDC_UPCYCLE0, WM_GETTEXT, 13,
                                    (LPARAM)(LPSTR)szBuffer) ;
                gfUpCycle[0] = (float)atof(szBuffer);

			   //Get Pulse Out Count
			    if(IsDlgButtonChecked(hDlg, IDC_OutCountEnable0))
				{
				   SendDlgItemMessage( hDlg, IDC_ED_OutCount0, EM_GETLINE, 0, (LPARAM)szTmp);
                   gusOutCount[0] = LOWORD( atol(szTmp));
				}
			    else
				   gusOutCount[0] = 0;   //0 means PWM pulse out not stop
			   }
			   else
			   {
                //J.L mod
				//gwChannel &= 0x02;
                //gwChannel &= 0x01;  //commented by yingsong.huang at 08/29/02
				gwChannel &= 0xfe;    //added by yingsong.huang
				gfGatePeriod[0] = 0.0;
                
				gfUpCycle[0] = 0.0;
			   }

               if(IsDlgButtonChecked(hDlg, IDC_PWMEnableCh1))
			   {
                // J.L mod				   
				//gwChannel |= 0x02;
                //gwChannel |= 0x01; //commented by yingsong.huang at 08/29/02
				   gwChannel |=0x02; //added by yingsong.huang
                
				SendDlgItemMessage(hDlg, IDC_GATEPERIOD1, WM_GETTEXT, 13,
                                   (LPARAM)(LPSTR)szBuffer) ;
                gfGatePeriod[1] = (float)atof(szBuffer);

                SendDlgItemMessage(hDlg, IDC_UPCYCLE1, WM_GETTEXT, 13,
                                    (LPARAM)(LPSTR)szBuffer) ;
                gfUpCycle[1] = (float)atof(szBuffer);

			   //Get Pulse Out Count
			    if(IsDlgButtonChecked(hDlg, IDC_OutCountEnable1))
				{
				   SendDlgItemMessage( hDlg, IDC_ED_OutCount1, EM_GETLINE, 0, (LPARAM)szTmp);
                   gusOutCount[1] = LOWORD( atol(szTmp));
				}
			    else
				   gusOutCount[1] = 0;   //0 means PWM pulse out not stop
			   }
			   else
			   {
                //gwChannel &= 0x01; 
				//gwChannel &= 0x0;     // J.L mod  //commented by yingsong.huang
				   gwChannel &= 0xfd;
                gfGatePeriod[1] = 0.0;
                gfUpCycle[1] = 0.0;
			   }


               EndDialog(hDlg, 0);
               return TRUE;


            case IDCANCEL :
                EndDialog(hDlg, 0);
                return TRUE;

            case IDC_OutCountEnable0 :
				if(IsDlgButtonChecked(hDlg,IDC_OutCountEnable0))
				{  EnableWindow(GetDlgItem(hDlg,IDC_ED_OutCount0),TRUE);
				   SetFocus(GetDlgItem(hDlg,IDC_ED_OutCount0));
				}
				else
					EnableWindow(GetDlgItem(hDlg,IDC_ED_OutCount0),FALSE);

            case IDC_OutCountEnable1 :
				if(IsDlgButtonChecked(hDlg,IDC_OutCountEnable1))
				{  EnableWindow(GetDlgItem(hDlg,IDC_ED_OutCount1),TRUE);
				   SetFocus(GetDlgItem(hDlg,IDC_ED_OutCount1));}
				else
					EnableWindow(GetDlgItem(hDlg,IDC_ED_OutCount1),FALSE);

            case IDC_PWMEnableCh0 :
				if(IsDlgButtonChecked(hDlg,IDC_PWMEnableCh0))
				{  EnableWindow(GetDlgItem(hDlg,IDC_GATEPERIOD0),TRUE);
                   EnableWindow(GetDlgItem(hDlg,IDC_UPCYCLE0),TRUE);
                   EnableWindow(GetDlgItem(hDlg, IDC_OutCountEnable0),TRUE);
				   SetFocus(GetDlgItem(hDlg,IDC_GATEPERIOD0));}
				else
				{
					EnableWindow(GetDlgItem(hDlg,IDC_GATEPERIOD0),FALSE);
					EnableWindow(GetDlgItem(hDlg,IDC_UPCYCLE0),FALSE);
                    EnableWindow(GetDlgItem(hDlg, IDC_OutCountEnable0),FALSE);
                    EnableWindow(GetDlgItem(hDlg, IDC_ED_OutCount0),FALSE);
				}

            case IDC_PWMEnableCh1 :
				if(IsDlgButtonChecked(hDlg,IDC_PWMEnableCh1))
				{  EnableWindow(GetDlgItem(hDlg,IDC_GATEPERIOD1),TRUE);
				   EnableWindow(GetDlgItem(hDlg,IDC_UPCYCLE1),TRUE);
                   EnableWindow(GetDlgItem(hDlg, IDC_OutCountEnable1),TRUE);
				   SetFocus(GetDlgItem(hDlg,IDC_GATEPERIOD1));}
				else
				{
					EnableWindow(GetDlgItem(hDlg,IDC_GATEPERIOD1),FALSE);
					EnableWindow(GetDlgItem(hDlg,IDC_UPCYCLE1),FALSE);
                    EnableWindow(GetDlgItem(hDlg, IDC_OutCountEnable1),FALSE);
                    EnableWindow(GetDlgItem(hDlg, IDC_ED_OutCount1),FALSE);
				}

            case IDC_DEVICE :

                //
                // When device selection is changed, it needs to
                // re-initialize sub-device combobox and input range
                // combobox.
                //

                if (HIWORD(wParam) == CBN_SELCHANGE)
                {
                    if ((dwIndex = SendDlgItemMessage(hDlg, IDC_DEVICE,
                        CB_GETCURSEL, 0, 0)) != CB_ERR)
                         gwDevice = (WORD)dwIndex;
                    else
                        return TRUE;


                    // ------------------------------------------------------
                    // Initialize Module Combobox for COM port or CAN devices
                    // ------------------------------------------------------

                    // check any device attached on this COM port or CAN

                    gnNumOfSubdevices = DeviceList[gwDevice].nNumOfSubdevices;
                    if (gnNumOfSubdevices > MAX_DEVICES)
                        gnNumOfSubdevices = MAX_DEVICES;


                    // retrieve the information of all installed devices

                    if (gnNumOfSubdevices != 0)
                    {
                        if ((ErrCde = DRV_DeviceGetSubList(
                            (DWORD)DeviceList[gwDevice].dwDeviceNum,
                            (DEVLIST far *)&SubDeviceList[0],
                            (SHORT)gnNumOfSubdevices,
                            (SHORT far *)&nOutEntries)) != (LONG)SUCCESS)
                        {
                            DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                            MessageBox(hMainWnd, (LPCSTR)szErrMsg,
                                "Driver Message", MB_OK);
                            return TRUE;
                        }


                        // initialize the Module List Combobox with the
                        // retrieved information

                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), TRUE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));

                        for (i = 0; i < gnNumOfSubdevices; i++)
                            SendDlgItemMessage(hDlg, IDC_MODULE, CB_ADDSTRING, 0,
                                (LPARAM)((LPSTR)SubDeviceList[i].szDeviceName));

                        gwSubDevice = 0;

                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_SETCURSEL,
							(WPARAM)gwSubDevice, (LPARAM)0);
                    }
                    else
                    {
                        EnableWindow(GetDlgItem(hDlg, IDC_MODULE), FALSE);
                        SendDlgItemMessage(hDlg, IDC_MODULE, CB_RESETCONTENT,
                            0, (LPARAM)((LPSTR)0));
                    }
                }

                return TRUE;
         }

         break;
    }

    return FALSE ;
}


/***************************************************************************
    FUNCTION: MainWndProc(HWND, unsigned, WPARAM, LPARAM)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_CREATE     - create window
        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window
****************************************************************************/

long FTYPE MainWndProc
(
    HWND        hWnd,                          // window handle
    unsigned    message,                       // type of message
    WPARAM      wParam,                        // additional information
    LPARAM      lParam                         // additional information
)
{
    static  HANDLE  hInstance ;
    HMENU           hMenu;
    HDC             hdc;
    char            szBuffer[120];
    RECT            rect;

    switch (message)
    {
        case WM_CREATE:

            //
            // initialize Stop Menu
            //

            hMenu = GetMenu(hWnd);
            EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);

            hInstance = ((LPCREATESTRUCT) lParam)->hInstance ;

            lpfnConfigDlgProc = MakeProcInstance (ConfigDlgProc, hInstance) ;

            return 0 ;

        case WM_COMMAND:     /* message: from application menu */

            hMenu = GetMenu(hWnd);

            switch (LOWORD(wParam))
            {
                case IDM_SETTING :
                    DialogBox (hInstance, MAKEINTRESOURCE(IDD_SETTING),
                         hWnd, lpfnConfigDlgProc) ;
                    return 0;

                case IDM_STOP :

                    //
                    // counter reset
                    //

                    if ((ErrCde = DRV_CounterReset(DriverHandle,
                        (LPARAM)gwChannel)) != 0)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
                    }

                    DRV_DeviceClose((LONG far *)&DriverHandle);

                    bRun = FALSE;

                    //
                    // reset menu item status
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_GRAYED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_ENABLED);

                    return 0;

                case IDM_START :

                    //
                    // Open Device
                    //

                    if (gnNumOfSubdevices == 0)
                        ErrCde = DRV_DeviceOpen(
                            DeviceList[gwDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);
                    else
                        ErrCde = DRV_DeviceOpen(
                            SubDeviceList[gwSubDevice].dwDeviceNum,
                                (LONG far *)&DriverHandle);

                    if (ErrCde != SUCCESS)
                    {
                        strcpy(szErrMsg,"Device open error !");
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Device Open",MB_OK);
                        return 0;
                    }


                    //
                    // get number of counter channels
                    //

                    ptDevFeatures.buffer = (LPDEVFEATURES)&DevFeatures;
                    ptDevFeatures.size = sizeof(DEVFEATURES);
                    if ((ErrCde = DRV_DeviceGetFeatures(DriverHandle,
                        (LPT_DeviceGetFeatures)&ptDevFeatures)) != SUCCESS)
                    {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg,"Driver Message",MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }

                    // check number of counter channels
                    if (DevFeatures.usMaxTimerChl == 0)
                    {
                        MessageBox(hWnd,(LPCSTR)"No Counter Channel",
                            "Driver Message",MB_OK);
                        DRV_DeviceClose((LONG far *)&DriverHandle);
                        return 0;
                    }


                    //
                    // start counter
                    //

                    // J.L mod
					//if((gwChannel & 0x01)==0x01)
					//if(!(gwChannel )) //commented by yingsong.huang
					if(gwChannel&0x01)  //added by yingsong.huang
					{
                     ptCounterPWMSetting.Port  = 0;
                     ptCounterPWMSetting.Period   = gfGatePeriod[0];
                     ptCounterPWMSetting.HiPeriod  = gfUpCycle[0];
					 ptCounterPWMSetting.OutCount = gusOutCount[0]; 
                     ptCounterPWMSetting.GateMode = gwGateMode[0];

                     if((ErrCde = DRV_CounterPWMSetting(DriverHandle,
                          (LPT_CounterPWMSetting)&ptCounterPWMSetting)) != 0)
                     {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
                        return TRUE;
                     }
					}

					//if((gwChannel & 0x02)==0x02)
				    //if((gwChannel & 0x01)==0x01)   // J.L mod //commented by yingsong
					if(gwChannel & 0x02)
					{
                     ptCounterPWMSetting.Port  = 1;
                     ptCounterPWMSetting.Period   = gfGatePeriod[1];
                     ptCounterPWMSetting.HiPeriod  = gfUpCycle[1];
					 ptCounterPWMSetting.OutCount = gusOutCount[1]; 
                     ptCounterPWMSetting.GateMode = gwGateMode[1];

                     if((ErrCde = DRV_CounterPWMSetting(DriverHandle,
                          (LPT_CounterPWMSetting)&ptCounterPWMSetting)) != 0)
                     {
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
                        return TRUE;
                     }
					}
                    
					// The original gwChannel should be convert to the suitable 
					//format of DRV_CounterPWMEnable function, If bit0=1,port0 
					//is enabled.If bit1=1,port1 is enabled. 
					// added by Yingsong.Huang at 08/25/02
                    //gwChanIn = 1 << gwChannel;  //commented again by yingsong.huang at 08/29/02
					                              //because we should provided independent PWM output
					                              // for 2 port
                    // add end--yingsong, huang

					if( (ErrCde = DRV_CounterPWMEnable(DriverHandle,
                         (LPARAM)gwChannel)) != 0) 
					{
                        DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                        MessageBox(hWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
					}

                    //
                    // enable Stop Menu item and disable the others
                    //

                    EnableMenuItem(hMenu, IDM_STOP, MF_BYCOMMAND | MF_ENABLED);
                    EnableMenuItem(hMenu, IDM_START, MF_BYCOMMAND | MF_GRAYED);

                    bRun = TRUE;

                    //
                    // display
                    //

                    hdc = GetDC(hWnd);
                    GetClientRect(hWnd, &rect);
                    sprintf(szBuffer, "Pulse Output Running ...... ");
                    DrawText(hdc, szBuffer, -1, &rect,
                        DT_SINGLELINE | DT_CENTER | DT_VCENTER);
                    ReleaseDC(hWnd, hdc);

                    return 0;
            }
            return 0;

        case WM_DESTROY:
            if (bRun == TRUE)
            {
                // counter reset
                if( (ErrCde = DRV_CounterReset(DriverHandle,
                    (LPARAM)gwChannel)) != 0)
                {
                    DRV_GetErrorMessage(ErrCde,(LPSTR)szErrMsg);
                    MessageBox(hWnd,(LPCSTR)szErrMsg, "Driver Message", MB_OK);
                }

                DRV_DeviceClose((LONG far *)&DriverHandle);
            }
            PostQuitMessage(0);
            break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }

    return ((LONG)NULL);
}

